/*
 * SalomeTMF is a Test Management Framework
 * Copyright (C) 2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * @author Aurore PENAULT
 *
 * Contact: mikael.marche@rd.francetelecom.com
 */

package salomeTMF_plug.cronExec;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.Timer;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import org.objectweb.salome_tmf.api.Api;
import org.objectweb.salome_tmf.api.data.ExecutionWrapper;
import org.objectweb.salome_tmf.data.Campaign;
import org.objectweb.salome_tmf.data.DataConstants;
import org.objectweb.salome_tmf.data.Execution;
import org.objectweb.salome_tmf.data.Family;
import org.objectweb.salome_tmf.data.Test;
import org.objectweb.salome_tmf.data.TestList;
import org.objectweb.salome_tmf.ihm.main.SalomeTMFContext;
import org.objectweb.salome_tmf.ihm.main.SalomeTMFPanels;
import org.objectweb.salome_tmf.ihm.main.datawrapper.DataModel;
import org.objectweb.salome_tmf.ihm.models.TestTreeModel;
import org.objectweb.salome_tmf.ihm.models.TreeRenderer;

import salomeTMF_plug.cronExec.languages.Language;

import com.toedter.calendar.JDateChooser;

/**
 * Classe qui construit la fenetre permettant de choisir les tests a inserer 
 * dans la documentation
 * @author vapu8214
 */
public class ExecChooser extends JDialog implements DataConstants {
	/**
	 * Modele de l'arbre des tests
	 */
	protected TestTreeModel campTreeModel;

	/**
	 * Modele de l'arbre des tests selectionnes
	 */
	protected TestTreeModel chosenTreeModel;

	/**
	 * l'arbre des tests choisis
	 */
	JTree chosenTree;

	/**
	 * l'arbre des tests
	 */
	JTree testTree;

	/**
	 * Noeud selectionne dans l'arbre des tests choisis
	 */
	DefaultMutableTreeNode chosenSelectedNode;

	/**
	 * Noeud selectionne dans l'arbre des tests
	 */
	DefaultMutableTreeNode testSelectedNode;

	/**
	 * Racine de l'arbre temporaire des tests choisis
	 */
	DefaultMutableTreeNode temporaryChosenRootNode;

	/**
	 * Racine de l'arbre final des tests choisis
	 */
	DefaultMutableTreeNode realChosenRoot;

	/**
	 * Liste des campagnes selectionnees
	 */
	ArrayList temporaryCampList;

	/**
	 * Liste des executions selectionnees
	 */
	ArrayList temporaryExecList;

	/**
	 * Liste des tests selectionnes
	 */
	ArrayList temporaryTestList;

	/**
	 * Liste des suites selectionnees
	 */
	ArrayList temporaryTestListList;

	/**
	 * Liste des familles selectionnees
	 */
	ArrayList temporaryFamilyList;

	ArrayList testSelectedNodes;

	ArrayList chosenSelectedNodes;
	
	JDateChooser date;
	JTextField hour;
	JTextField minutes;
	JTextField occurences;
	JTextField interval;
	JCheckBox stopOnError;
	
	CronExecPlugin cronPlg;
	
	Timer pTimer;

	/**
	 * Constructeur de la fenetre pour l'ajout de tests a la documentation.
	 * @param chosenRoot le noeud racine de la liste des tests selectionnes
	 * @param model le modele de donnees des tests
	 * @param testRoot la racine de l'arbre des tests 
	 */
	public ExecChooser(CronExecPlugin cron_plg, Timer timer) {
		
		super(SalomeTMFContext.getInstance().getSalomeFrame(), true);
		cronPlg = cron_plg;
		pTimer = timer;
		
		DefaultMutableTreeNode chosenRoot = new DefaultMutableTreeNode(Language.getInstance().getText("Selection"));
		
		temporaryCampList = new ArrayList();

		chosenSelectedNodes = new ArrayList();
		testSelectedNodes = new ArrayList();

		realChosenRoot = new DefaultMutableTreeNode(Language.getInstance().getText("Selection"));
		
		TreeRenderer chosenRenderer = new TreeRenderer();
		TreeRenderer testRenderer = new TreeRenderer();

		temporaryChosenRootNode = new DefaultMutableTreeNode(chosenRoot
				.getUserObject());

		chosenSelectedNode = temporaryChosenRootNode;

		chosenTree = new JTree();
		chosenTreeModel = new TestTreeModel(temporaryChosenRootNode, chosenTree, null);
		chosenTree.setModel(chosenTreeModel);

		initTemporaryTree(chosenRoot);
		
		DefaultMutableTreeNode testRoot = SalomeTMFPanels.getCampaignDynamicTree().getRoot();
		DefaultMutableTreeNode testRootClone = (DefaultMutableTreeNode)testRoot.clone();
	    for (int j = 0; j < testRoot.getChildCount(); j++) {
	        DefaultMutableTreeNode childNode = (DefaultMutableTreeNode)testRoot.getChildAt(j);
	        Campaign campaign = (Campaign)childNode.getUserObject();
	        addObject(testRootClone, campaign, true);
	    }
	    testTree = new JTree();
	    campTreeModel = new TestTreeModel(testRootClone, testTree, null);
		testTree.setModel(campTreeModel);
		
		testTree.setCellRenderer(testRenderer);
		testTree.addTreeSelectionListener(new TreeSelectionListener() {
			public void valueChanged(TreeSelectionEvent e) {
                                testSelectedNodes.clear();
                                TreePath[] pathTab = testTree.getSelectionPaths();
                                if (pathTab != null) {
                                    for (int i = 0; i < pathTab.length; i++) {
                                        testSelectedNodes.add(pathTab[i].getLastPathComponent());
                                    }
                                }
                        }
		});
		
		chosenTree.setCellRenderer(chosenRenderer);
		chosenTree.addTreeSelectionListener(new TreeSelectionListener() {
			public void valueChanged(TreeSelectionEvent e) {
				chosenSelectedNodes.clear();
				TreePath[] pathTab = chosenTree.getSelectionPaths();
				if (pathTab != null) {
					for (int i = 0; i < pathTab.length; i++) {
						chosenSelectedNodes.add(pathTab[i]
								.getLastPathComponent());
					}
				}
			}
		});

		
		// Executions de chaque campagne
		DefaultMutableTreeNode root = (DefaultMutableTreeNode) campTreeModel
				.getRoot();
		for (int j = 0; j < root.getChildCount(); j++) {
			DefaultMutableTreeNode campNode = (DefaultMutableTreeNode) root
					.getChildAt(j);
			if (campNode.getUserObject() instanceof Campaign) {
				campNode.removeAllChildren();
				Campaign camp = (Campaign) campNode.getUserObject();
				ArrayList execList = camp.getExecutionListFromModel();
				if (!execList.isEmpty()) {
					Iterator it = execList.iterator();
					while (it.hasNext()) {
						Execution exec = (Execution) it.next();
						addObject(campNode, exec, true);
					}
				}
			}
		}

		JButton addButton = new JButton("<");
		addButton.setToolTipText(Language.getInstance().getText("Retirer_de_la_selection"));
		addButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int i = chosenSelectedNodes.size() - 1;
				while (i >= 0) {
					DefaultMutableTreeNode node = (DefaultMutableTreeNode) chosenSelectedNodes
							.get(i);
					((DefaultTreeModel) chosenTree.getModel())
							.removeNodeFromParent(node);
					removeFromModel(node);
					i = chosenSelectedNodes.size() - 1;
				}
			}
		});

		JButton removeButton = new JButton(">");
		removeButton.setToolTipText(Language.getInstance().getText("Ajouter_a_la_selection"));
		removeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				for (int i = 0; i < testSelectedNodes.size(); i++) {
					Object nodeValue = ((DefaultMutableTreeNode) testSelectedNodes
							.get(i)).getUserObject();
					testSelectedNode = ((DefaultMutableTreeNode) testSelectedNodes
							.get(i));

					if (nodeValue instanceof Campaign) {
						addNodeToNode(testSelectedNode, temporaryChosenRootNode);
					} else if (nodeValue instanceof Execution) {
						DefaultMutableTreeNode campNode = findCampaignNodeInChosenTree(((Execution) nodeValue)
								.getCampagneFromModel().getNameFromModel());
						if (campNode != null) {
							addNodeToNode(testSelectedNode, campNode);
						} else {
							DefaultMutableTreeNode newCampNode = addObject(
									temporaryChosenRootNode,
									((Execution) nodeValue).getCampagneFromModel(), true);
							addNodeToNode(testSelectedNode, newCampNode);
						}
					}
				}
			}
		});

		JPanel buttonSet = new JPanel();
		buttonSet.setLayout(new BoxLayout(buttonSet, BoxLayout.Y_AXIS));
		buttonSet.add(removeButton);
		buttonSet.add(Box.createRigidArea(new Dimension(1, 25)));
		buttonSet.add(addButton);

		JScrollPane chosenScrollPane = new JScrollPane(chosenTree);
		chosenScrollPane.setBorder(BorderFactory
				.createTitledBorder(Language.getInstance().getText("Selection")));
		chosenScrollPane.setPreferredSize(new Dimension(300, 450));

		JScrollPane testScrollPane = new JScrollPane(testTree);
		testScrollPane.setBorder(BorderFactory.createTitledBorder(Language.getInstance().getText("Executions")));
		testScrollPane.setPreferredSize(new Dimension(300, 450));

		JLabel dateLabel = new JLabel(Language.getInstance().getText("Date_du_lancement_"));
        JPanel dateLabelPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        dateLabelPanel.add(dateLabel);
        date = new JDateChooser("d MMMMM yyyy", false);
        JPanel datePanel = new JPanel();
        datePanel.setLayout(new BoxLayout(datePanel, BoxLayout.Y_AXIS));
        datePanel.setBorder(BorderFactory.createTitledBorder(""));
        datePanel.add(dateLabelPanel);
        datePanel.add(Box.createRigidArea(new Dimension(1,10)));
        datePanel.add(date);
        
        JLabel hourLabel = new JLabel(Language.getInstance().getText("Heure_de_lancement_"));
        JPanel hourLabelPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        hourLabelPanel.add(hourLabel);
        hour = new JTextField(2);
        JLabel separator = new JLabel(":");
        minutes = new JTextField(2);
        JPanel hourFieldPanel = new JPanel();
        hourFieldPanel.add(hour);
        hourFieldPanel.add(separator);
        hourFieldPanel.add(minutes);
        JPanel hourPanel = new JPanel();
        hourPanel.setLayout(new BoxLayout(hourPanel, BoxLayout.Y_AXIS));
        hourPanel.setBorder(BorderFactory.createTitledBorder(""));
        hourPanel.add(hourLabelPanel);
        hourPanel.add(Box.createRigidArea(new Dimension(1,10)));
        hourPanel.add(hourFieldPanel);
        
        JLabel occurencesLabel = new JLabel(Language.getInstance().getText("Occurences_"));
        JPanel occurencesLabelPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        occurencesLabelPanel.add(occurencesLabel);
        occurences = new JTextField(4);
        occurences.setText("1");
        JPanel occurencesPanel = new JPanel();
        occurencesPanel.setLayout(new BoxLayout(occurencesPanel, BoxLayout.Y_AXIS));
        occurencesPanel.setBorder(BorderFactory.createTitledBorder(""));
        occurencesPanel.add(occurencesLabelPanel);
        occurencesPanel.add(Box.createRigidArea(new Dimension(1,10)));
        occurencesPanel.add(occurences);
        
        JLabel intervalLabel = new JLabel("Interval :");
        JPanel intervalLabelPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        intervalLabelPanel.add(intervalLabel);
        interval = new JTextField(4);
        interval.setText("0");
        JLabel min = new JLabel("min");
        JPanel intervalFieldPanel = new JPanel();
        intervalFieldPanel.add(interval);
        intervalFieldPanel.add(min);
        JPanel intervalPanel = new JPanel();
        intervalPanel.setLayout(new BoxLayout(intervalPanel, BoxLayout.Y_AXIS));
        intervalPanel.setBorder(BorderFactory.createTitledBorder(""));
        intervalPanel.add(intervalLabelPanel);
        intervalPanel.add(Box.createRigidArea(new Dimension(1,10)));
        intervalPanel.add(intervalFieldPanel);
        
        JLabel stopOnErrorLabel = new JLabel(Language.getInstance().getText("Options_d_execution_"));
        JPanel stopOnErrorPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
        stopOnErrorPanel.add(stopOnErrorLabel);
        stopOnError = new JCheckBox();
        stopOnError.setSelected(true);
        JLabel stopLabel = new JLabel(Language.getInstance().getText("Arreter_si_erreur"));
        JPanel checkPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        checkPanel.add(stopOnError);
        checkPanel.add(stopLabel);
        JPanel optionsPanel = new JPanel();
        optionsPanel.setLayout(new BoxLayout(optionsPanel, BoxLayout.Y_AXIS));
        optionsPanel.setBorder(BorderFactory.createTitledBorder(""));
        optionsPanel.add(stopOnErrorLabel);
        optionsPanel.add(Box.createRigidArea(new Dimension(1,10)));
        optionsPanel.add(checkPanel);
        
        JPanel infoPanel = new JPanel();
        infoPanel.setLayout(new GridLayout(1,5));
        infoPanel.add(datePanel);
        infoPanel.add(hourPanel);
        infoPanel.add(occurencesPanel);
        infoPanel.add(intervalPanel);
        infoPanel.add(optionsPanel);
		
		JPanel windowPanel = new JPanel();
		windowPanel.setLayout(new BoxLayout(windowPanel, BoxLayout.X_AXIS));
		windowPanel.add(testScrollPane);
		windowPanel.add(Box.createRigidArea(new Dimension(20, 50)));
		windowPanel.add(buttonSet);
		windowPanel.add(Box.createRigidArea(new Dimension(20, 50)));
		windowPanel.add(chosenScrollPane);

		JButton validate = new JButton(Language.getInstance().getText("Valider"));
		validate.setToolTipText(Language.getInstance().getText("Valider"));
		validate.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				init();
				String hourText = hour.getText();
				String minText = minutes.getText();
				String occText = occurences.getText();
				String intText = interval.getText();

				if ((hourText == null) || (hourText.equalsIgnoreCase("")) || 
					(minText == null) || (minText.equalsIgnoreCase("")) ||
					(occText == null) || (occText.equalsIgnoreCase("")) ||
					(intText == null) || (intText.equalsIgnoreCase(""))) {
					JOptionPane.showMessageDialog(
                            ExecChooser.this,
                            Language.getInstance().getText("Vous_devez_saisir_l_heure_le_nombre_d_occurence_et_l_intervalle_entre_les_executions"),
                            Language.getInstance().getText("Erreur_"),
                            JOptionPane.ERROR_MESSAGE);
				} else {
					try {
						int hourValue = Integer.parseInt(hourText);
						int minutesValue = Integer.parseInt(minText);
						int occurencesValue = Integer.parseInt(occText);
						int intervalValue = Integer.parseInt(intText);
						
						if( (hourValue < 0) || (hourValue > 23) || (minutesValue < 0) ||(minutesValue > 59) ) {
							JOptionPane.showMessageDialog(
		                            ExecChooser.this,
		                            Language.getInstance().getText("Format_de_l_heure_incorrect"),
		                            Language.getInstance().getText("Erreur_"),
		                            JOptionPane.ERROR_MESSAGE);
						} else {
							if (temporaryExecList.size() == 0) {
								JOptionPane.showMessageDialog(
			                            ExecChooser.this,
			                            Language.getInstance().getText("Vous_devez_selectionner_au_moins_une_execution"),
			                            Language.getInstance().getText("Erreur_"),
			                            JOptionPane.ERROR_MESSAGE);
							} else {
								Date firstTime = date.getDate();
								firstTime.setHours(hourValue);
								firstTime.setMinutes(minutesValue);
								firstTime.setSeconds(0);
								
								if (firstTime.before(new Date())) {
									JOptionPane.showMessageDialog(
				                            ExecChooser.this,
				                            Language.getInstance().getText("La_date_programmee_est_deja_passee"),
				                            Language.getInstance().getText("Erreur_"),
				                            JOptionPane.ERROR_MESSAGE);
								} else {
								
									if (occurencesValue <= 0) {
										JOptionPane.showMessageDialog(
					                            ExecChooser.this,
					                            Language.getInstance().getText("Le_nombre_d_occurences_doit_etre_superieur_ou_egal_a_1_"),
					                            Language.getInstance().getText("Erreur_"),
					                            JOptionPane.ERROR_MESSAGE);
									}else if (occurencesValue == 1) {
										ExecTask execTask = new ExecTask(temporaryExecList,pTimer,occurencesValue,stopOnError.isSelected());
										pTimer.schedule(execTask,firstTime);
										cronPlg.setAlreadyCron(true);
										cronPlg.getCronButton().setIcon(new javax.swing.ImageIcon(getClass().getResource("/salomeTMF_plug/cronExec/resources/cron_on.gif")));
										if (!Api.isNET_CHANGE_TRACK()){
											Api.runListenerThread(DataModel.getCurrentProject().getNameFromModel());
										}
										ExecChooser.this.dispose();
									} else {
										if (intervalValue <= 0) {
											JOptionPane.showMessageDialog(
						                            ExecChooser.this,
						                            Language.getInstance().getText("L_intervalle_entre_les_executions_doit_etre_superieur_ou_egal_a_1_"),
						                            Language.getInstance().getText("Erreur_"),
						                            JOptionPane.ERROR_MESSAGE);
										} else {
											long period = intervalValue*60000;
											
											ExecTask execTask = new ExecTask(temporaryExecList,pTimer,occurencesValue,stopOnError.isSelected());
											pTimer.schedule(execTask,firstTime,period);
											cronPlg.setAlreadyCron(true);
											cronPlg.getCronButton().setIcon(new javax.swing.ImageIcon(getClass().getResource("/salomeTMF_plug/cronExec/resources/cron_on.gif")));
											if (!Api.isNET_CHANGE_TRACK()){
												Api.runListenerThread(DataModel.getCurrentProject().getNameFromModel());
											}
											ExecChooser.this.dispose();
										}
									}
								}
							}
						}
						
					} catch (NumberFormatException nfe) {
						JOptionPane.showMessageDialog(
	                            ExecChooser.this,
	                            Language.getInstance().getText("Problemes_dans_le_format_des_entrees_saisies"),
	                            Language.getInstance().getText("Erreur_"),
	                            JOptionPane.ERROR_MESSAGE);
					}
				}
			}
		});

		JButton cancel = new JButton("Annuler");
		cancel.setToolTipText(Language.getInstance().getText("Annuler"));
		cancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				cronPlg.setAlreadyCron(false);
				cronPlg.getCronButton().setIcon(new javax.swing.ImageIcon(getClass().getResource("/salomeTMF_plug/cronExec/resources/cron_off.gif")));
				ExecChooser.this.dispose();
			}
		});

		JPanel secondButtonSet = new JPanel();
		secondButtonSet.add(validate);
		secondButtonSet.add(cancel);

		JPanel center = new JPanel();
		center.add(windowPanel);

		JPanel page = new JPanel();
		page.setLayout(new BoxLayout(page, BoxLayout.Y_AXIS));
		page.add(infoPanel);
		page.add(center);
		page.add(secondButtonSet);

		Container contentPaneFrame = this.getContentPane();
		contentPaneFrame.add(page, BorderLayout.CENTER);
		this.setTitle(Language.getInstance().getText("Cron_executions"));
		//this.setLocation(300, 200);
		 this.setLocationRelativeTo(this.getParent()); 
		this.pack();
		this.setVisible(true);

	} // Fin du constructeur ExecChooser

	/**
	 * Methode Recursive qui ajoute un noeud a un autre noeud. Tous les fils du 
	 * noeud a ajouter sont aussi ajouter recursivement. 
	 * @param nodeToBeAdded le noeud a ajouter
	 * @param nodeToReceive le noeud qui reçoit
	 */
	private void addNodeToNode(DefaultMutableTreeNode nodeToBeAdded,
			DefaultMutableTreeNode nodeToReceive) {
		DefaultMutableTreeNode newNode = null;
		for (int j = 0; j < nodeToReceive.getChildCount(); j++) {
			if (((DefaultMutableTreeNode) nodeToReceive.getChildAt(j))
					.getUserObject().equals(nodeToBeAdded.getUserObject())) {
				newNode = (DefaultMutableTreeNode) nodeToReceive.getChildAt(j);
			}
		}
		if (newNode == null) {
			newNode = new DefaultMutableTreeNode(nodeToBeAdded.getUserObject());
			chosenTreeModel.insertNodeInto(newNode, nodeToReceive,
					nodeToReceive.getChildCount());
			chosenTree.scrollPathToVisible(new TreePath(newNode.getPath()));
			if (nodeToBeAdded.getUserObject() instanceof Family) {
				temporaryFamilyList.add(nodeToBeAdded.getUserObject());
			} else if (nodeToBeAdded.getUserObject() instanceof TestList) {
				temporaryTestListList.add(nodeToBeAdded.getUserObject());
			} else if (nodeToBeAdded.getUserObject() instanceof Test) {
				temporaryTestList.add(nodeToBeAdded.getUserObject());
			}
		}
		for (int i = 0; i < nodeToBeAdded.getChildCount(); i++) {
			addNodeToNode((DefaultMutableTreeNode) nodeToBeAdded.getChildAt(i),
					newNode);
		}
	} // Fin de la methode addNodeToNode/2

	/**
	 * Methode qui retourne le noeud correspondant a une campagne si le nom passe
	 * en parametre est le nom d'une campagne presente dans l'arbre des selections
	 * @param campName un nom
	 * @return lengthnoeud correspondant a une campagne si lengthnom passe 
	 * en parametre est lengthnom d'une campagne presente dans l'arbre des selections,
	 * <code>null</code> sinon.
	 */
	public DefaultMutableTreeNode findCampaignNodeInChosenTree(String campName) {
		DefaultMutableTreeNode root = (DefaultMutableTreeNode) chosenTreeModel
				.getRoot();
		for (int j = 0; j < root.getChildCount(); j++) {
			DefaultMutableTreeNode campNode = (DefaultMutableTreeNode) root
					.getChildAt(j);
			if (campNode.getUserObject() instanceof Campaign
					&& ((Campaign) campNode.getUserObject()).getNameFromModel().equals(
							campName)) {
				return campNode;
			}
		}
		return null;
	} // Fin de la methode findCampaignNode/1 

	/**
	 * Methode qui retourne le noeud correspondant a une famille si le nom passe
	 * en parametre est le nom d'une famille presente dans l'arbre des selections 
	 * @param familyName un nom
	 * @return le noeud correspondant a une famille si le nom passe
	 * en parametre est le nom d'une famille presente dans l'arbre des selections,
	 * <code>null</code> sinon.
	 */
	public DefaultMutableTreeNode findFamilyNodeInChosenTree(String familyName) {
		DefaultMutableTreeNode root = (DefaultMutableTreeNode) chosenTreeModel
				.getRoot();
		for (int j = 0; j < root.getChildCount(); j++) {
			DefaultMutableTreeNode familyNode = (DefaultMutableTreeNode) root
					.getChildAt(j);
			if (familyNode.getUserObject() instanceof Family
					&& ((Family) familyNode.getUserObject()).getNameFromModel().equals(
							familyName)) {
				return familyNode;
			}
		}
		return null;
	} // Fin de la methode findFamilyNode/1

	/**
	 * Methode qui retourne le noeud correspondant a une suite si le nom passe
	 * en parametre est le nom d'une suite presente dans l'arbre des selections 
	 * @param testListName un nom
	 * @return le noeud correspondant a une famille si le nom passe
	 * en parametre est le nom d'une famille presente dans l'arbre des selections,
	 * <code>null</code> sinon. 
	 */
	public DefaultMutableTreeNode findTestListNodeInChosenTree(
			String testListName, String familyName) {
		DefaultMutableTreeNode root = (DefaultMutableTreeNode) chosenTreeModel
				.getRoot();
		for (int j = 0; j < root.getChildCount(); j++) {
			DefaultMutableTreeNode familyNode = (DefaultMutableTreeNode) root
					.getChildAt(j);
			if (familyNode.getUserObject() instanceof Family
					&& ((Family) familyNode.getUserObject()).getNameFromModel().equals(
							familyName)) {
				for (int k = 0; k < familyNode.getChildCount(); k++) {
					DefaultMutableTreeNode testListNode = (DefaultMutableTreeNode) familyNode
							.getChildAt(k);
					if (testListNode.getUserObject() instanceof TestList
							&& ((TestList) testListNode.getUserObject())
									.getNameFromModel().equals(testListName)) {
						return testListNode;
					}
				}
			}
		}
		return null;
	} // Fin de la methode findFamilyNode/1

	/**
	 * Methode d'ajout d'un noeud dans l'arbre sous le parent.
	 * @param parent le parent 
	 * @param child le noeud a ajouter
	 * @param shouldBeVisible visible ou non
	 * @return le nouveau noeud de l'arbre
	 */
	public DefaultMutableTreeNode addObject(DefaultMutableTreeNode parent,
			Object child, boolean shouldBeVisible) {

		DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(child);
		if (parent == null) {
			parent = temporaryChosenRootNode;
		}

		// Insertion du noeud
		chosenTreeModel.insertNodeInto(childNode, parent, parent
				.getChildCount());

		// on s'assure que le noeud est visible
		if (shouldBeVisible) {
			chosenTree.scrollPathToVisible(new TreePath(childNode.getPath()));
		}
		return childNode;
	} // Fin de la classe addObject/3

	/**
	 * Supprime du modele les elements retires de la selection.
	 * @param node le noeud a partir duquel on retire les elements
	 */
	private void removeFromModel(DefaultMutableTreeNode node) {
		for (int i = 0; i < node.getChildCount(); i++) {
			removeFromModel((DefaultMutableTreeNode) node.getChildAt(i));
		}
		if (node.getUserObject() instanceof Family) {
			temporaryFamilyList.remove(node.getUserObject());
		} else if (node.getUserObject() instanceof TestList) {
			temporaryTestListList.remove(node.getUserObject());
		} else if (node.getUserObject() instanceof Test) {
			temporaryTestList.remove(node.getUserObject());
		}
	} // Fin de la methode removeFromModel/1

	/**
	 * Methode qui initialise l'arbre temporaire a partir de la racine passe
	 * en parametre. 
	 * @param root la racine du nouvel arbre temporaire
	 */
	public void initTemporaryTree(DefaultMutableTreeNode root) {
		for (int i = 0; i < root.getChildCount(); i++) {
			addNodeToNode((DefaultMutableTreeNode) root.getChildAt(i),
					temporaryChosenRootNode);
		}
	} // Fin de la methode initTemporaryTree/1

	public void init() {

		temporaryCampList = new ArrayList();
		temporaryExecList = new ArrayList();
		DefaultMutableTreeNode root = (DefaultMutableTreeNode) chosenTreeModel
				.getRoot();
		for (int j = 0; j < root.getChildCount(); j++) {
			DefaultMutableTreeNode campNode = (DefaultMutableTreeNode) root
					.getChildAt(j);

			temporaryCampList.add((Campaign)campNode.getUserObject());
			for (int k = 0; k < campNode.getChildCount(); k++) {
				DefaultMutableTreeNode execNode = (DefaultMutableTreeNode)campNode
						.getChildAt(k);
				if (execNode.getUserObject() instanceof Execution) {
					Execution exec = (Execution) execNode.getUserObject();
					ExecutionWrapper execWrap = new ExecutionWrapper();
					execWrap.setName(exec.getNameFromModel());
					execWrap.setLastDate(exec.getLastDateFromModel());
					execWrap.setCreationDate(exec.getCreationDateFromModel());
					execWrap.setIdBDD(exec.getIdBdd());
					execWrap.setDescription(exec.getDescriptionFromModel());
					execWrap.setCampId(exec.getCampagneFromModel().getIdBdd());
					temporaryExecList.add(execWrap);
				}
			}

		}
	}

} // Fin de la classe ExecChooser
