/*
 * TextArea.java - The Area where date are written
 * Copyright (C) 2003 Alexandre THOMAS
 * alexthomas@free.fr
 * http://helpgui.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package salomeTMF_plug.helpgui.gui;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.io.IOException;
import java.net.URL;

import javax.swing.JTextPane;
import javax.swing.text.View;

import org.objectweb.salome_tmf.api.Util;

import salomeTMF_plug.helpgui.page.Page;
import salomeTMF_plug.helpgui.util.Out;


/**
 * Class where the date are showed
 * 
 * @author Alexandre THOMAS
 */
public class TextArea extends JTextPane implements Printable {

	/* Attribut for printing */
	int currentPage = -1;                   
	double pageEndY = 0;              
	double pageStartY = 0;      
	boolean scaleWidthToFit = true;
	
	/** Constructor */
	public TextArea() {
		super();
		setEditable(false);
	}

	/** Fonction call when a new page has to be show */
	public void update(Page page) {
		if (!page.isLeaf() && page.getTarget() == null) {
			return;
		}

		try {
			URL url;
			if (page.isPluginPage()){
				Util.log("[helpgui:page] Try open plug-in page : "+MainPanel.pluginDir +"/"+ page.getPluginsName() +"/"+ MainPanel.helpDir +"/" +MainPanel.lang + "/" + page.getLink());
				try {
					url = new URL(MainPanel.helpPath + "/" + MainPanel.pluginDir +"/"+ page.getPluginsName() +"/" +MainPanel.helpDir +"/" +MainPanel.lang + "/" + page.getLink());
					setPage(url);	
				} catch (Exception e1) {
					Util.log("[helpgui:page] Error then try to open main page : "+MainPanel.pluginDir +"/"+ page.getPluginsName() +"/"+ MainPanel.helpDir +"/" +MainPanel.defaultlang + "/" + page.getLink());
					url = new URL(MainPanel.helpPath + "/" + MainPanel.pluginDir +"/"+ page.getPluginsName() +"/" +MainPanel.helpDir +"/" +MainPanel.defaultlang + "/" + page.getLink());
					setPage(url);
				}
			} else {
				Util.log("[helpgui:page] Try open main help page : "+MainPanel.helpDir +"/" +MainPanel.lang + "/" + page.getLink());
				try {
					url = new URL(MainPanel.helpPath + "/"+MainPanel.helpDir +"/" +MainPanel.lang + "/" + page.getLink());
					setPage(url);
				} catch (Exception e1) {
					Util.log("[helpgui:page] Error then try open main page : "+MainPanel.helpDir +"/" +MainPanel.defaultlang + "/" + page.getLink());
					url = new URL(MainPanel.helpPath + "/"+MainPanel.helpDir +"/" +MainPanel.defaultlang + "/" + page.getLink());
					setPage(url);
				}
			}
		} catch (IOException e) {
			Out.msg("Unable to show the page : " + e, Out.FAILED);
			setText("Error in showing the page : " + e);
		} catch (Exception e) {
			Out.msg("Error in showing the page", Out.FAILED);
			setText("Error in showing the page : " + e);
		}
	}

	/** Print the current page */
//	 Adding code for print capacity
	public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) 
	{
		//This function is the main printable overided function
 
		double scale = 1.0;
		Graphics2D graphics2D;
		View rootView;
		graphics2D = (Graphics2D) graphics;
 
		setSize((int)pageFormat.getImageableWidth(),Integer.MAX_VALUE);
		validate();
 
		rootView = this.getUI().getRootView(this);
 
		if((scaleWidthToFit) && (getMinimumSize().getWidth() > pageFormat.getImageableWidth())) {
			scale = pageFormat.getImageableWidth()/getMinimumSize().getWidth();
			graphics2D.scale(scale, scale);
		}
 
		//The below four command lines shows that the content is clipped 
		//to the size of the printable page
 
		graphics2D.setClip((int)(pageFormat.getImageableX()/scale),
		(int)(pageFormat.getImageableY()/scale),
		(int)(pageFormat.getImageableWidth()/scale),
		(int)(pageFormat.getImageableHeight()/scale));
 
		//The below if statement is to check to see if there is a new page to render
 
		if(pageIndex > currentPage){
			currentPage = pageIndex;
			pageStartY += pageEndY;
			pageEndY = graphics2D.getClipBounds().getHeight();
		}
 
		graphics2D.translate(graphics2D.getClipBounds().getX(),
		graphics2D.getClipBounds().getY());
 
		Rectangle allocation = new Rectangle(0, (int)-pageStartY,
		(int)(getMinimumSize().getWidth()),
		(int)(getPreferredSize().getHeight()));
 
		//The below if else statements return PAGE_EXISTS only if the class 
		//sees that there are some contents in the document by calling the printView class 
 
		if(printView(graphics2D,allocation,rootView)){
			return PAGE_EXISTS;
		}
 
		else {
			pageStartY = 0;
			pageEndY = 0;
			currentPage = -1;
			return NO_SUCH_PAGE;
		}
	}
 
	protected boolean printView(Graphics2D graphics2D, Shape allocation, View view) {
		//This function paints the page if it exists
 
		boolean pageExists = false;
		Rectangle clipRectangle = graphics2D.getClipBounds();
		Shape childAllocation;
		View childView;
 
		if(view.getViewCount() > 0) {
			for(int i = 0;i<view.getViewCount();i++) {
				childAllocation = view.getChildAllocation(i,allocation);
				if (childAllocation != null) {
					childView = view.getView(i);
					if(printView(graphics2D,childAllocation,childView))  {
						pageExists = true;
					}
				}
 
			}
 
		} else  {
			//The below if statement checks if there are pages currently to paint
			if(allocation.getBounds().getMaxY() >= clipRectangle.getY()) {
				pageExists = true;
				if((allocation.getBounds().getHeight() > clipRectangle.getHeight()) &&
				(allocation.intersects(clipRectangle))) {
					view.paint(graphics2D,allocation);
				} else {
					if(allocation.getBounds().getY() >= clipRectangle.getY()) {
						if(allocation.getBounds().getMaxY() <= clipRectangle.getMaxY() - 15) {
							view.paint(graphics2D,allocation);
						} else {
							if(allocation.getBounds().getY() < pageEndY) {
								pageEndY = allocation.getBounds().getY();
							}
						}
					}
				}
			}
		}
		return pageExists;
	}
	
	
}
