/*
* SalomeTMF is a Test Management Framework
* Copyright (C) 2005 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* @author Marche Mikael
*
* Contact: mikael.marche@rd.francetelecom.com
*/

package salomeTMF_plug.requirements.ihm;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.FontMetrics;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PiePlot3D;
import org.jfree.data.general.DefaultPieDataset;
import org.jfree.util.Rotation;
import org.objectweb.salome_tmf.api.ApiConstants;
import org.objectweb.salome_tmf.data.Campaign;
import org.objectweb.salome_tmf.data.DataConstants;
import org.objectweb.salome_tmf.data.Environment;
import org.objectweb.salome_tmf.data.Execution;
import org.objectweb.salome_tmf.data.ExecutionResult;
import org.objectweb.salome_tmf.data.ManualTest;
import org.objectweb.salome_tmf.data.Test;
import org.objectweb.salome_tmf.ihm.languages.Language;
import org.objectweb.salome_tmf.ihm.main.ActionDetailsView;
import org.objectweb.salome_tmf.ihm.main.AttachmentView;
import org.objectweb.salome_tmf.ihm.main.AttachmentViewWindow;
import org.objectweb.salome_tmf.ihm.main.SalomeTMFContext;
import org.objectweb.salome_tmf.ihm.main.ViewCampaignOfTest;
import org.objectweb.salome_tmf.ihm.main.datawrapper.DataModel;
import org.objectweb.salome_tmf.ihm.main.htmleditor.EkitCore;
import org.objectweb.salome_tmf.ihm.models.DynamicTree;
import org.objectweb.salome_tmf.plugins.core.BugTracker;

import salomeTMF_plug.requirements.ReqPlugin;
import salomeTMF_plug.requirements.data.ReqFilter;
import salomeTMF_plug.requirements.data.ReqLeaf;
import salomeTMF_plug.requirements.data.Requirement;
import salomeTMF_plug.requirements.sqlWrapper.ReqWrapper;

public class RequirementDescriptionPanel extends JPanel implements Observer, ListSelectionListener, ActionListener{
	
	//JTextPane requirementDescription;
	EkitCore pRequirementDescription;
	HistoryPanel pHistoryPanel;
	JPanel requirementIDPanel;
	//JPanel requirementIDFamPanel;
	JPanel requirementInfoPanel;
	
	
	JComboBox priorityBox;
	Vector itemPriority;
	
	JComboBox catBox;
	Vector itemCat;
	
	JComboBox complexeBox;
	Vector itemComplex;
	
	JComboBox stateBox;
	Vector itemState;
	
	JTextField versionField;
	//Vector versionList;
	JComboBox origineField;
	Vector origineList;
	JComboBox verifmodeField;
	Vector verifList;
	JComboBox referenceField;
	Vector referenceList;
	
	
	JComboBox comboFiltreChoice;
	JComboBox comboNameChoice;
	JButton changeVersion;
	
	JTabbedPane requirementWorkSpace;
	AttachmentView attachment;
	String lastDes;
	
	Requirement pRequirement;
	Requirement poldRequirement;
	
	
	
	//boolean tabChanged = false;
	boolean isPie3D = false;
	
	JTable testReqFamilyTable;
	JTable testReqLeafTable;
	JTable satisfactionTable;
	
	Vector testCover ;
	DynamicTree pDynamicTree;
	
	JButton viewTestFamilyReqButton;
	JButton viewCampFamilyReqButton;
	JPanel coverFamilyPanel;
	
	JButton viewTestReqLeafButton;
	JButton viewCampReqLeafButton;
	JPanel coverLeafPanel;
	
	JPanel details ;
	
	JButton viewResExecReqButton;
	JButton viewCampButton;
	JPanel statisfactionPanel;
	JLabel coverTestRepresend ;
	JLabel satisfaction;
	
	ChartPanel chartPanAllCover;
	PiePlot3D plotAllCover;
    DefaultPieDataset datasetAllCover;
    
    ChartPanel chartPanFamilyCover;
	PiePlot3D plotFamilyCover;
    DefaultPieDataset datasetFamilyCover;
    
    //ChartPanel chartPanSatisCover;
    JTabbedPane defectTabPanel;
    
	PiePlot3D plotSatisCover;
    DefaultPieDataset datasetSatisCover;
    
    boolean canDoUpdatePrirotity = false;
    boolean canDoUpdateNameChoise = true;
    boolean canDoUpdateState = false;
    boolean canDoUpdateComplexe = false;
    boolean canDoUpdateCat = false;
    
    StatRequirement pStatRequirement;
    
    /*Vartiable used for computation (refhresh by updateData*/    
    Vector satisfactionTableData = new Vector();
    Vector satisfactionTableFiltredData = new Vector();
    
    //Vector leafInCurrentFamily;
    //Hashtable coverReqsInCurrentFamily  = new Hashtable();
    FiltrePanel pFiltrePanel;
    RequirementTree pReqTree;
	public RequirementDescriptionPanel(StatRequirement _pStatRequirement, FiltrePanel _pFiltrePanel, RequirementTree reqTree){
		super(new BorderLayout());
		pFiltrePanel = _pFiltrePanel;
		pReqTree = reqTree;
		pStatRequirement = _pStatRequirement;
		initComponent();
		add(requirementWorkSpace, BorderLayout.CENTER);
	}
	
	void initComponent(){
		testCover = new Vector();
		requirementWorkSpace = new JTabbedPane();
		
		attachment = new AttachmentView(null, -1 , -1, null, DataConstants.NORMAL_SIZE_FOR_ATTACH, null);
	
		details = initDetailPanel();
		
		coverLeafPanel = initCoverLeafPanel();
		coverFamilyPanel = initCoverFamilyPanel();
		chartPanAllCover = initChartPanAllCover();
		statisfactionPanel = initSatisfactionReqPanel();
		//requirementWorkSpace.addTab(Language.getInstance().getText("Details"),details) ;
		//requirementWorkSpace.addTab(Language.getInstance().getText("Attachements"), attachment);
		//requirementWorkSpace.addTab(Language.getInstance().getText("Couverture"), coverFamilyPanel);
		requirementWorkSpace.addTab(Language.getInstance().getText("Couverture"), chartPanAllCover);
	}
	
	
	JPanel initDetailPanel(){
		JPanel jp_details = new JPanel(new BorderLayout());
		itemPriority = new Vector();
		itemPriority.add(Language.getInstance().getText("Filtre_Exigence_Haute"));
		itemPriority.add(Language.getInstance().getText("Filtre_Exigence_Moyenne"));
		itemPriority.add(Language.getInstance().getText("Filtre_Exigence_Faible"));
		itemPriority.add(Language.getInstance().getText("Filtre_Exigence_Option"));
		
		
		itemCat = new Vector();
		itemCat.add(Language.getInstance().getText("Cat_Fonctionnel"));
		itemCat.add(Language.getInstance().getText("Cat_Interoperabilite"));
		itemCat.add(Language.getInstance().getText("Cat_Charge"));
		itemCat.add(Language.getInstance().getText("Cat_Performance"));
		itemCat.add(Language.getInstance().getText("Cat_Disponibilite"));
		itemCat.add(Language.getInstance().getText("Cat_Securite"));
		itemCat.add(Language.getInstance().getText("Cat_Exploitabilite"));
		itemCat.add(Language.getInstance().getText("Cat_Autre"));

		itemState = new Vector();
		itemState.add(Language.getInstance().getText("State_A_Analyser"));
		itemState.add(Language.getInstance().getText("State_Analysee"));
		itemState.add(Language.getInstance().getText("State_Approuvee"));
		itemState.add(Language.getInstance().getText("State_Verifiee"));
		itemState.add(Language.getInstance().getText("State_Finalisee"));
		itemState.add(Language.getInstance().getText("State_Reportee"));
		itemState.add(Language.getInstance().getText("State_Abandonnee"));
		
		itemComplex = new Vector();
		itemComplex.add(Language.getInstance().getText("Com_C0"));
		itemComplex.add(Language.getInstance().getText("Com_C1"));
		itemComplex.add(Language.getInstance().getText("Com_C2"));
		itemComplex.add(Language.getInstance().getText("Com_C3"));
		
		requirementIDPanel = new JPanel();
		requirementIDPanel.setLayout(new GridLayout(2, 4));
		
		JLabel labelCat = new JLabel(Language.getInstance().getText("Categorie") + " :");
		catBox = new JComboBox(itemCat);
		catBox.setEditable(false);
		catBox.addActionListener(this);
		
		
		JLabel labelPriority = new JLabel(Language.getInstance().getText("Priority")+ " :");
		priorityBox = new JComboBox(itemPriority);
		priorityBox.setEditable(false);
		priorityBox.addActionListener(this);
		
		
		JLabel labelComplexe = new JLabel(Language.getInstance().getText("Complexite")+ " :");
		complexeBox = new JComboBox(itemComplex);
		complexeBox.setEditable(false);
		complexeBox.addActionListener(this);
		
		JLabel labelState = new JLabel(Language.getInstance().getText("Etat")+ " :");
		stateBox = new JComboBox(itemState);
		stateBox.setEditable(false);
		stateBox.addActionListener(this);
		
		requirementIDPanel.add(labelCat);
		requirementIDPanel.add(labelPriority);
		requirementIDPanel.add(labelComplexe);
		requirementIDPanel.add(labelState);
		requirementIDPanel.add(catBox);
		requirementIDPanel.add(priorityBox);
		requirementIDPanel.add(complexeBox);
		requirementIDPanel.add(stateBox);
		requirementIDPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK),"id : "));
		
		
		//JPanel desc_histoPanel = new JPanel(new BorderLayout());
		JPanel desc_histoPanel = new JPanel(new GridLayout(2, 1));
		/*requirementDescription = new JTextPane();
		requirementDescription.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK),"Description"));
		*/
		
		pRequirementDescription = new EkitCore(pReqTree, true);
		pHistoryPanel = new HistoryPanel();
		//desc_histoPanel.add(requirementDescription, BorderLayout.CENTER);
		
		/*JSplitPane jspltDisplay = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		jspltDisplay.setTopComponent(pRequirementDescription);
		jspltDisplay.setBottomComponent(pHistoryPanel);
			*/
		desc_histoPanel.add(pRequirementDescription, BorderLayout.CENTER);
		desc_histoPanel.add(pHistoryPanel);
		//desc_histoPanel.add(jspltDisplay, BorderLayout.CENTER);
		
		
		requirementInfoPanel = new JPanel();
		requirementInfoPanel.setLayout(new GridLayout(2, 6));
		requirementInfoPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK),"Information"));
		
		origineList = new Vector();
		origineList.add(Language.getInstance().getText("Origine_value_0"));
		origineList.add(Language.getInstance().getText("Origine_value_1"));
		origineList.add(Language.getInstance().getText("Origine_value_2"));
		origineList.add(Language.getInstance().getText("Origine_value_3"));	
		
		verifList  = new Vector();
		verifList.add(Language.getInstance().getText("Verif_value_0"));
		verifList.add(Language.getInstance().getText("Verif_value_1"));
		verifList.add(Language.getInstance().getText("Verif_value_2"));
		verifList.add(Language.getInstance().getText("Verif_value_3"));
		
		referenceList = new Vector();
		referenceList.add(Language.getInstance().getText("Reference_value_0"));
		referenceList.add(Language.getInstance().getText("Reference_value_1"));
		referenceList.add(Language.getInstance().getText("Reference_value_2"));
		referenceList.add(Language.getInstance().getText("Reference_value_3"));
		
		JLabel labelOrigine = new JLabel("Origine : ");
		origineField = new JComboBox();
		origineField.setEditable(true);
		
		JLabel labelVersion = new JLabel("Version : ");
		versionField = new JTextField();
		
		JLabel labelReference = new JLabel(Language.getInstance().getText("Reference") +" : ");
		referenceField = new JComboBox();
		referenceField.setEditable(true);
		
		JLabel labelVerif = new JLabel(Language.getInstance().getText("Mode_de_verification"));
		verifmodeField = new JComboBox();
		verifmodeField.setEditable(true);
		
		
		changeVersion = new JButton(Language.getInstance().getText("Valider"));
		changeVersion.addActionListener(this);
		
		requirementInfoPanel.add(labelOrigine);
		requirementInfoPanel.add(origineField);
		//requirementInfoPanel.add(Box.createRigidArea(new Dimension(5,5)));
		requirementInfoPanel.add(labelReference);
		requirementInfoPanel.add(referenceField);
		//requirementInfoPanel.add(Box.createRigidArea(new Dimension(5,5)));		
		requirementInfoPanel.add(labelVersion);
		requirementInfoPanel.add(versionField);
		
		requirementInfoPanel.add(labelVerif);
		requirementInfoPanel.add(verifmodeField);
		requirementInfoPanel.add(Box.createRigidArea(new Dimension(5,5)));
		requirementInfoPanel.add(Box.createRigidArea(new Dimension(5,5)));
		requirementInfoPanel.add(Box.createRigidArea(new Dimension(5,5)));
		requirementInfoPanel.add(changeVersion);
	
		
		jp_details.add(requirementIDPanel,BorderLayout.NORTH);
		//jp_details.add(requirementDescription,BorderLayout.CENTER);
		jp_details.add(desc_histoPanel,BorderLayout.CENTER);
		jp_details.add(requirementInfoPanel,BorderLayout.SOUTH);
		
		
		/*requirementInfoPanel = new JPanel();
		requirementInfoPanel.setLayout(new GridLayout(1, 6));
		JLabel labelPriority = new JLabel("Priority : ");
		priorityBox = new JComboBox(itemPriority);
		priorityBox.setEditable(false);
		priorityBox.addActionListener(this);
		JLabel labelVersion = new JLabel("Version : ");
		versionField = new JTextField();
		changeVersion = new JButton(Language.getInstance().getText("Valider"));
		changeVersion.addActionListener(this);
		
		requirementInfoPanel.add(labelPriority);
		requirementInfoPanel.add(priorityBox);
		requirementInfoPanel.add(Box.createRigidArea(new Dimension(15,15)));
		requirementInfoPanel.add(labelVersion);
		requirementInfoPanel.add(versionField);
		requirementInfoPanel.add(changeVersion);
		requirementInfoPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK),"Information"));
	
		requirementDescription = new JTextPane();
		requirementDescription.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK),"Description"));
		//requirementDescription.addFocusListener(new requirementDescriptionFocusListener());
		
		jp_details.add(requirementInfoPanel,BorderLayout.NORTH);
		jp_details.add(requirementDescription,BorderLayout.CENTER);
		*/
		return jp_details;
	}
	
	JPanel initSatisfactionReqPanel(){
		JPanel jp_sat_Req = new JPanel(new GridLayout(2,1));
		
		viewResExecReqButton = new JButton(Language.getInstance().getText("Visualiser") + " " + Language.getInstance().getText("Derniere_Execution"));
		viewResExecReqButton.addActionListener(this);
		viewCampButton= new JButton(Language.getInstance().getText("Visualiser") + " " +Language.getInstance().getText("Campagne"));
		viewCampButton.addActionListener(this);
		/*viewCampFamilyReqButton = new JButton(Language.getInstance().getText("Campagne"));
		viewCampFamilyReqButton.addActionListener(this);
		*/
		JPanel buttonsPanel = new JPanel(new GridLayout(1,2));
    	buttonsPanel.add(viewResExecReqButton);
    	buttonsPanel.add(viewCampButton);
    	//buttonsPanel.add(viewCampFamilyReqButton);
    	
		SimpleTableModel model = new SimpleTableModel();
    	
		satisfactionTable = new JTable(model);

		model.addColumn(Language.getInstance().getText("Exigence"));
        model.addColumn(Language.getInstance().getText("Test"));
        model.addColumn(Language.getInstance().getText("Campagnes") + "/" + Language.getInstance().getText("Executions"));
        model.addColumn(Language.getInstance().getText("Resultats_d_execution"));
        model.addColumn(Language.getInstance().getText("Derniere_Execution"));
        
        satisfactionTable.setPreferredScrollableViewportSize(new Dimension(600, 200));
        satisfactionTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    	ListSelectionModel rowSM = satisfactionTable.getSelectionModel();
    	rowSM.addListSelectionListener(this);
        
    	JScrollPane tablePane = new JScrollPane(satisfactionTable);
    	tablePane.setBorder(BorderFactory.createRaisedBevelBorder());
    
    	JPanel infoSatisCoverPanel = new JPanel(new BorderLayout());
    	JPanel fitrePanel = initFiltreSatisCoverPanel();
    	infoSatisCoverPanel.add(fitrePanel, BorderLayout.NORTH);
    	infoSatisCoverPanel.add(buttonsPanel, BorderLayout.SOUTH);
    	infoSatisCoverPanel.add(tablePane, BorderLayout.CENTER);
    	//chartPanSatisCover = initChartPanSatisCover();
    	
    	JPanel defectPanel = new JPanel(new BorderLayout());
    	JPanel labelPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
    	JLabel defectLabel = new JLabel(Language.getInstance().getText("Anomalies"));
    	labelPanel.add(defectLabel);
    	defectTabPanel = new JTabbedPane();
    	defectPanel.add(labelPanel, BorderLayout.NORTH);
    	defectPanel.add(defectTabPanel, BorderLayout.CENTER);
    	
    	jp_sat_Req.add(infoSatisCoverPanel);
    	//jp_sat_Req.add(chartPanSatisCover);
    	jp_sat_Req.add(defectPanel);
    	 
		
		return jp_sat_Req;
	}
	
	JPanel initFiltreSatisCoverPanel() {
		
		JPanel filtreSatisCoverPanel =  new JPanel(new GridLayout(2,3));
		
		JLabel filtreByLabel = new JLabel(Language.getInstance().getText("Filtrer_par") + " : ");
		Vector choice = new Vector();
		choice.add(Language.getInstance().getText("Tout"));
		choice.add(Language.getInstance().getText("Campagnes"));
		choice.add(Language.getInstance().getText("Campagnes") + "/" +Language.getInstance().getText("Executions"));
		choice.add(Language.getInstance().getText("Environnements"));
		choice.add(Language.getInstance().getText("Campagnes") + "/" +Language.getInstance().getText("Environnements"));
		comboFiltreChoice = new JComboBox(choice);
		comboFiltreChoice.setEditable(false);
		comboFiltreChoice.addActionListener(this);
		
		
		JLabel nameLabel = new JLabel(Language.getInstance().getText("Nom") + " : ");
		
		Vector choice2 = new Vector();
		choice2.add(Language.getInstance().getText("Tout"));
		comboNameChoice = new JComboBox(choice2);
		comboNameChoice.setEditable(false);
		comboNameChoice.addActionListener(this);
		
		coverTestRepresend = new  JLabel(Language.getInstance().getText("Test_couvert_represente") + ": 100%" ); 
		
		satisfaction = new JLabel(Language.getInstance().getText("Satisfaction"));
		
		
		filtreSatisCoverPanel.add(filtreByLabel);
		filtreSatisCoverPanel.add(nameLabel);
		filtreSatisCoverPanel.add(coverTestRepresend);
		
		filtreSatisCoverPanel.add(comboFiltreChoice);
		filtreSatisCoverPanel.add(comboNameChoice);
		filtreSatisCoverPanel.add(satisfaction);
		
		return filtreSatisCoverPanel;
	}
	
	
	JPanel initCoverFamilyPanel(){
		JPanel jp_cover_fam = new JPanel(new GridLayout(2,1));
		
		viewTestFamilyReqButton = new JButton(Language.getInstance().getText("Visualiser"));
		viewTestFamilyReqButton.addActionListener(this);
		viewCampFamilyReqButton = new JButton(Language.getInstance().getText("Campagne"));
		viewCampFamilyReqButton.addActionListener(this);
		
		JPanel buttonsPanel = new JPanel(new GridLayout(1,2));
    	buttonsPanel.add(viewTestFamilyReqButton);
    	buttonsPanel.add(viewCampFamilyReqButton);
    	
		SimpleTableModel model = new SimpleTableModel();
    	
		testReqFamilyTable = new JTable(model);

    	model.addColumn(Language.getInstance().getText("Famille"));
        model.addColumn(Language.getInstance().getText("Suite"));
        model.addColumn(Language.getInstance().getText("Test"));
        model.addColumn(Language.getInstance().getText("Execute"));
        
        
        testReqFamilyTable.setPreferredScrollableViewportSize(new Dimension(600, 200));
        testReqFamilyTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    	//ListSelectionModel rowSM = testTable.getSelectionModel();
    	//rowSM.addListSelectionListener(this);
        
    	JScrollPane tablePane = new JScrollPane(testReqFamilyTable);
    	tablePane.setBorder(BorderFactory.createRaisedBevelBorder());
    
    	JPanel infoTestCoverPanel = new JPanel(new BorderLayout());
    	infoTestCoverPanel.add(buttonsPanel, BorderLayout.NORTH);
    	infoTestCoverPanel.add(tablePane, BorderLayout.CENTER);
    	chartPanFamilyCover = initChartPanFamilyCover();
    	jp_cover_fam.add(infoTestCoverPanel);
    	jp_cover_fam.add(chartPanFamilyCover);
    	
    	
    	
    	return jp_cover_fam;
	}
	
	
	JPanel initCoverLeafPanel(){
		
		viewTestReqLeafButton = new JButton(Language.getInstance().getText("Visualiser"));
		viewTestReqLeafButton.addActionListener(this);
		viewCampReqLeafButton = new JButton(Language.getInstance().getText("Campagne"));
		viewCampReqLeafButton.addActionListener(this);
		
		JPanel buttonsPanel = new JPanel(new GridLayout(1,2));
    	buttonsPanel.add(viewTestReqLeafButton);
    	buttonsPanel.add(viewCampReqLeafButton);
    	
		SimpleTableModel model = new SimpleTableModel();
    	
		testReqLeafTable = new JTable(model);

    	model.addColumn(Language.getInstance().getText("Famille"));
        model.addColumn(Language.getInstance().getText("Suite"));
        model.addColumn(Language.getInstance().getText("Test"));
        model.addColumn(Language.getInstance().getText("Execute"));
        
        
        testReqLeafTable.setPreferredScrollableViewportSize(new Dimension(600, 200));
        testReqLeafTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    	//ListSelectionModel rowSM = testTable.getSelectionModel();
    	//rowSM.addListSelectionListener(this);
        
    	JScrollPane tablePane = new JScrollPane(testReqLeafTable);
    	tablePane.setBorder(BorderFactory.createRaisedBevelBorder());
    
    	JPanel infoTestCoverPanel = new JPanel(new BorderLayout());
    	infoTestCoverPanel.add(buttonsPanel, BorderLayout.NORTH);
    	infoTestCoverPanel.add(tablePane, BorderLayout.CENTER);
    	return infoTestCoverPanel;
	}
	
	ChartPanel initChartPanSatisCover() {
		datasetSatisCover = new  DefaultPieDataset();
		JFreeChart chart = ChartFactory.createPieChart3D(
				Language.getInstance().getText("Satisfaction"),  // chart title
				datasetSatisCover,                // data
	            true,                   // include legend
	            true,
	            false
	        );
	
		plotSatisCover =  (PiePlot3D) chart.getPlot();
		plotSatisCover.setStartAngle(290);
		plotSatisCover.setDirection(Rotation.CLOCKWISE);
		plotSatisCover.setForegroundAlpha(0.5f);
		plotSatisCover.setNoDataMessage("No data to display");
		return  new ChartPanel(chart);
		
	}
	
	ChartPanel initChartPanFamilyCover(){
		datasetFamilyCover = new  DefaultPieDataset();
		JFreeChart chart = ChartFactory.createPieChart3D(
				Language.getInstance().getText("Exigence_couverte"),  // chart title
	            datasetFamilyCover,                // data
	            true,                   // include legend
	            true,
	            false
	        );
	
		plotFamilyCover =  (PiePlot3D) chart.getPlot();
		plotFamilyCover.setStartAngle(290);
		plotFamilyCover.setDirection(Rotation.CLOCKWISE);
		plotFamilyCover.setForegroundAlpha(0.5f);
		plotFamilyCover.setNoDataMessage("No data to display");
		return  new ChartPanel(chart);
	}
	
	
	ChartPanel initChartPanAllCover(){
		datasetAllCover = new  DefaultPieDataset();
		JFreeChart chart = ChartFactory.createPieChart3D(
				Language.getInstance().getText("Exigence_couverte"),  // chart title
	            datasetAllCover,                // data
	            true,                   // include legend
	            true,
	            false
	        );
	
		plotAllCover =  (PiePlot3D) chart.getPlot();
		plotAllCover.setStartAngle(290);
		plotAllCover.setDirection(Rotation.CLOCKWISE);
		plotAllCover.setForegroundAlpha(0.5f);
		plotAllCover.setNoDataMessage("No data to display");
		//chartPanAllCover = new ChartPanel(chart);
		return  new ChartPanel(chart);
	}
	
	
	 void  setPie3D(int filtre){
		if (isPie3D)
			return;
		isPie3D = true;
		/*requirementWorkSpace.remove(details) ;
		requirementWorkSpace.remove(attachment);
		requirementWorkSpace.remove(coverFamilyPanel);*/
		requirementWorkSpace.removeAll();
		calcStatAllCover(filtre);
		requirementWorkSpace.addTab(Language.getInstance().getText("Couverture"), chartPanAllCover);
		
	}
	
	void unsetPie3D(int filtre, Requirement pReq){
		//if (!isPie3D)
		//return;
		isPie3D = false;
		//requirementWorkSpace.remove(chartPanAllCover);
		int selectedTab = requirementWorkSpace.getSelectedIndex();
		requirementWorkSpace.removeAll();
		requirementWorkSpace.addTab(Language.getInstance().getText("Details"),details) ;
		requirementWorkSpace.addTab(Language.getInstance().getText("Attachements"), attachment);
		if (pReq.isFamilyReq()){
			calcStatFamilyCover(filtre);
			requirementWorkSpace.addTab(Language.getInstance().getText("Couverture"), coverFamilyPanel);
		} else {
			requirementWorkSpace.addTab(Language.getInstance().getText("Couverture"), coverLeafPanel);
		}
		requirementWorkSpace.addTab(Language.getInstance().getText("Satisfaction"),statisfactionPanel);
		if (selectedTab < requirementWorkSpace.getTabCount()) {
			requirementWorkSpace.setSelectedIndex(selectedTab);
		}
	}
	
	
	public void update(Observable o, Object arg) {
		//System.out.println("requirementdescription-update : ReqEvent.REQSELECTED");
		if (arg instanceof ReqEvent){
			//System.out.println("requirementdescription-update : ReqEvent.REQSELECTED");
			ReqEvent event = (ReqEvent)arg;
			if (event.code == ReqEvent.REQSELECTED ){
				poldRequirement = pRequirement;
				pRequirement = (Requirement)event.arg;
				if (!pRequirement.getNameFromModel().equals("Requirements_" + ReqPlugin.getProjectRef().getNameFromModel())){
					attachment.setAttachmentObject(pRequirement);
					if (pRequirement.isFamilyReq()){
						updateData(pFiltrePanel.getFiltre(), (SimpleTableModel)testReqFamilyTable.getModel());
					} else {
						updateData(pFiltrePanel.getFiltre(), (SimpleTableModel)testReqLeafTable.getModel());
					}
					upDateHistory(pRequirement);
				}
				
				if (!pRequirement.getNameFromModel().equals("Requirements_" +ReqPlugin.getProjectRef().getNameFromModel())){
					//requirementDescription.setText(pRequirement.getDescriptionFromModel());
					pRequirementDescription.setText(pRequirement.getDescriptionFromModel());
				}
				
				if (pRequirement instanceof ReqLeaf) {
					requirementInfoPanel.setVisible(true);
					requirementIDPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK),"id : " + pRequirement.getIdBdd()));
					requirementIDPanel.setVisible(true);
					pRequirementDescription.setBorder(BorderFactory.createRaisedBevelBorder());
					
					versionField.setText(((ReqLeaf)pRequirement).getVersionFromModel());
					setInfo(referenceField,referenceList, ((ReqLeaf)pRequirement).getReferenceFromModel());
					setInfo(origineField, origineList, ((ReqLeaf)pRequirement).getOrigineFromModel());
					setInfo(verifmodeField, verifList, ((ReqLeaf)pRequirement).getVerifFromModel());
					
					canDoUpdatePrirotity = false;
					int priority = ((ReqLeaf)pRequirement).getPriorityFromModel();
					if (priority == ReqFilter.P_HIGHT){
						priorityBox.setSelectedIndex(0);
					} else if (priority == ReqFilter.P_MEDIUM) {
						priorityBox.setSelectedIndex(1);
					} else if (priority == ReqFilter.P_LOW) {
						priorityBox.setSelectedIndex(2);
					} else if (priority == ReqFilter.P_NONE) {
						priorityBox.setSelectedIndex(3);
					}
					canDoUpdatePrirotity = true;
					
					canDoUpdateState=false;
					int state = ((ReqLeaf)pRequirement).getStateFromModel();
					stateBox.setSelectedIndex(state);
					canDoUpdateState=true;
					
					canDoUpdateComplexe=false;
					int complexe = ((ReqLeaf)pRequirement).getComplexeFromModel();
					if (complexe == ReqFilter.C0){
						complexeBox.setSelectedIndex(0);
					} else if (complexe == ReqFilter.C1) {
						complexeBox.setSelectedIndex(1);
					} else if (complexe == ReqFilter.C2) {
						complexeBox.setSelectedIndex(2);
					} else if (complexe == ReqFilter.C3) {
						complexeBox.setSelectedIndex(3);
					}
					canDoUpdateComplexe=true;
					
					canDoUpdateCat=false;
					int cat = ((ReqLeaf)pRequirement).getCatFromModel();
					catBox.setSelectedIndex(cat);
					canDoUpdateCat=true;
					
					
				} else {
					requirementInfoPanel.setVisible(false);	
					requirementIDPanel.setVisible(false);
					pRequirementDescription.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK),"id : " + pRequirement.getIdBdd()));
				}
				
				if (pRequirement.getParent() == null){
					setPie3D(pFiltrePanel.getFiltre());
				} else {
					unsetPie3D(pFiltrePanel.getFiltre(), pRequirement);
				} 
			}else if (event.code == ReqEvent.FILTRE_REQ_CHANGE){
				if (pRequirement != null && pRequirement.getParent() != null){
					calcStatAllCover(pFiltrePanel.getFiltre());
				}
				
			}
		}
	}
	
	void updateData(int filtre, SimpleTableModel pTestTable){
		int typeSelected = comboFiltreChoice.getSelectedIndex();
		String nameStelected = (String) comboNameChoice.getSelectedItem();	
		canDoUpdateNameChoise = false;
		if (typeSelected > 0){
			comboNameChoice.removeAllItems();
			comboNameChoice.addItem(Language.getInstance().getText("Tout"));
		}
		
		pTestTable.clearTable();
		SimpleTableModel satisfactionTableModel = (SimpleTableModel) satisfactionTable.getModel();
		satisfactionTableModel.clearTable();
		satisfactionTableData.clear();
		satisfactionTableFiltredData.clear();
		
		Hashtable executedStatCovredByCurrentReq = pStatRequirement.getEexecutedStatCovredByCurrentReq();
		Enumeration enumCovredTest = pStatRequirement.getAllTestCovredByCurrentReq().elements();
		testCover.clear();
		int i = 0;
		int j = 0;
		int nbLastPass = 0;
		Vector listTest = new Vector();
		while (enumCovredTest.hasMoreElements()){
			Test pTest = (Test) enumCovredTest.nextElement();
			Vector data = new Vector();
			testCover.add(pTest);
			String famName = pTest.getTestListFromModel().getFamilyFromModel().getNameFromModel();
			String suiteName = pTest.getTestListFromModel().getNameFromModel();
			String testName = pTest.getNameFromModel();
			data.add(famName);
			data.add(pTest.getTestListFromModel().getNameFromModel());
			data.add(testName);
			boolean executed = false;
			Vector statExec = (Vector) executedStatCovredByCurrentReq.get(pTest);
			if (statExec != null){
				//statExec = Vector[campain, Execution, {ExecResult}, nbP, nbF, nbU, lastExecRusult, lastResult]
				executed = true;
				int size = statExec.size();
				for (int k=0; k < size ; k++){
					Vector statExecLine = (Vector) statExec.elementAt(k);
					Vector data2 = new Vector();
					String str_listOfReq = "";
					Hashtable hashTestReq = pStatRequirement.getAllReqCovredByTest();
					Vector listOfReq = (Vector) hashTestReq.get(new Integer(pTest.getIdBdd()));
					if (listOfReq != null && listOfReq.size()>0){
						str_listOfReq = ((Requirement)listOfReq.elementAt(0)).getNameFromModel();
						for (int l = 1 ; l < listOfReq.size(); l++){
							str_listOfReq += ", " + ((Requirement)listOfReq.elementAt(l)).getNameFromModel();
						}
					}
					data2.add(str_listOfReq);
					data2.add(testName + " (" + famName + "/" + suiteName + ")");
					if (!listTest.contains(pTest)){
						listTest.add(pTest);
					}
					Campaign pCamp = (Campaign) statExecLine.elementAt(0);
					Execution pExec = (Execution) statExecLine.elementAt(1);
					int nbPass = ((Integer) statExecLine.elementAt(3)).intValue();
					int nbFail = ((Integer) statExecLine.elementAt(4)).intValue();
					int nbUnknow = ((Integer) statExecLine.elementAt(5)).intValue();
					String lastResult = (String) statExecLine.elementAt(7);
					data2.add(pCamp.getNameFromModel() + "/" + pExec.getNameFromModel());
					data2.add("" + nbPass + "P, " +nbFail + "F, " +nbUnknow + "I");
					data2.add(lastResult);
					if (lastResult.equals(ApiConstants.SUCCESS)){
						nbLastPass++;
					}
					satisfactionTableModel.insertRow(j,data2);
					satisfactionTableData.add(j, statExecLine);
					satisfactionTableFiltredData.add(j, statExecLine);
					j++;
					//Set campagne, exec, env Filtres //remplir des hashtables
				}
			}
			//boolean executed = pStatRequirement.isCovredTestIsExecuted(pTest);
			data.add(""+executed);
			pTestTable.insertRow(i,data);
			i++;
		}
		float pourcentSat = 0;
		if (j!=0){
			pourcentSat= nbLastPass*100/j;
		}
		//satisfaction.setText(Language.getInstance().getText("Satisfaction") + " : " + pourcentSat +"%");
		
		int nbTest = pStatRequirement.getAllTestCovredByCurrentReq().size();
		int nbTest2 = pStatRequirement.getEexecutedStatCovredByCurrentReq().size();
		float pourcentTest = 0;
		float pourcentTest2 = 0;
		if (nbTest!=0){
			pourcentTest = listTest.size()*100/nbTest;
		}
		if (nbTest2 != 0){
			pourcentTest2 = listTest.size()*100/nbTest2;
		}
		coverTestRepresend.setText(Language.getInstance().getText("Test_couvert_represente") + ": " + pourcentTest+"%" ); 
		satisfaction.setText(Language.getInstance().getText("Satisfaction") + " : " + pourcentSat+"% / " + pourcentTest2 +"% " + Language.getInstance().getText("Execute"));
		canDoUpdateNameChoise = true;
		if (typeSelected > 0){
			changeFiltreChoicePerformed(null);
			int nbName = comboNameChoice.getItemCount();
			if (nbName > 0){
				int index = 0;
				boolean trouve = false;
				while (index < nbName && !trouve ){
					String tempName = (String) comboNameChoice.getItemAt(index);
					if (tempName.equals(nameStelected)){
						trouve = true;
						comboNameChoice.setSelectedIndex(index);
					}
					index++;
				}
			}
		}
		
		Vector allTrackers = SalomeTMFContext.getInstance().getBugTracker();
		defectTabPanel.removeAll();
		int size = allTrackers.size();
		for (int k = 0 ; k < size ; k++){
			BugTracker pBugTracker =  (BugTracker)allTrackers.elementAt(k);
			JPanel pJPanel = pBugTracker.getDefectPanelForTests(listTest);
			if (pJPanel != null){
				defectTabPanel.addTab(pBugTracker.getBugTrackerName(), pJPanel);
			}
		}
		setColumnSize(testReqFamilyTable);
		setColumnSize(testReqLeafTable);
		setColumnSize(satisfactionTable);
	}
	

	void upDateHistory(Requirement pReq){
		pHistoryPanel.updateData(pReq, false);
	}
	
	void calcStatFamilyCover(int filtre){
		try {
			
			int nbTotalReq = 0;
			int nbCoveredReq = 0;
			if (filtre> 0){
				//nbTotalReq = leafInCurrentFamily.size();
				nbTotalReq = pStatRequirement.getLeafInCurrentFamily().size();
				//int nbTotalReq2 = 0;
				for (int i = 0 ; i < nbTotalReq; i++){
					//ReqLeaf pReqLeaf = (ReqLeaf) leafInCurrentFamily.elementAt(i);
					ReqLeaf pReqLeaf = (ReqLeaf) pStatRequirement.getLeafInCurrentFamily().elementAt(i);
					
					//int reqP = pReqLeaf.getPriorityFromModel();
					//if ((reqP & filtre) > 0){
					//nbTotalReq2++;
					/*if (coverReqsInCurrentFamily.get(new Integer(pReqLeaf.getIdBdd())) != null){
					 nbCoveredReq++;
					 }*/
					if (pStatRequirement.getAllTestCovredByReq().get(new Integer(pReqLeaf.getIdBdd())) != null){
						nbCoveredReq++;
					}
					//}
					
				}
				//nbTotalReq = nbTotalReq2;
				
			}
			
			//int nbTotalReq = Requirement.getReqWrapperLeafInCurrentProject().size();
			//int nbCoveredReq =  Requirement.getReqWrapperCovered().size(); 
			if (nbTotalReq > 0) {
				Double nbCovered = new Double( (nbCoveredReq*100)/nbTotalReq);
				Double nbNotCovered = new Double(100 -  (nbCoveredReq*100)/nbTotalReq);
				datasetFamilyCover.setValue(Language.getInstance().getText("Exigence_non_couverte"), nbNotCovered);
				datasetFamilyCover.setValue(Language.getInstance().getText("Exigence_couverte"),nbCovered);
				plotFamilyCover.setDataset(datasetFamilyCover);
			} 
			
		} catch (Exception e){
			
		}
	}
	
	void initCalcStatAllCover(int filtre){
		try {
			int nbTotalReq = 1;
			int nbCoveredReq = 0;
			if (filtre> 0){
				Vector leafInCurrentProject = Requirement.getReqWrapperLeafInCurrentProject();
				nbTotalReq = leafInCurrentProject.size();
				
				Vector reqWrapperCovered =  Requirement.getReqWrapperCovered(); 
				nbCoveredReq = reqWrapperCovered.size();
				if (filtre != 111){
					int nbTotalReq2 = 0;
					for (int i = 0 ; i < nbTotalReq; i++){
						int reqP = ((ReqWrapper) leafInCurrentProject.elementAt(i)).getPriority();
						if ((reqP & filtre) > 0){
							nbTotalReq2++;
						}
					}
					int nbCoveredReq2 = 0;
					for (int i = 0 ; i < nbCoveredReq; i++){
						int reqP = ((ReqWrapper) reqWrapperCovered.elementAt(i)).getPriority();
						if ((reqP & filtre) > 0){
							nbCoveredReq2++;
						}
					}
					nbTotalReq = nbTotalReq2;
					nbCoveredReq = nbCoveredReq2;
				}
			}
			
		
			
			if (nbTotalReq > 0) {
				Double nbCovered = new Double( (nbCoveredReq*100)/nbTotalReq);
				Double nbNotCovered = new Double(100 -  (nbCoveredReq*100)/nbTotalReq);
				datasetAllCover.setValue(Language.getInstance().getText("Exigence_non_couverte"), nbNotCovered);
				datasetAllCover.setValue(Language.getInstance().getText("Exigence_couverte"),nbCovered);
				plotAllCover.setDataset(datasetAllCover);
			} 
			
		} catch (Exception e){
			
		}
	}
	
	void calcStatAllCover(int filtre){
		try {
			/*int nbTotalReq = 1;
			int nbCoveredReq = 0;
			if (filtre> 0){
				Vector leafInCurrentProject = Requirement.getReqWrapperLeafInCurrentProject();
				nbTotalReq = leafInCurrentProject.size();
				
				Vector reqWrapperCovered =  Requirement.getReqWrapperCovered(); 
				nbCoveredReq = reqWrapperCovered.size();
				if (filtre != 111){
					int nbTotalReq2 = 0;
					for (int i = 0 ; i < nbTotalReq; i++){
						int reqP = ((ReqWrapper) leafInCurrentProject.elementAt(i)).getPriority();
						if ((reqP & filtre) > 0){
							nbTotalReq2++;
						}
					}
					int nbCoveredReq2 = 0;
					for (int i = 0 ; i < nbCoveredReq; i++){
						int reqP = ((ReqWrapper) reqWrapperCovered.elementAt(i)).getPriority();
						if ((reqP & filtre) > 0){
							nbCoveredReq2++;
						}
					}
					nbTotalReq = nbTotalReq2;
					nbCoveredReq = nbCoveredReq2;
				}
			}*/
			
			int nbTotalReq = 1;
			int nbCoveredReq = 0;
			if (filtre> 0){
				//nbTotalReq = leafInCurrentFamily.size();
				nbTotalReq = pStatRequirement.getLeafInCurrentFamily().size();
				//int nbTotalReq2 = 0;
				for (int i = 0 ; i < nbTotalReq; i++){
					//ReqLeaf pReqLeaf = (ReqLeaf) leafInCurrentFamily.elementAt(i);
					ReqLeaf pReqLeaf = (ReqLeaf) pStatRequirement.getLeafInCurrentFamily().elementAt(i);
					
					//int reqP = pReqLeaf.getPriorityFromModel();
					//if ((reqP & filtre) > 0){
					//nbTotalReq2++;
					/*if (coverReqsInCurrentFamily.get(new Integer(pReqLeaf.getIdBdd())) != null){
					 nbCoveredReq++;
					 }*/
					if (pStatRequirement.getAllTestCovredByReq().get(new Integer(pReqLeaf.getIdBdd())) != null){
						nbCoveredReq++;
					}
					//}
					
				}
				//nbTotalReq = nbTotalReq2;
				
			}
			
			if (nbTotalReq > 0) {
				Double nbCovered = new Double( (nbCoveredReq*100)/nbTotalReq);
				Double nbNotCovered = new Double(100 -  (nbCoveredReq*100)/nbTotalReq);
				datasetAllCover.setValue(Language.getInstance().getText("Exigence_non_couverte"), nbNotCovered);
				datasetAllCover.setValue(Language.getInstance().getText("Exigence_couverte"),nbCovered);
				plotAllCover.setDataset(datasetAllCover);
			} 
			
		} catch (Exception e){
			
		}
	}
	
	void setRequirement(Requirement pRequirement){
		poldRequirement = this.pRequirement;
		this.pRequirement = pRequirement;
		if (!pRequirement.getNameFromModel().equals("Requirements_" +ReqPlugin.getProjectRef().getNameFromModel())){
			attachment.setAttachmentObject(pRequirement);
			if (pRequirement.isFamilyReq()){
				updateData(pFiltrePanel.getFiltre(), (SimpleTableModel)testReqFamilyTable.getModel());
			} else {
				updateData(pFiltrePanel.getFiltre(), (SimpleTableModel)testReqLeafTable.getModel());
			}
		}
		if (pRequirement.getParent() == null){
			setPie3D(pFiltrePanel.getFiltre());
		} else {
			unsetPie3D(pFiltrePanel.getFiltre(), pRequirement);
		} 
	}
	
	public void actionPerformed(ActionEvent e){
		if (e.getSource().equals(viewTestFamilyReqButton)){
			viewTestPerformed(e, testReqFamilyTable);
		} else if (e.getSource().equals(viewCampFamilyReqButton)){
			viewCampPerformed(e, testReqFamilyTable);
		} else if (e.getSource().equals(viewTestReqLeafButton)){
			viewTestPerformed(e, testReqLeafTable);
		} else if (e.getSource().equals(viewCampReqLeafButton)){
			viewCampPerformed(e, testReqLeafTable);
		} else if (e.getSource().equals(priorityBox)){
			priorityBoxPerformed(e);
		}  else if (e.getSource().equals(stateBox)){
			stateBoxPerformed(e);
		} else if (e.getSource().equals(complexeBox)){
			complexeBoxPerformed(e);
		} else if (e.getSource().equals(catBox)){
			catBoxPerformed(e);
		} else if (e.getSource().equals(changeVersion)){
			changeVersionPerformed(e);
		} else if (e.getSource().equals(comboNameChoice)){
			changeNamePerformed(e);
		} else if (e.getSource().equals(comboFiltreChoice)){
			changeFiltreChoicePerformed(e);
		} else if (e.getSource().equals(viewResExecReqButton)){
			viewResExecReqButtonPerformed(e);
		} else if (e.getSource().equals(viewCampButton)){
			viewCampButtonButtonPerformed(e);
		}
	}

	void setTestTree(DynamicTree _pDynamicTree){
		pDynamicTree = _pDynamicTree;
	}
	
	void viewCampButtonButtonPerformed(ActionEvent e) {
		int selectedRow = satisfactionTable.getSelectedRow();
		if (selectedRow >-1){
			Vector statExecLine = (Vector) satisfactionTableFiltredData.elementAt(selectedRow);
			Campaign pCamp = (Campaign) statExecLine.elementAt(0);
			DataModel.view(pCamp);
		}
	}
	
	void viewResExecReqButtonPerformed(ActionEvent e){
		//Get selected 
		int selectedIndex = satisfactionTable.getSelectedRow();
		if (selectedIndex >-1){
			Vector statExecLine = (Vector) satisfactionTableFiltredData.elementAt(selectedIndex);
			ExecutionResult lastExecutionResult = (ExecutionResult) statExecLine.elementAt(6);
			Test pTest = (Test) statExecLine.elementAt(8);
			DataModel.setCurrentExecutionTestResult(lastExecutionResult.getExecutionTestResultFromModel(pTest));
			DataModel.setObervedExecutionResult(lastExecutionResult);
			if (pTest instanceof ManualTest){
				new ActionDetailsView((ManualTest)pTest, lastExecutionResult.getExecution(), lastExecutionResult, lastExecutionResult.getExecutionTestResultFromModel(pTest));
			} else {
				new AttachmentViewWindow(SalomeTMFContext.getBaseIHM(), DataConstants.EXECUTION_RESULT_TEST, DataModel.getCurrentExecutionTestResult());	
			}
		}
	}
	
	void changeNamePerformed(ActionEvent e){
		if (!canDoUpdateNameChoise){
			return ;
		}
		int choice = comboFiltreChoice.getSelectedIndex();
		Vector data = null;
		Enumeration keys;
		boolean trouve = false;
		String name = (String) comboNameChoice.getSelectedItem();
		if (choice == 0){
			//ALL
			//USE satisfactionTableData
			reloadDefaultSatisfactionTable();
		} else if (choice == 1){
			//Camp
			keys = pStatRequirement.executedStatByCamp.keys();
			while (keys.hasMoreElements() && !trouve) {
				Campaign pCamp = (Campaign) keys.nextElement();
				if (pCamp.getNameFromModel().equals(name)){
					trouve = true;
					data = (Vector) pStatRequirement.executedStatByCamp.get(pCamp);
				}
			} 
		} else if (choice == 2){
			//Exex
			keys = pStatRequirement.executedStatByExec.keys();
			while (keys.hasMoreElements() && !trouve) {
				Execution pExec = (Execution) keys.nextElement();
				String longName = pExec.getCampagneFromModel().getNameFromModel() + "/" + pExec.getNameFromModel();
				if (longName.equals(name)){
					trouve = true;
					data = (Vector) pStatRequirement.executedStatByExec.get(pExec);
				}
			}
		} else if (choice == 3){
			//Env
			keys = pStatRequirement.executedStatByEnv.keys();
			while (keys.hasMoreElements() && !trouve) {
				Environment pEnv = (Environment) keys.nextElement();
				if (pEnv.getNameFromModel().equals(name)){
					trouve = true;
					data = (Vector) pStatRequirement.executedStatByEnv.get(pEnv);
				}
			} 
		} else if (choice == 4){
			//Exex
			keys = pStatRequirement.executedStatByExec.keys();
			while (keys.hasMoreElements()) {
				Execution pExec = (Execution) keys.nextElement();
				String longName = pExec.getCampagneFromModel().getNameFromModel() + "/" + pExec.getEnvironmentFromModel().getNameFromModel();
				if (longName.equals(name)){
					if (data == null){
						data = new Vector();
					} 
					Vector data2 = (Vector) pStatRequirement.executedStatByExec.get(pExec);
					int size = data2.size();
					for (int i = 0; i< size ; i++){
						data.add(data2.elementAt(i));
					}
				}
			}
		}
		
		if (data != null){
			//loadFromFiltre(data);
			loadSatisfactionTable(data);
		} else {
			reloadDefaultSatisfactionTable();
		}
	}
	
	void changeFiltreChoicePerformed(ActionEvent e){
		int choice = comboFiltreChoice.getSelectedIndex();
		
		//if e != null
		canDoUpdateNameChoise = false;
		comboNameChoice.removeAllItems();
		comboNameChoice.addItem(Language.getInstance().getText("Tout"));
		Enumeration data = null;
		
		if (choice == 0){
			
		} else if (choice == 1){
			//Camp
			data = pStatRequirement.executedStatByCamp.keys();
		} else if (choice == 2){
			//Exex
			data = pStatRequirement.executedStatByExec.keys();
		} else if (choice == 3){
			//Env
			data = pStatRequirement.executedStatByEnv.keys();
		} else if (choice == 4){
			//Env
			data = pStatRequirement.executedStatByExec.keys();
		} 
		if (data != null){
			Vector unique = new Vector();
			while (data.hasMoreElements()){
				if (choice == 1){
					//Camp
					Campaign pCamp = (Campaign) data.nextElement();
					comboNameChoice.addItem(pCamp.getNameFromModel());
				} else if (choice == 2){
					//Exex
					Execution pExec = (Execution) data.nextElement();
					comboNameChoice.addItem(pExec.getCampagneFromModel().getNameFromModel() + "/" + pExec.getNameFromModel() );
				} else if (choice == 3){
					//Env
					Environment pEnv = (Environment) data.nextElement();
					comboNameChoice.addItem(pEnv.getNameFromModel() );
				} else if (choice == 4){
					Execution pExec = (Execution) data.nextElement();
					String name = pExec.getCampagneFromModel().getNameFromModel() + "/" + pExec.getEnvironmentFromModel().getNameFromModel();
					if (!unique.contains(name)){
						comboNameChoice.addItem(pExec.getCampagneFromModel().getNameFromModel() + "/" + pExec.getEnvironmentFromModel().getNameFromModel() );
						unique.add(name);
					}
				} 
			}
		}
		canDoUpdateNameChoise = true;
		//end if e!=null
		
		//SimpleTableModel satisfactionTableModel = (SimpleTableModel) satisfactionTable.getModel();
		//if (satisfactionTableData.size() != satisfactionTableModel.getRowCount()){
			reloadDefaultSatisfactionTable();
		//}
		
	}
	
	/*void loadFromFiltre(Vector filtredData){
		satisfactionTableFiltredData.clear();
		SimpleTableModel satisfactionTableModel = (SimpleTableModel) satisfactionTable.getModel();
		satisfactionTableModel.clearTable();
		int nbLastPass = 0;
		Vector listTest = new Vector();
		int size = filtredData.size();
		for (int k=0; k < size ; k++){
			Vector statExecLine = (Vector) filtredData.elementAt(k);
			Vector data2 = new Vector();
			Test pTest =  (Test) statExecLine.elementAt(8);
			if (!listTest.contains(pTest)){
				listTest.add(pTest);
			}
			String famName = pTest.getTestListFromModel().getFamilyFromModel().getNameFromModel();
			String suiteName = pTest.getTestListFromModel().getNameFromModel();
			String testName = pTest.getNameFromModel();
			
			data2.add(testName + " (" + famName + "/" + suiteName + ")");
			Campaign pCamp = (Campaign) statExecLine.elementAt(0);
			Execution pExec = (Execution) statExecLine.elementAt(1);
			int nbPass = ((Integer) statExecLine.elementAt(3)).intValue();
			int nbFail = ((Integer) statExecLine.elementAt(4)).intValue();
			int nbUnknow = ((Integer) statExecLine.elementAt(5)).intValue();
			String lastResult = (String) statExecLine.elementAt(7);
			data2.add(pCamp.getNameFromModel() + "/" + pExec.getNameFromModel());
			data2.add("" + nbPass + "P, " +nbFail + "F, " +nbUnknow + "I");
			data2.add(lastResult);
			if (lastResult.equals(ApiConstants.SUCCESS)){
				nbLastPass++;
			}
			satisfactionTableModel.insertRow(k,data2);
			satisfactionTableFiltredData.add(k, statExecLine);
			//j++;
			//Set campagne, exec, env Filtres //remplir des hashtables
		}
		float pourcentSat = 0;
		if (size!=0){
			pourcentSat= nbLastPass*100/size;
		}
		//satisfaction.setText(Language.getInstance().getText("Satisfaction") + " : " + pourcentSat+"%");
		int nbTest = pStatRequirement.getAllTestCovredByCurrentReq().size();
		int nbTest2 = pStatRequirement.getEexecutedStatCovredByCurrentReq().size();
		float pourcentTest = 0;
		float pourcentTest2 = 0;
		if (nbTest!=0){
			pourcentTest = listTest.size()*100/nbTest;
		}
		if (nbTest2 != 0){
			pourcentTest2 = listTest.size()*100/nbTest2;
		}
		coverTestRepresend.setText(Language.getInstance().getText("Test_couvert_represente") + ": " + pourcentTest+"%" ); 
		satisfaction.setText(Language.getInstance().getText("Satisfaction") + " : " + pourcentSat+"% / " + pourcentTest2 +"% " + Language.getInstance().getText("Execute"));
		
	}*/
	
	void reloadDefaultSatisfactionTable(){
		loadSatisfactionTable(satisfactionTableData);
		/*int size = satisfactionTableData.size();
		Vector listTest = new Vector();
		satisfactionTableFiltredData.clear();
		SimpleTableModel satisfactionTableModel = (SimpleTableModel) satisfactionTable.getModel();
		satisfactionTableModel.clearTable();
		int nbLastPass = 0;
		for (int k=0; k < size ; k++){
			Vector statExecLine = (Vector) satisfactionTableData.elementAt(k);
			Vector data2 = new Vector();
			Test pTest =  (Test) statExecLine.elementAt(8);
			if (!listTest.contains(pTest)){
				listTest.add(pTest);
			}
			String famName = pTest.getTestListFromModel().getFamilyFromModel().getNameFromModel();
			String suiteName = pTest.getTestListFromModel().getNameFromModel();
			String testName = pTest.getNameFromModel();
			
			data2.add(testName + " (" + famName + "/" + suiteName + ")");
			Campaign pCamp = (Campaign) statExecLine.elementAt(0);
			Execution pExec = (Execution) statExecLine.elementAt(1);
			int nbPass = ((Integer) statExecLine.elementAt(3)).intValue();
			int nbFail = ((Integer) statExecLine.elementAt(4)).intValue();
			int nbUnknow = ((Integer) statExecLine.elementAt(5)).intValue();
			String lastResult = (String) statExecLine.elementAt(7);
			data2.add(pCamp.getNameFromModel() + "/" + pExec.getNameFromModel());
			data2.add("" + nbPass + "P, " +nbFail + "F, " +nbUnknow + "I");
			data2.add(lastResult);
			if (lastResult.equals(ApiConstants.SUCCESS)){
				nbLastPass++;
			}
			satisfactionTableModel.insertRow(k,data2);
			satisfactionTableFiltredData.add(k, statExecLine);
			//j++;
			//Set campagne, exec, env Filtres //remplir des hashtables
		}
		float pourcentSat = 0;
		if (size!=0){
			pourcentSat= nbLastPass*100/size;
		}
		
		coverTestRepresend.setText(Language.getInstance().getText("Test_couvert_represente") + ": 100%" ); 
		int nbTest = pStatRequirement.getAllTestCovredByCurrentReq().size();
		int nbTest2 = pStatRequirement.getEexecutedStatCovredByCurrentReq().size();
		float pourcentTest = 0;
		float pourcentTest2 = 0;
		if (nbTest!=0){
			pourcentTest = listTest.size()*100/nbTest;
		}
		if (nbTest2 != 0){
			pourcentTest2 = listTest.size()*100/nbTest2;
		}
		coverTestRepresend.setText(Language.getInstance().getText("Test_couvert_represente") + ": " + pourcentTest+"%"); 
		satisfaction.setText(Language.getInstance().getText("Satisfaction") + " : " + pourcentSat+"% / " + pourcentTest2 +"% " + Language.getInstance().getText("Execute"));
		*/
	}

	void loadSatisfactionTable(Vector satisfactionData){
		int size = satisfactionData.size();
		Vector listTest = new Vector();
		
		satisfactionTableFiltredData.clear();
		SimpleTableModel satisfactionTableModel = (SimpleTableModel) satisfactionTable.getModel();
		satisfactionTableModel.clearTable();
		
		int nbLastPass = 0;
		for (int k=0; k < size ; k++){
			Vector statExecLine = (Vector) satisfactionData.elementAt(k);
			Vector data2 = new Vector();
			
			
			Test pTest =  (Test) statExecLine.elementAt(8);
			if (!listTest.contains(pTest)){
				listTest.add(pTest);
			}
			String str_listOfReq = "";
			Hashtable hashTestReq = pStatRequirement.getAllReqCovredByTest();
			Vector listOfReq = (Vector) hashTestReq.get(new Integer(pTest.getIdBdd()));
			if (listOfReq != null && listOfReq.size()>0){
				str_listOfReq = ((Requirement)listOfReq.elementAt(0)).getNameFromModel();
				for (int l = 1 ; l < listOfReq.size(); l++){
					str_listOfReq += ", " + ((Requirement)listOfReq.elementAt(l)).getNameFromModel();
				}
			}
			data2.add(str_listOfReq);
			String famName = pTest.getTestListFromModel().getFamilyFromModel().getNameFromModel();
			String suiteName = pTest.getTestListFromModel().getNameFromModel();
			String testName = pTest.getNameFromModel();
			
			data2.add(testName + " (" + famName + "/" + suiteName + ")");
			Campaign pCamp = (Campaign) statExecLine.elementAt(0);
			Execution pExec = (Execution) statExecLine.elementAt(1);
			int nbPass = ((Integer) statExecLine.elementAt(3)).intValue();
			int nbFail = ((Integer) statExecLine.elementAt(4)).intValue();
			int nbUnknow = ((Integer) statExecLine.elementAt(5)).intValue();
			String lastResult = (String) statExecLine.elementAt(7);
			data2.add(pCamp.getNameFromModel() + "/" + pExec.getNameFromModel());
			data2.add("" + nbPass + "P, " +nbFail + "F, " +nbUnknow + "I");
			data2.add(lastResult);
			if (lastResult.equals(ApiConstants.SUCCESS)){
				nbLastPass++;
			}
			satisfactionTableModel.insertRow(k,data2);
			satisfactionTableFiltredData.add(k, statExecLine);
			//j++;
			//Set campagne, exec, env Filtres //remplir des hashtables
		}
		float pourcentSat = 0;
		if (size!=0){
			pourcentSat= nbLastPass*100/size;
		}
		
		coverTestRepresend.setText(Language.getInstance().getText("Test_couvert_represente") + ": 100%" ); 
		int nbTest = pStatRequirement.getAllTestCovredByCurrentReq().size();
		int nbTest2 = pStatRequirement.getEexecutedStatCovredByCurrentReq().size();
		float pourcentTest = 0;
		float pourcentTest2 = 0;
		if (nbTest!=0){
			pourcentTest = listTest.size()*100/nbTest;
		}
		if (nbTest2 != 0){
			pourcentTest2 = listTest.size()*100/nbTest2;
		}
		coverTestRepresend.setText(Language.getInstance().getText("Test_couvert_represente") + ": " + pourcentTest+"%"); 
		satisfaction.setText(Language.getInstance().getText("Satisfaction") + " : " + pourcentSat+"% / " + pourcentTest2 +"% " + Language.getInstance().getText("Execute"));
		
		Vector allTrackers = SalomeTMFContext.getInstance().getBugTracker();
		defectTabPanel.removeAll();
		int size2 = allTrackers.size();
		for (int k = 0 ; k < size2 ; k++){
			BugTracker pBugTracker =  (BugTracker)allTrackers.elementAt(k);
			JPanel pJPanel = pBugTracker.getDefectPanelForTests(listTest);
			if (pJPanel != null){
				defectTabPanel.addTab(pBugTracker.getBugTrackerName(), pJPanel);
			}
		}
	}
	
	void changeVersionPerformed(ActionEvent e){
		//if (canDoUpdatePrirotity){
			ReqLeaf pReqLeaf = null; 
			if (pRequirement instanceof ReqLeaf){
				try {
					pReqLeaf = (ReqLeaf)pRequirement;
					pReqLeaf.updateInfoInDBAndModel(versionField.getText().trim(), 
									((String)origineField.getSelectedItem()).trim(), 
									((String)verifmodeField.getSelectedItem()).trim(),
									((String)referenceField.getSelectedItem()).trim());
					pHistoryPanel.updateData(pReqLeaf, true);
				} catch (Exception ex){
					ex.printStackTrace();
					if ( pReqLeaf != null ){
						versionField.setText(((ReqLeaf)pRequirement).getVersionFromModel());
						setInfo(referenceField,referenceList, ((ReqLeaf)pRequirement).getReferenceFromModel());
						setInfo(origineField, origineList, ((ReqLeaf)pRequirement).getOrigineFromModel());
						setInfo(verifmodeField, verifList, ((ReqLeaf)pRequirement).getVerifFromModel());
						
					}
				}
			}
		//}
	}
	
	void stateBoxPerformed(ActionEvent e){
		if (canDoUpdateState){
			int indexSelected = stateBox.getSelectedIndex();
			int oldstate = -1;
			ReqLeaf pReqLeaf = null; 
			if (pRequirement instanceof ReqLeaf){
				try {
					pReqLeaf = (ReqLeaf)pRequirement;
					oldstate = pReqLeaf.getStateFromModel();
					if (oldstate == indexSelected){
						return;
					}
					pReqLeaf.updateStateInDBAndModel(indexSelected);
					pHistoryPanel.updateData(pReqLeaf, true);
				} catch (Exception ex){
					if ( pReqLeaf != null && oldstate != -1){
						pReqLeaf.updateStateInModel(oldstate);
					}
				}
			}
		}
	}
	
	void catBoxPerformed(ActionEvent e){
		if (canDoUpdateCat){
			int indexSelected = catBox.getSelectedIndex();
			int oldcat = -1;
			ReqLeaf pReqLeaf = null; 
			if (pRequirement instanceof ReqLeaf){
				try {
					pReqLeaf = (ReqLeaf)pRequirement;
					oldcat = pReqLeaf.getCatFromModel();
					if (oldcat == 	indexSelected){
						return;
					}
					pReqLeaf.updateCatInDBAndModel(indexSelected);
					pHistoryPanel.updateData(pReqLeaf, true);
				} catch (Exception ex){
					if ( pReqLeaf != null && oldcat != -1){
						pReqLeaf.updateCatInModel(oldcat);
					}
				}
			}
		}
	}
	void  complexeBoxPerformed(ActionEvent e) {
		if (canDoUpdateComplexe){
			int indexSelected = complexeBox.getSelectedIndex();
			int complexeSelected = 100;
			if (indexSelected == 0){
				complexeSelected = ReqFilter.C0;
			} else if (indexSelected == 1){
				complexeSelected = ReqFilter.C1;
			} else if (indexSelected == 2){
				complexeSelected = ReqFilter.C2;
			}else if (indexSelected == 3){
				complexeSelected = ReqFilter.C3;
			}
			int oldComplexe = -1;
			ReqLeaf pReqLeaf = null; 
			if (pRequirement instanceof ReqLeaf){
				try {
					pReqLeaf = (ReqLeaf)pRequirement;
					oldComplexe = pReqLeaf.getComplexeFromModel();
					if (oldComplexe == 	complexeSelected){
						return;
					}
					pReqLeaf.updateComplexeInDBAndModel(complexeSelected);
					pHistoryPanel.updateData(pReqLeaf, true);
				} catch (Exception ex){
					if ( pReqLeaf != null && oldComplexe != -1){
						pReqLeaf.updateComplexeInModel(oldComplexe);
					}
				}
			}
		}
	}
	
	void priorityBoxPerformed(ActionEvent e) {
		if (canDoUpdatePrirotity){
			int indexSelected = priorityBox.getSelectedIndex();
			int priroritySelected = 1000;
			if (indexSelected == 0){
				priroritySelected = 1000;
			} else if (indexSelected == 1){
				priroritySelected = 100;
			} else if (indexSelected == 2){
				priroritySelected = 10;
			} else if (indexSelected == 3){
				priroritySelected = 1;
			}
			int oldPriority = -1;
			ReqLeaf pReqLeaf = null; 
			if (pRequirement instanceof ReqLeaf){
				try {
					pReqLeaf = (ReqLeaf)pRequirement;
					oldPriority = pReqLeaf.getPriorityFromModel();
					if (oldPriority == priroritySelected){
						return;
					}
					pReqLeaf.updatePriorityInDBAndModel(priroritySelected);
					pHistoryPanel.updateData(pReqLeaf, true);
				} catch (Exception ex){
					if ( pReqLeaf != null && oldPriority != -1){
						pReqLeaf.updatePriorityInModel(oldPriority);
					}
				}
			}
		}
	}
	
	void viewTestPerformed(ActionEvent e, JTable pTable){
		int selectedRow = pTable.getSelectedRow();
		if (selectedRow >-1){
			if (testCover != null) {
				Test pTest = (Test)testCover.elementAt(selectedRow);
				if (pDynamicTree != null){
					DefaultMutableTreeNode node = pDynamicTree.findRemoveTestNode(pTest.getNameFromModel(), 
							pTest.getTestListFromModel().getNameFromModel(), pTest.getTestListFromModel().getFamilyFromModel().getNameFromModel(), false);
					if (node != null){
						//pReqTree.refreshNode(node);
						ReqPlugin.selectTestTab();
						pDynamicTree.getTree().setSelectionPath(new TreePath(node.getPath()));
						
					}
				}
			}
		}
	}

	void viewCampPerformed(ActionEvent e, JTable pTable){
		int selectedRow = pTable.getSelectedRow();
		if (selectedRow >-1){
			if (testCover != null) {
				Test pTest = (Test)testCover.elementAt(selectedRow);
				if (DataModel.isTestExecuted(pTest)){
					new ViewCampaignOfTest(pTest);
				}
			}
		}
	}
	
	public void valueChanged(ListSelectionEvent e) {
        //Ignore extra messages.
        if (e.getValueIsAdjusting()) {
        	return;
        }
        
        ListSelectionModel lsm = (ListSelectionModel)e.getSource();
        if (lsm.isSelectionEmpty()) {
            //no rows are selected
        	
        } else {
            //int selectedRow = lsm.getMinSelectionIndex();
            //selectedRow is selected
        }
	}
	
	/********************* ComboField from info ************************/
	public void setColumnSize(JTable table){   
	       FontMetrics fm = table.getFontMetrics(table.getFont());
	       for (int i = 0 ; i < table.getColumnCount() ; i++)
	       {
	           int max = 0;
	           for (int j = 0 ; j < table.getRowCount() ; j++)
	           {
	              int taille = fm.stringWidth((String)table.getValueAt(j,i));
	              if (taille > max)
	                   max = taille;
	           }
	           String nom = (String)table.getColumnModel().getColumn(i).getIdentifier();
	           int taille = fm.stringWidth(nom);
	           if (taille > max)
	                  max = taille;
	          table.getColumnModel().getColumn(i).setPreferredWidth(max+10);
	       }
	   }
	
	void setInfo(JComboBox pJcomboBox, Vector pVector, String value){
		if (value == null){
			value = "";
		}
		pJcomboBox.removeAllItems();
		int size = pVector.size();
		int selectedItem = -1;
		for (int i = 0 ; i < size ; i++){
			String item = (String) pVector.elementAt(i);
			pJcomboBox.addItem(item);
			if (value.equals(item)){
				selectedItem = i;
			}
		}
		if (selectedItem == -1){
			pJcomboBox.addItem(value);
			pJcomboBox.setSelectedIndex(size);
		} else {
			pJcomboBox.setSelectedIndex(selectedItem);
		}
	}
}
