/*
* SalomeTMF is a Test Management Framework
* Copyright (C) 2005 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* @author Marche Mikael
*
* Contact: mikael.marche@rd.francetelecom.com
*/

package org.objectweb.salome_tmf.databaseSQL;

import java.sql.Date;
import java.sql.PreparedStatement;

import org.objectweb.salome_tmf.api.Api;
import org.objectweb.salome_tmf.api.ApiConstants;
import org.objectweb.salome_tmf.api.Util;
import org.objectweb.salome_tmf.api.sql.ISQLUrlAttachment;

public class SQLUrlAttachment extends SQLAttachment implements ISQLUrlAttachment{

	/**
	 * Insert an Url attchement in table ATTACHEMENT
	 * @param url
	 * @param description
	 * @return the id of the attachment
	 * @throws Exception
	 * no permission needed
	 */
	public int insert(String url, String description) throws Exception  {
		int idAttach = -1;
		int transNumber = -1;
		try {	
			transNumber = SQLEngine.beginTransaction(0, ApiConstants.INSERT_ATTACHMENT);
			
			Date pDate = Util.getCurrentDate();
			
			PreparedStatement prep = SQLEngine.getSQLAddQuery("insertUrlAttachIntoDB"); //ok
			prep.setString(1,url);
			prep.setString(2,description);
			prep.setDate(3,pDate);
			SQLEngine.runAddQuery(prep);
			
			idAttach = getLastIdAttach();
			SQLEngine.commitTrans(transNumber);
		} catch (Exception e){
			Util.log("[SQLUrlAttachment->insert]" + e);
			if (Api.isDEBUG()){
				e.printStackTrace();
			}
			SQLEngine.rollBackTrans(transNumber);
			throw e;
		}
		return idAttach;
	}
}
