/*
* SalomeTMF is a Test Management Framework
* Copyright (C) 2005 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* @author Fayçal SOUGRATI, Vincent Pautret, Marche Mikael
*
* Contact: mikael.marche@rd.francetelecom.com
*/

package org.objectweb.salome_tmf.ihm.main;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.objectweb.salome_tmf.api.Api;
import org.objectweb.salome_tmf.api.ApiConstants;
import org.objectweb.salome_tmf.api.Util;
import org.objectweb.salome_tmf.data.DataConstants;
import org.objectweb.salome_tmf.data.ExecutionResult;
import org.objectweb.salome_tmf.data.ExecutionTestResult;
import org.objectweb.salome_tmf.data.ManualTest;
import org.objectweb.salome_tmf.data.Test;
import org.objectweb.salome_tmf.ihm.IHMConstants;
import org.objectweb.salome_tmf.ihm.languages.Language;
import org.objectweb.salome_tmf.ihm.main.datawrapper.DataModel;
import org.objectweb.salome_tmf.ihm.main.plugins.PluginsTools;
import org.objectweb.salome_tmf.ihm.models.ActionDetailsResultMouseListener;
import org.objectweb.salome_tmf.ihm.models.AutomaticTestResultMouseListener;
import org.objectweb.salome_tmf.ihm.models.MyTableModel;
import org.objectweb.salome_tmf.ihm.models.TableSorter;
import org.objectweb.salome_tmf.ihm.tools.Tools;
import org.objectweb.salome_tmf.plugins.UICompCst;

/**
 * Classe repr?sentant la vue sur les ex?cutions de test
 * @author teaml039
 * @version 0.1
 */
public class ExecutionResultView extends JDialog implements DataConstants,IHMConstants {
    
    /**
     * Mod?le de donn?es de la table
     */
    MyTableModel testResultTableModel;
    
    /**
     * Table des r?sultats sur les tests
     */
    JTable testResultTable;
    TableSorter  sorter;
    
    ListSelectionModel rowSM;
    
    
    ExecutionResult currentExecResult;
    
    JButton detailsButton;
    
    /******************************************************************************/
    /** 							CONSTRUCTEUR								***/
    /******************************************************************************/
    
    
    /**
     * Constructeur de la vue sur les ex?cutions de tests
     * @param name le label d'information sur l'ex?cution
     */
    public ExecutionResultView(String name, ExecutionResult executionResult) {
        
        super(SalomeTMFContext.getInstance().ptrFrame, true);
        testResultTableModel = new MyTableModel();
        testResultTable = new JTable();
        JLabel executionName = new JLabel(name);
        executionName.setFont(new Font(null, Font.BOLD,18));
        
        
        
        testResultTableModel.addColumnNameAndColumn(Language.getInstance().getText("Famille"));
        testResultTableModel.addColumnNameAndColumn(Language.getInstance().getText("Suite"));
        testResultTableModel.addColumnNameAndColumn(Language.getInstance().getText("Test"));
        testResultTableModel.addColumnNameAndColumn(Language.getInstance().getText("Resultats"));
        
        sorter = new TableSorter(testResultTableModel);
        testResultTable.setModel(sorter);
        //testResultTable.setModel(testResultTableModel);
        
        sorter.setTableHeader(testResultTable.getTableHeader());
        
        testResultTable.setPreferredScrollableViewportSize(new Dimension(600, 200));
        testResultTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        
        // Mapping entre objets graphiques et constantes
        SalomeTMFContext.getInstance().addToUIComponentsMap(UICompCst.EXECUTION_RESULT_DETAILS_TESTS_TABLE,testResultTable);
        // Activation des plugins associ?s
        PluginsTools.activateAssociatedPlgs(UICompCst.EXECUTION_RESULT_DETAILS_TESTS_TABLE);
        
        JScrollPane tablePane = new JScrollPane(testResultTable);
        tablePane.setBorder(BorderFactory.createRaisedBevelBorder());
        
        
        rowSM = testResultTable.getSelectionModel();
        rowSM.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (e.getValueIsAdjusting())
                    return;
                
                int selectedRowIndex = testResultTable.getSelectedRow();
                if (selectedRowIndex != -1 && testResultTableModel.getRowCount() > 0) {
                    Test test = DataModel.getCurrentProject().getTestFromModel(
                    (String)sorter.getValueAt(selectedRowIndex, 0),
                    (String)sorter.getValueAt(selectedRowIndex, 1),
                    (String)sorter.getValueAt(selectedRowIndex, 2)
                    );
                    DataModel.setTestObservedInExecution(test);
                    DataModel.setCurrentExecutionTestResult(currentExecResult.getExecutionTestResultFromModel(test));
                    detailsButton.setEnabled(true);
                } else {
                    detailsButton.setEnabled(false);
                }
            }
        });
        
        
        // Gestion de la souris
        testResultTable.addMouseListener(new ActionDetailsResultMouseListener());
        testResultTable.addMouseListener(new AutomaticTestResultMouseListener());
        
        JButton quitButton = new JButton(Language.getInstance().getText("Terminer"));
        quitButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                ExecutionResultView.this.dispose();
            }
        });
        
        
        detailsButton = new JButton(Language.getInstance().getText("Details"));
        detailsButton.setEnabled(false);
        detailsButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
            	boolean attachUpdate = false;
            	boolean automatic = false;
            	ExecutionTestResult pExecutionTestResult = DataModel.getCurrentExecutionTestResult();
            	HashMap oldAttachMap = pExecutionTestResult.getCopyOfAttachmentMapFromModel();
                
            	if (DataModel.getTestObservedInExecution() instanceof ManualTest) {
            		Util.log("[ExecutionResultView->detailsButton] -> view manual test result");
            		new ActionDetailsView((ManualTest)DataModel.getTestObservedInExecution(), DataModel.getObservedExecution(), DataModel.getObservedExecutionResult(), pExecutionTestResult, ExecutionResultView.this);
            		//attachUpdate = false;
                } else {
                    //HashMap oldAttachMap = (HashMap)DataModel.getCurrentExecutionTestResult().getAttachmentMapFromModel().clone();
                    //HashMap oldAttachMap = DataModel.getCurrentExecutionTestResult().getCopyOfAttachmentMapFromModel();
                	Util.log("[ExecutionResultView->detailsButton] -> view automatic test result");
                	//AttachmentViewWindow attachWindow = new AttachmentViewWindow(SalomeTMF.ptrSalomeTMF, EXECUTION_RESULT_TEST, DataModel.getCurrentExecutionTestResult(), DataModel.getObservedExecution(), DataModel.getObservedExecutionResult(), DataModel.getTestObservedInExecution());
                	AttachmentViewWindow attachWindow = new AttachmentViewWindow(SalomeTMFContext.getInstance().getBaseIHM(), EXECUTION_RESULT_TEST, DataModel.getCurrentExecutionTestResult());
                	automatic = true;
                	if (!attachWindow.cancelPerformed){
                    	attachUpdate = true;
                    }
                }
                if (attachUpdate && automatic){
                    int transNumber = -1;
                    Util.log("[ExecutionResultView->detailsButton] -> valide Perfomed");
                    try {
                    	Util.log("[ExecutionResultView] Api.INSERT_ATTACHMENT");
                        transNumber = Api.beginTransaction(10, ApiConstants.INSERT_ATTACHMENT);
                        
                        pExecutionTestResult.updateAttachement(oldAttachMap);
                        
                        Api.commitTrans(transNumber);
                        
                    } catch (Exception exception) {
                    	Api.forceRollBackTrans(transNumber);
                        DataModel.getCurrentExecutionTestResult().setAttachmentMapInModel(oldAttachMap);
                        exception.printStackTrace();
                        Tools.ihmExceptionView(exception);
                    }
                   } else if (automatic){
                	   Util.log("[ExecutionResultView->detailsButton] -> cancel Perfomed");
                	   DataModel.getCurrentExecutionTestResult().setAttachmentMapInModel(oldAttachMap);
                   }
                    
                }
                
            //}
        });
        
        JPanel upPart = new JPanel(new BorderLayout());
        upPart.add(executionName, BorderLayout.NORTH);
        
        JPanel allView = new JPanel(new BorderLayout());
        
        allView.add(upPart, BorderLayout.NORTH);
        allView.add(tablePane, BorderLayout.CENTER);
        
        JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
        buttonPanel.add(detailsButton);
        buttonPanel.add(quitButton);
        
        
        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
        panel.add(allView);
        panel.add(buttonPanel);
        
        
        
        initTestResults(executionResult);
        
        Container contentPaneFrame = this.getContentPane();
        contentPaneFrame.add(panel, BorderLayout.CENTER);
        //this.setLocation(400,300);
        this.setTitle(Language.getInstance().getText("Resultats_d_execution"));
        /*this.pack();
        this.setLocationRelativeTo(this.getParent()); 
        this.setVisible(true);
        */
        centerScreen();
    } // Fin du constructeur ExecutionResult/1
    
    void centerScreen() {
		  Dimension dim = getToolkit().getScreenSize();
		  this.pack();
		  Rectangle abounds = getBounds();
		  setLocation((dim.width - abounds.width) / 2,
		      (dim.height - abounds.height) / 2);	 
	      this.setVisible(true); 
		  requestFocus();
	}
    /**
     *
     * @param executionResult
     */
    public void initTestResults(ExecutionResult executionResult) {
        
        Test[] pTabTest =  executionResult.getTestOrderedFromModel();
        for (int j = 0; j < pTabTest.length ; j++){
            Test test = pTabTest[j];
            ArrayList data = new ArrayList();
            data.add(test.getTestListFromModel().getFamilyFromModel().toString());
            data.add(test.getTestListFromModel().toString());
            data.add(test.toString());
            ImageIcon image = Tools.getActionStatusIcon(executionResult.getTestResultStatusFromModel(test));
            if (image == null) {
                data.add(Tools.createAppletImageIcon(PATH_TO_GOON_ICON, ""));
            } else {
                data.add(Tools.getActionStatusIcon(executionResult.getTestResultStatusFromModel(test)));
                
                
            }
            currentExecResult = executionResult;
            testResultTableModel.addRow(data);
            
        }
    } // Fin de la m?thode initTestResults/1
    
} // Fin de la classe ExecutionResult
