/*
 * SalomeTMF is a Test Management Framework
 * Copyright (C) 2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * @author Fayçal SOUGRATI, Vincent Pautret, Marche Mikael
 *
 * Contact: mikael.marche@rd.francetelecom.com
 */

package org.objectweb.salome_tmf.ihm.main;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.objectweb.salome_tmf.api.Api;
import org.objectweb.salome_tmf.api.ApiConstants;
import org.objectweb.salome_tmf.api.Permission;
import org.objectweb.salome_tmf.data.Action;
import org.objectweb.salome_tmf.data.Attachment;
import org.objectweb.salome_tmf.data.ManualTest;
import org.objectweb.salome_tmf.data.Parameter;
import org.objectweb.salome_tmf.data.Test;
import org.objectweb.salome_tmf.ihm.languages.Language;
import org.objectweb.salome_tmf.ihm.main.datawrapper.DataModel;
import org.objectweb.salome_tmf.ihm.models.RowTransferHandler;
import org.objectweb.salome_tmf.ihm.models.RowTransferable;
import org.objectweb.salome_tmf.ihm.tools.Tools;
import org.objectweb.salome_tmf.plugins.UICompCst;


public class ManualActionView extends JPanel implements ActionListener, ListSelectionListener, PropertyChangeListener {
	
	JPanel actionsPanel;
	
	/**
	 * Bouton pour la suppression des actions pour les tests manuels
	 */
	static JButton deleteManualTestAction;
	
	/**
	 * Table des actions des tests manuels
	 */
	JTable actionTable;
	
	static JButton modifyAction;
	static JButton addAction;
	RowTransferHandler transferHandler;
	
	static JButton paste;
	static JButton cut;
	static JButton copy;
	static JButton orderAction;
	
	//JTabbedPane m_ParentTab;
	
	public ManualActionView()  {
		actionsPanel = new JPanel();
		
		deleteManualTestAction = new JButton(Language.getInstance().getText("Supprimer"));
		actionTable = new JTable();
		modifyAction = new JButton(Language.getInstance().getText("Modifier"));
		copy = new JButton(Language.getInstance().getText("Copier"));
		paste = new JButton(Language.getInstance().getText("Coller"));
		cut = new JButton(Language.getInstance().getText("Couper"));
		
		addAction = new JButton(Language.getInstance().getText("Ajouter"));
		addAction.setToolTipText(Language.getInstance().getText("Ajouter_une_action"));
		addAction.addActionListener(this); 
		
		modifyAction.setToolTipText(Language.getInstance().getText("Modifier_une_action"));
		modifyAction.setEnabled(false);
		modifyAction.addActionListener(this);
		
		orderAction = new JButton(Language.getInstance().getText("Ordonner"));
		orderAction.setToolTipText(Language.getInstance().getText("Ordonner_les_actions"));
		orderAction.addActionListener(this);
		
		deleteManualTestAction.setEnabled(false);
		deleteManualTestAction.setToolTipText(Language.getInstance().getText("Supprimer_une_action"));
		deleteManualTestAction.addActionListener(this);
		
		copy.setToolTipText(Language.getInstance().getText("Copier"));
		copy.setEnabled(false);
		copy.addActionListener(this);
		
		cut.setToolTipText(Language.getInstance().getText("Couper"));
		cut.setEnabled(false);
		cut.addActionListener(this);
		
		paste.setToolTipText(Language.getInstance().getText("Coller"));
		paste.setEnabled(false);
		paste.addActionListener(this);
		
		JPanel buttonsPanel = new JPanel(new GridLayout(1,7));
		buttonsPanel.add(addAction);
		buttonsPanel.add(modifyAction);
		buttonsPanel.add(orderAction);
		buttonsPanel.add(deleteManualTestAction);
		buttonsPanel.add(copy);
		buttonsPanel.add(cut);
		buttonsPanel.add(paste);
		buttonsPanel.setBorder(BorderFactory.createRaisedBevelBorder());
		
		transferHandler = new RowTransferHandler();
		actionTable.setModel(DataModel.getActionTableModel());
		
		//TableSorter sorter = new TableSorter(actionTable.getModel() , actionTable.getTableHeader());
		//actionTable.setModel(sorter);
		
		actionTable.setPreferredScrollableViewportSize(new Dimension(600, 200));
		actionTable.setTransferHandler(transferHandler);
		actionTable.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		actionTable.addPropertyChangeListener(this);
		JScrollPane tablePane = new JScrollPane(actionTable);
		
		ListSelectionModel rowSM = actionTable.getSelectionModel();
		rowSM.addListSelectionListener(this);
		
		
		actionsPanel.setLayout(new BorderLayout());
		actionsPanel.add(buttonsPanel, BorderLayout.NORTH);
		actionsPanel.add(tablePane, BorderLayout.CENTER);
		

		this.setLayout(new BorderLayout());
		this.add(actionsPanel);
		
		// Mapping GUI Object for plugins//
		// actionsPanel
		SalomeTMFContext.getInstance().addToUIComponentsMap(UICompCst.MANUAL_TEST_ACTIONS_PANEL,actionsPanel);
		UICompCst.staticUIComps.add(UICompCst.MANUAL_TEST_ACTIONS_PANEL);
		
		// actionTable
		SalomeTMFContext.getInstance().addToUIComponentsMap(UICompCst.MANUAL_TEST_ACTIONS_TABLE,actionTable);
		UICompCst.staticUIComps.add(UICompCst.MANUAL_TEST_ACTIONS_TABLE);
		
		//buttonsPanel
		SalomeTMFContext.getInstance().addToUIComponentsMap(UICompCst.MANUAL_TEST_ACTIONS_BUTTONS_PANEL,buttonsPanel);
		UICompCst.staticUIComps.add(UICompCst.MANUAL_TEST_ACTIONS_BUTTONS_PANEL);
	} // Fin du constructeur ManualActionView/0
	
	
	public void propertyChange (PropertyChangeEvent e){
		if (e.getOldValue() == null){
			//System.out.println("EVENT ------> = " + e.getPropertyName() + "[" +   e.getOldValue() +"->"+  e.getNewValue() +"]");
			setColumnSize();
		}
	}
	public void setColumnSize(){   
		JTable table = actionTable;
	       FontMetrics fm = table.getFontMetrics(table.getFont());
	       for (int i = 0 ; i < table.getColumnCount() ; i++)
	       {
	           int max = 0;
	           for (int j = 0 ; j < table.getRowCount() ; j++)
	           {
	        	   Object obj  = table.getValueAt(j,i);
	        	   String valueStr = null;
	        	   if ( obj instanceof String ) {
	        		   valueStr = (String)table.getValueAt(j,i);
	        	   } else if (obj != null){
	        		   valueStr = obj.toString();
	        	   }
	        	  if (valueStr != null) {
	        		  int taille = fm.stringWidth(valueStr);
	        		  if (taille > max)
	                   max = taille;
	        	  }
	           }
	           String nom = (String)table.getColumnModel().getColumn(i).getIdentifier();
	           int taille = fm.stringWidth(nom);
	           if (taille > max)
	                  max = taille;
	          table.getColumnModel().getColumn(i).setPreferredWidth(max+10);
	       }
	       //table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
	   }
	
	
	/*public void setParent(JTabbedPane _ParentTab){
		if (m_ParentTab != null){
			return;
		} else {
			m_ParentTab = _ParentTab;
			m_ParentTab.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					if (((JTabbedPane)e.getSource()).getSelectedComponent().equals(ManualActionView.this)){
						
					}
				}
			});
		}
	}
	*/
	/**
	 *
	 *
	 */
	public static  void giveAccessToIhmManualActionView() {
		if (!Permission.canDeleteTest()) {
			deleteManualTestAction.setEnabled(false);
			cut.setEnabled(false);
		}
		if (!Permission.canCreateTest()) {
			addAction.setEnabled(false);
			copy.setEnabled(false);
			paste.setEnabled(false);
		}
		if (!Permission.canUpdateTest()) {
			modifyAction.setEnabled(false);
			orderAction.setEnabled(false);
		}
			
	}
	
	/**
	 * Mise a jour des numero d'ordre des tests d'une campagne
	 * @param campaign une campagne
	 */
	private static void updateIhmActionOrder(ManualTest test) {
		ArrayList actionList = test.getActionListFromModel(false);
		for (int i = 0; i < actionList.size(); i++) {
			Action action = (Action)actionList.get(i); 
			action.setOrderIndex(i);
		}
		
	} // Fin de la m?thode updateTestOrderInCampaignTree/1
	
	/********************** Manage events *************************/
	
	public void actionPerformed(ActionEvent e){
		if (e.getSource().equals(addAction)){
			addActionPerformed();
			setColumnSize();
		} else if (e.getSource().equals(modifyAction)){
			modifyActionPerformed();
			setColumnSize();
		} else if (e.getSource().equals(orderAction)){
			orderActionPerformed(); 
			setColumnSize();
		} else if (e.getSource().equals(deleteManualTestAction)){
			deleteManualTestActionPerformed();
			setColumnSize();
		} else if (e.getSource().equals(copy)) {
			copyPerformed() ;
		} else if (e.getSource().equals(cut)){
			cutPerformed();
		} else if (e.getSource().equals(paste)){
			pastePerformed();
			setColumnSize();
		}
	}
	
	
	public void addActionPerformed() {
		String message = DataModel.getCurrentProject().getCampaignWithExecResultWhereTestIsUse(DataModel.getCurrentTest());
		if ((!message.equals(""))&& (Api.isLockExecutedTest())) {
			JOptionPane
			.showMessageDialog(
					ManualActionView.this,
					Language
					.getInstance()
					.getText(
					"Ce_test_est_utilise_dans_les_campagnes__")
					+ message
					+ Language
					.getInstance()
					.getText(
					"qui_possedent_des_resultats_d_executions_Afin_de_conserver_l_integrite_des_resultats_vous_ne_pouvez_pas_ajouter_une_nouvelle_action"),
					Language.getInstance().getText("Erreur_"),
					JOptionPane.ERROR_MESSAGE);
		} else {
			if ((!message.equals(""))&& (!Api.isLockExecutedTest())) { 
				Object[] options = {Language.getInstance().getText("Oui"), Language.getInstance().getText("Non")};
				int choice = SalomeTMFContext.getInstance().askQuestion(
						Language.getInstance().getText("Ce_test_est_utilise_dans_les_campagnes__") + message +  Language.getInstance().getText("IntegiteCheck"),
						Language.getInstance().getText("Attention_"),
						JOptionPane.WARNING_MESSAGE,
						options);
				if (choice == JOptionPane.NO_OPTION) {
					return;
				}
			}
			AskNewAction addAction = new AskNewAction(ManualActionView.this,
					Language.getInstance().getText("Ajout_d_une_action"));
			Action action = addAction.getAction();
			if (action != null) {
				if (!((ManualTest) DataModel.getCurrentTest()).hasActionNameInModel(action.getNameFromModel())) {
					int transNumber = -1;
					try {
						transNumber = Api.beginTransaction(101,ApiConstants.INSERT_ACTION);
						//action.setOrderIndex(DataModel.getActionTableModel()
						//		.getRowCount());
						((ManualTest) DataModel.getCurrentTest()).addActionInDBAndModel(action);
						
						Set keySet = action.getAttachmentMapFromModel().keySet();
						for (Iterator iter = keySet.iterator(); iter.hasNext();) {
							Attachment attach = (Attachment) action.getAttachmentMapFromModel().get(iter.next());
							action.addAttachementInDB(attach);
						}
						HashSet pHashSet = new HashSet(action.getParameterHashSetFromModel().values()); //FOR HashTable2HashSet
						for (Iterator iterator = pHashSet.iterator(); iterator.hasNext();) {
							Parameter param = (Parameter) iterator.next();
							action.setUseParamInDBAndModel(param);
						}
						Api.commitTrans(transNumber);
						transNumber = -1;
						ArrayList dataList = new ArrayList();
						dataList.add(action.getNameFromModel());
						dataList.add(action.getDescriptionFromModel());
						dataList.add(action.getAwaitedResultFromModel());
						dataList.add(action.getAttachmentMapFromModel().keySet());
						DataModel.getActionTableModel().addRow(dataList);
						//action.setOrderIndex(DataModel.getActionTableModel().getRowCount()-1);
						//((ManualTest) DataModel.getCurrentTest())
						//		.addAction(action);
						
					} catch (Exception exception) {
						Api.forceRollBackTrans(transNumber);
						((ManualTest) DataModel.getCurrentTest()).deleteActionInModel(action);
						Tools.ihmExceptionView(exception);
					}
					
				} else {
					JOptionPane
					.showMessageDialog(
							ManualActionView.this,
							Language
							.getInstance()
							.getText(
							"Ce_nom_d_action_existe_deja_pour_ce_test_"),
							Language.getInstance().getText("Erreur_"),
							JOptionPane.ERROR_MESSAGE);
				}
			}
		}
		//setColumnSize(actionTable);	
	}
	
	
	public void modifyActionPerformed() {
		int selectedRowIndex = actionTable.getSelectedRow();
		if (selectedRowIndex != -1) {
			String message = DataModel.getCurrentProject().getCampaignWithExecResultWhereTestIsUse(DataModel.getCurrentTest());
			if ((!message.equals(""))&& (Api.isLockExecutedTest())) {
				JOptionPane.showMessageDialog(ManualActionView.this,
						Language.getInstance().getText("Ce_test_est_utilise_dans_les_campagnes__") + message + Language.getInstance().getText("qui_possedent_des_resultats_d_executions_Afin_de_conserver_l_integrite_des_resultats_vous_ne_pouvez_modifier_que_les_attachements_lies_cette_action"),
						Language.getInstance().getText("Avertissement_"),
						JOptionPane.WARNING_MESSAGE);
				Action oldAction = ((ManualTest)DataModel.getCurrentTest()).getActionFromModel((String)DataModel.getActionTableModel().getValueAt(selectedRowIndex, 0));
				HashMap oldAttachMap = oldAction.getCopyOfAttachmentMapFromModel();
				String oldName = oldAction.getNameFromModel();
				
				AskNewAction addAction = new AskNewAction(ManualActionView.this, Language.getInstance().getText("Modifier_une_action"), oldAction);
				
				if (addAction.getAction() != null) {
					int transNumber = -1;
					try {
						// BdD
						transNumber = Api.beginTransaction(100, ApiConstants.UPDATE_ACTION);
						//ConnectionData.getSuiteTestUpdate().updateAction(DataModel.getCurrentTest().getTestList().getFamily().getName(), DataModel.getCurrentTest().getTestList().getName(), DataModel.getCurrentTest().getName(), oldActionName, addAction.getAction().getName(), addAction.getAction().getDescription(), addAction.getAction().getAwaitedResult(), addAction.getAction().getOrderIndex());
						Action pAction = addAction.getAction();
						String newName = pAction.getNameFromModel();
						pAction.updateInModel(oldName, pAction.getDescriptionFromModel(), pAction.getAwaitedResultFromModel());
						addAction.getAction().updateInDBAndModel(newName, pAction.getDescriptionFromModel(), pAction.getAwaitedResultFromModel());
						
						// Liste des anciens fichiers attach?s
						addAction.getAction().updateAttachement(oldAttachMap);
						
						Api.commitTrans(transNumber);
						transNumber = -1;
						// IHM
						ArrayList dataList = new ArrayList();
						dataList.add(addAction.getAction().getNameFromModel());
						dataList.add(addAction.getAction().getDescriptionFromModel());
						dataList.add(addAction.getAction().getAwaitedResultFromModel());
						//HashMap newMap = addAction.getAction().getAttachmentMap();
						dataList.add(addAction.getAction().getAttachmentMapFromModel().keySet());
						DataModel.getActionTableModel().modifyData(dataList,selectedRowIndex);
					} catch (Exception exception) {
						Api.forceRollBackTrans(transNumber);
						addAction.getAction().setAttachmentMapInModel(oldAttachMap);
						Tools.ihmExceptionView(exception);
					}
					
				} else {
					oldAction.setAttachmentMapInModel(oldAttachMap);
				}
			} else {
				if ((!message.equals(""))&& (!Api.isLockExecutedTest())) { 
    				Object[] options = {Language.getInstance().getText("Oui"), Language.getInstance().getText("Non")};
    				int choice = SalomeTMFContext.getInstance().askQuestion(
    						Language.getInstance().getText("Ce_test_est_utilise_dans_les_campagnes__") + message +  Language.getInstance().getText("IntegiteCheck"),
    						Language.getInstance().getText("Attention_"),
    						JOptionPane.WARNING_MESSAGE,
    						options);
    				if (choice == JOptionPane.NO_OPTION) {
    					return;
    				}
    			}
				if (DataModel.getCurrentTest() instanceof ManualTest) {
					
					Action oldAction = ((ManualTest)DataModel.getCurrentTest()).getActionFromModel((String)DataModel.getActionTableModel().getValueAt(selectedRowIndex, 0));
					String oldActionName = oldAction.getNameFromModel();
					String oldDescription = oldAction.getDescriptionFromModel();
					String oldAwaitedResult = oldAction.getAwaitedResultFromModel();
					
					HashMap oldAttachMap = oldAction.getCopyOfAttachmentMapFromModel();
					Hashtable oldParameter = oldAction.getCopyOfParameterHashTableFromModel();
					
					HashSet oldParamSet = new HashSet(oldAction.getParameterHashSetFromModel().values());//FOR HashTable2HashSet
					
					AskNewAction addAction = new AskNewAction(ManualActionView.this, Language.getInstance().getText("Modifier_une_action"), oldAction);
					int transNumber = -1;
					
					if (addAction.getAction() != null) {
						Action pAction = addAction.getAction();
						try {
							// BdD
							transNumber = Api.beginTransaction(101, ApiConstants.UPDATE_ACTION);
							pAction.updateInDBAndModel(pAction.getNameFromModel(), pAction.getDescriptionFromModel(), pAction.getAwaitedResultFromModel());
							
							/* Les attachement */
							pAction.updateAttachement(oldAttachMap);
							
							
							/* Les Paramtres */
							Hashtable newParameter = pAction.getParameterHashSetFromModel();
							Set keysSet = newParameter.keySet();
							for (Iterator iter = keysSet.iterator(); iter.hasNext();) {
								Object paramName =  iter.next();
								Parameter param = (Parameter) oldParameter.get(paramName);
								Parameter pParam = (Parameter)newParameter.get(paramName);
								if (param == null){		
									pAction.setUseParamInDB(pParam.getIdBdd());
								} else {
									/* Update */
								}
							}
							
							keysSet = oldParameter.keySet();
							for (Iterator iter = keysSet.iterator(); iter.hasNext();) {
								Object paramName =  iter.next();
								Parameter param = (Parameter) newParameter.get(paramName);
								Parameter pParam = (Parameter)oldParameter.get(paramName);
								if (param == null) {
									pAction.deleteUseParamInDB(pParam.getIdBdd());
								}
							}
							
							
							Api.commitTrans(transNumber);
							
							// IHM
							ArrayList dataList = new ArrayList();
							dataList.add(addAction.getAction().getNameFromModel());
							dataList.add(addAction.getAction().getDescriptionFromModel());
							dataList.add(addAction.getAction().getAwaitedResultFromModel());
							//HashMap newMap = addAction.getAction().getAttachmentMap();
							dataList.add(addAction.getAction().getAttachmentMapFromModel().keySet());
							DataModel.getActionTableModel().modifyData(dataList,selectedRowIndex);
							
						} catch (Exception exception) {
							Api.forceRollBackTrans(transNumber);
							addAction.getAction().updateInModel(oldActionName, oldDescription,oldAwaitedResult);
							addAction.getAction().setAttachmentMapInModel(oldAttachMap);
							addAction.getAction().setParameterHashSetInModel(oldParamSet);
							Tools.ihmExceptionView(exception);
						}
					} else {
						// Annuler
						oldAction.setAttachmentMapInModel(oldAttachMap);
						oldAction.setParameterHashtableInModel(oldParameter);
					}
					
				}
			}
		}
			
	}
	
	
	
	public void orderActionPerformed() {
		String message = DataModel.getCurrentProject().getCampaignWithExecResultWhereTestIsUse(DataModel.getCurrentTest());
		if ((!message.equals(""))&& (Api.isLockExecutedTest())) {
			JOptionPane.showMessageDialog(ManualActionView.this,
					Language.getInstance().getText("Ce_test_est_utilise_dans_les_campagnes__") + message + Language.getInstance().getText("qui_possedent_des_resultats_d_executions_Afin_de_conserver_l_integrite_des_resultats_vous_ne_pouvez_pas_reordonner_les_actions"),
					Language.getInstance().getText("Erreur_"),
					JOptionPane.ERROR_MESSAGE);
		} else {
			if ((!message.equals(""))&& (!Api.isLockExecutedTest())) { 
				Object[] options = {Language.getInstance().getText("Oui"), Language.getInstance().getText("Non")};
				int choice = SalomeTMFContext.getInstance().askQuestion(
						Language.getInstance().getText("Ce_test_est_utilise_dans_les_campagnes__") + message +  Language.getInstance().getText("IntegiteCheck"),
						Language.getInstance().getText("Attention_"),
						JOptionPane.WARNING_MESSAGE,
						options);
				if (choice == JOptionPane.NO_OPTION) {
					return;
				}
			}
			if (DataModel.getCurrentTest() instanceof ManualTest) {
				new ActionOrdering(((ManualTest)DataModel.getCurrentTest()).getActionListFromModel(false));
			}
		}
	}
	
	public void deleteManualTestActionPerformed() {
		String message = DataModel.getCurrentProject().getCampaignWithExecResultWhereTestIsUse(DataModel.getCurrentTest());
		if ((!message.equals(""))&& (Api.isLockExecutedTest())) {
			JOptionPane.showMessageDialog(ManualActionView.this,
					Language.getInstance().getText("Ce_test_est_utilise_dans_les_campagnes__") + message + Language.getInstance().getText("qui_possedent_des_resultats_d_executions_Afin_de_conserver_l_integrite_des_resultats_vous_ne_pouvez_pas_supprimer_cette_action"),
					Language.getInstance().getText("Erreur_"),
					JOptionPane.ERROR_MESSAGE);
		} else {
			if ((!message.equals(""))&& (!Api.isLockExecutedTest())) { 
				Object[] options = {Language.getInstance().getText("Oui"), Language.getInstance().getText("Non")};
				int choice = SalomeTMFContext.getInstance().askQuestion(
						Language.getInstance().getText("Ce_test_est_utilise_dans_les_campagnes__") + message +  Language.getInstance().getText("IntegiteCheck"),
						Language.getInstance().getText("Attention_"),
						JOptionPane.WARNING_MESSAGE,
						options);
				if (choice == JOptionPane.NO_OPTION) {
					return;
				}
			}
			Object[] options = {Language.getInstance().getText("Oui"), Language.getInstance().getText("Non")};
			int choice = -1;
			int[] selectedRows = actionTable.getSelectedRows();
			String actionList = "";
			for (int i = selectedRows.length - 1; i >= 0 ; i--) {
				actionList = actionList + "* " + (String)DataModel.getActionTableModel().getValueAt(selectedRows[i],0);
				if (i == 0) {
					actionList = actionList + " ?";
				} else {
					actionList = actionList + " \n";
				}
			}
			choice = JOptionPane.showOptionDialog(ManualActionView.this,
					Language.getInstance().getText("Etes_vous_sur_de_vouloir_supprimer_les_actions__") + actionList,
					Language.getInstance().getText("Attention_"),
					JOptionPane.YES_NO_OPTION,
					JOptionPane.QUESTION_MESSAGE,
					null,
					options,
					options[1]);
			if (choice == JOptionPane.YES_OPTION) {
				for (int i = selectedRows.length - 1; i >= 0 ; i--) {
					
					try {
						// BdD
						Action action = ((ManualTest)DataModel.getCurrentTest()).getActionFromModel((String)DataModel.getActionTableModel().getValueAt(selectedRows[i], 0));
						//action.deleteFromDB(true);
						((ManualTest)DataModel.getCurrentTest()).deleteActionInDBModel(action);
						//action.deleteFromBddAndModel(true);
						// IHM
						//((ManualTest)DataModel.getCurrentTest()).deleteAction((String)DataModel.getActionTableModel().getValueAt(selectedRows[i],0));
						if (DataModel.getActionTableModel().getRowCount() == 0) {
							deleteManualTestAction.setEnabled(false);
						}
						DataModel.getActionTableModel().removeData(selectedRows[i]);
						updateIhmActionOrder((ManualTest)DataModel.getCurrentTest());
						
					} catch (Exception exception) {
						Tools.ihmExceptionView(exception);
					}
				}
			}
		}
		//setColumnSize(actionTable);	
	}
	
	
	public void copyPerformed() {
		int[] selectedRows = actionTable.getSelectedRows();
		ArrayList data = new ArrayList();
		for (int i = 0; i < selectedRows.length; i++) {
			data.add(((ManualTest)DataModel.getCurrentTest()).getActionListFromModel(false).get(selectedRows[i]));
		}
		try {
			RowTransferable rowTransferable = new RowTransferable(data);
			Toolkit.getDefaultToolkit().getSystemClipboard().setContents(rowTransferable,null);
		} catch( IllegalStateException ise) {
			JOptionPane.showMessageDialog(ManualActionView.this,
					Language.getInstance().getText("Le_presse_papier_est_deja_utilise_par_une_autre_application_"),
					Language.getInstance().getText("Erreur_"),
					JOptionPane.ERROR_MESSAGE);
		}
		paste.setEnabled(true);
	}
	
	public void cutPerformed() {
		String message = DataModel.getCurrentProject().getCampaignWithExecResultWhereTestIsUse(DataModel.getCurrentTest());
		boolean error = false;
		if ((!message.equals(""))&& (Api.isLockExecutedTest())) {
			JOptionPane.showMessageDialog(ManualActionView.this,
					Language.getInstance().getText("Ce_test_est_utilise_dans_les_campagnes__") + message + Language.getInstance().getText("qui_possedent_des_resultats_d_executions_Afin_de_conserver_l_integrite_des_resultats_vous_ne_pouvez_pas_supprimer_cette_action"),
					Language.getInstance().getText("Erreur_"),
					JOptionPane.ERROR_MESSAGE);
		} else {
			if ((!message.equals(""))&& (!Api.isLockExecutedTest())) { 
				Object[] options = {Language.getInstance().getText("Oui"), Language.getInstance().getText("Non")};
				int choice = SalomeTMFContext.getInstance().askQuestion(
						Language.getInstance().getText("Ce_test_est_utilise_dans_les_campagnes__") + message +  Language.getInstance().getText("IntegiteCheck"),
						Language.getInstance().getText("Attention_"),
						JOptionPane.WARNING_MESSAGE,
						options);
				if (choice == JOptionPane.NO_OPTION) {
					return;
				}
			}
			int[] selectedRows = actionTable.getSelectedRows();
			ArrayList data = new ArrayList();
			for (int i = 0; i < selectedRows.length; i++) {
				data.add(((ManualTest)DataModel.getCurrentTest()).getActionListFromModel(false).get(selectedRows[i]));
			}
			try {
				RowTransferable rowTransferable = new RowTransferable(data);
				Toolkit.getDefaultToolkit().getSystemClipboard().setContents(rowTransferable,null);
				
			} catch( IllegalStateException ise) {
				JOptionPane.showMessageDialog(ManualActionView.this,
						Language.getInstance().getText("Le_presse_papier_est_deja_utilise_par_une_autre_application_"),
						Language.getInstance().getText("Erreur_"),
						JOptionPane.ERROR_MESSAGE);
				error = true;
			}
			if (!error) {
				for (int i = selectedRows.length - 1; i >=0 ; i--) {
					
					try {
						//BdD
						Action action = ((ManualTest)DataModel.getCurrentTest()).getActionFromModel((String)DataModel.getActionTableModel().getValueAt(selectedRows[i], 0));
						//action.deleteFromDB(true);
						//((ManualTest)DataModel.getCurrentTest()).deleteActionInModel(action);
						((ManualTest)DataModel.getCurrentTest()).deleteActionInDBModel(action);
						//action.deleteFromBddAndModel(true);
						// IHM
						//((ManualTest)DataModel.getCurrentTest()).deleteAction((String)DataModel.getActionTableModel().getValueAt(selectedRows[i],0));
						DataModel.getActionTableModel().removeData(selectedRows[i]);
					} catch (Exception exception) {
						Tools.ihmExceptionView(exception);
					}
				}
				updateIhmActionOrder((ManualTest)DataModel.getCurrentTest());
				paste.setEnabled(true);
			}
		}
	}
	
	public void pastePerformed() {
		String message = DataModel.getCurrentProject().getCampaignWithExecResultWhereTestIsUse(DataModel.getCurrentTest());
		if ((!message.equals(""))&& (Api.isLockExecutedTest())) {
			JOptionPane.showMessageDialog(ManualActionView.this,
					Language.getInstance().getText("Ce_test_est_utilise_dans_les_campagnes__") + message + Language.getInstance().getText("qui_possedent_des_resultats_d_executions_Afin_de_conserver_l_integrite_des_resultats_vous_ne_pouvez_pas_ajouter_une_nouvelle_action"),
					Language.getInstance().getText("Erreur_"),
					JOptionPane.ERROR_MESSAGE);
		} else {
			if ((!message.equals(""))&& (!Api.isLockExecutedTest())) { 
				Object[] options = {Language.getInstance().getText("Oui"), Language.getInstance().getText("Non")};
				int choice = SalomeTMFContext.getInstance().askQuestion(
						Language.getInstance().getText("Ce_test_est_utilise_dans_les_campagnes__") + message +  Language.getInstance().getText("IntegiteCheck"),
						Language.getInstance().getText("Attention_"),
						JOptionPane.WARNING_MESSAGE,
						options);
				if (choice == JOptionPane.NO_OPTION) {
					return;
				}
			}
			Transferable t = Toolkit.getDefaultToolkit().getSystemClipboard().getContents(null);
			try {
				DataFlavor dataFlavor = new DataFlavor(ArrayList.class,"ArrayList");
				/** V?rification que le contenu est de type texte. */
				if( t!=null && t.isDataFlavorSupported(dataFlavor)) {
					ArrayList actionsList = (ArrayList)t.getTransferData(dataFlavor);
					Test pTest = DataModel.getCurrentTest();
					for (int i = 0; i < actionsList.size(); i++) {
						int transNumber= -1;
						try {
							//Init
							ArrayList data = new ArrayList();
							Action pActionToCopy = (Action)actionsList.get(i);
							Action pNewAction = new Action(pActionToCopy, pTest) ;
							
							if (((ManualTest)pTest).hasActionNameInModel(pNewAction.getNameFromModel())) {
								String newName = Language.getInstance().getText("Copie_de_") + pNewAction.getNameFromModel();
								while (((ManualTest)pTest).hasActionNameInModel(newName)) {
									newName = Language.getInstance().getText("Copie_de_") + newName;
								}
								pNewAction.updateInModel(newName, pNewAction.getDescriptionFromModel(), pNewAction.getAwaitedResultFromModel());
							}
							//BDD
							transNumber = Api.beginTransaction(101, ApiConstants.INSERT_ACTION);
							
							//1 Ad action
							pNewAction.setOrderIndex(DataModel.getActionTableModel().getRowCount());
							((ManualTest)pTest).addActionInDB(pNewAction);
							
							// 2 Add Use Param
							Vector paramToAdd2Test = new Vector();
							HashSet paramsSet = new HashSet(pActionToCopy.getParameterHashSetFromModel().values()); //FOR HashTable2HashSet
							for (Iterator iter = paramsSet.iterator(); iter.hasNext();) {
								Parameter param = (Parameter)iter.next();
								if (!pTest.hasUsedParameterNameFromModel(param.getNameFromModel())) {
									pTest.setUseParamInDBAndModel(param);
									//pTest.setUseParamInDB(param.getIdBdd());
									paramToAdd2Test.add(param);
								}
								//pNewAction.setUseParamInDB(param.getIdBdd());
								pNewAction.setUseParamInDBAndModel(param);
							}
							Api.commitTrans(transNumber);
							transNumber = -1;
							//IHM
							data.add(pNewAction.getNameFromModel());
							data.add(pNewAction.getDescriptionFromModel());
							data.add(pNewAction.getAwaitedResultFromModel());
							data.add(pNewAction.getAttachmentMapFromModel());
							for(int j = 0; j <paramToAdd2Test.size() ; j++ ){
								Parameter param = (Parameter)paramToAdd2Test.get(j);
								ArrayList dataParam = new ArrayList();
								dataParam.add(param.getNameFromModel());
								dataParam.add(param.getDescriptionFromModel());
								DataModel.getTestParameterTableModel().addRow(dataParam);
							}
							paramToAdd2Test.clear();
							DataModel.getActionTableModel().addRow(data);
							((ManualTest)pTest).addActionInModel(pNewAction);
						} catch (Exception e_copy){
							Api.forceRollBackTrans(transNumber);
							Tools.ihmExceptionView(e_copy);
							
						}
					}
				}
			} catch( UnsupportedFlavorException e1) {
				JOptionPane.showMessageDialog(ManualActionView.this,
						Language.getInstance().getText("Impossible_de_copier__il_ne_s_agit_pas_d_actions_"),
						Language.getInstance().getText("Erreur_"),
						JOptionPane.ERROR_MESSAGE);
			} catch( IOException e2 ) {
				if (Api.isDEBUG()){
					e2.printStackTrace();
				}
				JOptionPane.showMessageDialog(ManualActionView.this,
						Language.getInstance().getText("Probleme_lors_de_la_recuperation_des_donnees_"),
						Language.getInstance().getText("Erreur_"),
						JOptionPane.ERROR_MESSAGE);
			}
		}
	}
	
	
	//************** Event table *******************//
	
	 
	
	public void valueChanged(ListSelectionEvent e){
		if (e.getSource().equals(actionTable.getSelectionModel())){
			actionTablevalueChanged(e);
		}
		//setColumnSize2(actionTable);
	}
	
	public void actionTablevalueChanged(ListSelectionEvent e) {
		if (e.getValueIsAdjusting())
			return;
	
		int nbOfSelectedRows = actionTable.getSelectedRowCount();
		int selectedRow = actionTable.getSelectedRow();
		if (selectedRow != -1) {
			if (nbOfSelectedRows != 1) {
				if (Permission.canDeleteTest()) {
					deleteManualTestAction.setEnabled(true);
				}
				modifyAction.setEnabled(false);
			} else {
				if (Permission.canUpdateTest()) {
					modifyAction.setEnabled(true);
				}
				if (Permission.canDeleteTest()) {
					deleteManualTestAction.setEnabled(true);
				}
			}
			if (Permission.canDeleteTest())	cut.setEnabled(true);
			if (Permission.canCreateTest()) copy.setEnabled(true);
			
			
		} else {
			deleteManualTestAction.setEnabled(false);
			modifyAction.setEnabled(false);
			cut.setEnabled(false);
			copy.setEnabled(false);
		}
		
	}
	
	
	
} // Fin de la classe ManualActionView
