/*
* SalomeTMF is a Test Management Framework
* Copyright (C) 2005 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* @author Fayçal SOUGRATI, Vincent Pautret, Marche Mikael
*
* Contact: mikael.marche@rd.francetelecom.com
*/

package org.objectweb.salome_tmf.ihm.main;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

import org.objectweb.salome_tmf.api.Api;
import org.objectweb.salome_tmf.api.ApiConstants;
import org.objectweb.salome_tmf.data.Action;
import org.objectweb.salome_tmf.data.DataConstants;
import org.objectweb.salome_tmf.data.Execution;
import org.objectweb.salome_tmf.data.ExecutionResult;
import org.objectweb.salome_tmf.data.ExecutionTestResult;
import org.objectweb.salome_tmf.data.FileAttachment;
import org.objectweb.salome_tmf.data.ManualExecutionResult;
import org.objectweb.salome_tmf.data.ManualTest;
import org.objectweb.salome_tmf.data.WithAttachment;
import org.objectweb.salome_tmf.ihm.IHMConstants;
import org.objectweb.salome_tmf.ihm.languages.Language;
import org.objectweb.salome_tmf.ihm.main.datawrapper.DataModel;
import org.objectweb.salome_tmf.ihm.main.plugins.PluginsTools;
import org.objectweb.salome_tmf.ihm.models.MyTableModel;
import org.objectweb.salome_tmf.ihm.tools.Tools;
import org.objectweb.salome_tmf.plugins.UICompCst;
import org.objectweb.salome_tmf.plugins.core.BugTracker;

/**
 * Classe qui cr?e la fen?tre tracant l'?x?cution d'un test
 * @author teaml039
 * @version : 0.1
 */
public class ManualExecution extends JDialog  implements ApiConstants, DataConstants, IHMConstants, Runnable {
    
    /**
     * Bouton pour indiquer que l'action a r?ussi
     */
    JButton statusOkButton;
    
    /**
     * Bouton pour indiquer que l'action a ?chou?
     */
    JButton statusFailedButton;
    
    /**
     * Bouton pour indiquer que l'on ne conna?t pas le r?sultat de l'action
     */
    JButton statusUnknowButton;
    
    /**
     * Bouton pour revenir au test pr?c?dent
     */
    JButton prevTestButton;
    
    /**
     * Bouton pour passer au test suivant
     */
    JButton nextTestButton;
    
    /**
     * Bouton pour interrompre l'ex?cution
     */
    //JButton stopButton;
    
    /**
     * Bouton pour sortir de la fen?tre
     */
    JButton endButton;
    
    
    JButton screenShootButton;
    
    
    /**
     * Mod?le de donn?es pour la table des actions
     */
    MyTableModel actionsTableModel;
    
    /**
     * Table des actions
     */
    JTable actionsTable;
    
    /**
     * Description de l'action courante
     */
    JTextPane actionDescription;
    
    /**
     * R?sultat attendu de l'action courante
     */
    JTextPane awaitedResult;
    
    /**
     * R?sultat effectif de l'action courante
     */
    JTextPane effectiveResult;
    
    /**
     * Liste des tests ? ex?cuter
     */
    ArrayList testsToBeExecuted;
    
    /**
     * Nom du test courant
     */
    JLabel testName;
    
    /**
     * Description du test courant
     */
    JTextPane testDescription;
    HTMLDocument m_oDocument;
	HTMLEditorKit m_oHTML;
    /**
     * Conteneur pour l'affichage
     */
    Container contentPaneFrame;
    
    /**
     * Le test en cours d'ex?cution
     */
    ManualTest testInExecution;
    
    ManualExecutionResult execTestRes ;
    /**
     * Indice du test en cours d'ex?cution
     */
    int indexOfTextInExecution;
    
    /**
     * R?sultat de l'?x?cution
     */
    ExecutionResult execResult;
    
    /**
     * Nom du r?sultat d'ex?cution courant
     */
    String currentExecutionResultName;
    
    /**
     * Indexe de bloquage dans la table des actions (pour emp?cher la s?lection)
     */
    int blockIndex;
    
    /**
     * Mod?le de s?lection pour la table des actions
     */
    ListSelectionModel rowSM;
    
   
    
    /**
     * Execution courante
     */
    Execution currentExecution;
    
    HashMap oldMap;
    
    
    Thread t;
    
    //ExecutionView execView;
    
    //JButton addBugButton;
    
    boolean finished = false;
    boolean stop_exec = false;
    
    String title;
    /******************************************************************************/
    /** 							CONSTRUCTEUR								***/
    /******************************************************************************/
    //ExecutionResult finalExecResult;
    int t_x = 1024 -100;
    int t_y = 768 -200; 
    int t_x2 = 1024;
    int t_y2 = 768 ;
    
    
    /****** Gestion des attachements par onglet *******/
    AttachmentView resExecTestAttachmentPanel;
    AttachmentView testAttachmentPanel;
    AttachmentView actionAttachmentPanel;
    JTabbedPane pJTabbedPane;
    
    /**
     * Constructeur de la fen?tre
     * @param testList liste de tests ? ex?cuter
     */
    public ManualExecution(ArrayList testList, Execution execution, ExecutionResult executionResult, boolean contineExec) {
        
        super(SalomeTMFContext.getInstance().ptrFrame,true);
        //setResizable(false);
        //this.finalExecResult = finalExecResult;
        
        DataModel.setObervedExecutionResult(executionResult);
        
        try {
        	GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        	GraphicsDevice[] gs = ge.getScreenDevices();
        	GraphicsDevice gd = gs[0];
        	GraphicsConfiguration[] gc = gd.getConfigurations();
        	Rectangle r = gc[0].getBounds();
        	t_x = r.width -100;
        	t_y = r.height -200;  
        	t_x2 = r.width;
            t_y2 = r.height ;
        } catch(Exception E){
        	
        }
        
        finished = false;
        blockIndex = 0;
       
        
        oldMap = new HashMap();
        
        statusOkButton = new JButton(Language.getInstance().getText("Succes"));
        
        statusFailedButton = new JButton(Language.getInstance().getText("Echec"));
        
        statusUnknowButton = new JButton(Language.getInstance().getText("Inconclusif"));
        
        prevTestButton = new JButton(Language.getInstance().getText("Test_precedent"));
        
        nextTestButton = new JButton(Language.getInstance().getText("Test_suivant"));
        
        //stopButton = new JButton(Language.getInstance().getText("Stop"));
        
        endButton = new JButton(Language.getInstance().getText("Terminer"));
        
        screenShootButton = new JButton();
        screenShootButton.setIcon(Tools.createAppletImageIcon(PATH_TO_SCREENSHOT_ICON,""));
        
        
        actionsTableModel = new MyTableModel();
        
        actionsTable = new JTable();
        
        actionDescription = new JTextPane();
        
        awaitedResult = new JTextPane();
        
        effectiveResult = new JTextPane();
        
        testsToBeExecuted = new ArrayList();
        
        testName = new JLabel();
        
        testDescription = new JTextPane();
        m_oDocument = new HTMLDocument();
        m_oHTML  = new HTMLEditorKit(); 
        testDescription.setEditorKit(m_oHTML);
        testDescription.setDocument(m_oDocument);
        
        // Mapping entre objets graphiques et constantes
        SalomeTMFContext.getInstance().addToUIComponentsMap(UICompCst.MANUAL_EXECUTION_TEST_DESCRIPTION_TEXTPANE,testDescription);
        SalomeTMFContext.getInstance().addToUIComponentsMap(UICompCst.MANUAL_EXECUTION_ACTION_DESCRIPTION_TEXTPANE,actionDescription);
        SalomeTMFContext.getInstance().addToUIComponentsMap(UICompCst.MANUAL_EXECUTION_ACTION_WAITED_RESULT_TEXTPANE,awaitedResult);
        SalomeTMFContext.getInstance().addToUIComponentsMap(UICompCst.MANUAL_EXECUTION_ACTION_EFFECTIVE_RESULT_TEXTPANE,effectiveResult);
        // Activation des plugins associ?s
        PluginsTools.activateAssociatedPlgs(UICompCst.MANUAL_EXECUTION_TEST_DESCRIPTION_TEXTPANE);
        PluginsTools.activateAssociatedPlgs(UICompCst.MANUAL_EXECUTION_ACTION_DESCRIPTION_TEXTPANE);
        PluginsTools.activateAssociatedPlgs(UICompCst.MANUAL_EXECUTION_ACTION_WAITED_RESULT_TEXTPANE);
        PluginsTools.activateAssociatedPlgs(UICompCst.MANUAL_EXECUTION_ACTION_EFFECTIVE_RESULT_TEXTPANE);
        
      
        //execView = executionView;
        
        statusOkButton.setToolTipText(Language.getInstance().getText("L_action_a_reussie"));
        statusOkButton.setIcon(Tools.createAppletImageIcon(PATH_TO_SUCCESS_ICON,""));
        statusOkButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                int selectedRowIndex = actionsTable.getSelectedRow();
                Action pAction = testInExecution.getActionFromModel((String)actionsTableModel.getValueAt(selectedRowIndex, 0));
                if (selectedRowIndex != -1 && pAction !=null) {
                    actionsTableModel.setValueAt(Tools.createAppletImageIcon(PATH_TO_SUCCESS_ICON,""), selectedRowIndex, actionsTableModel.getColumnCount() - 1);
                    //execResult.addStatusForAction(testInExecution.getActionFromModel((String)actionsTableModel.getValueAt(selectedRowIndex, 0)), SUCCESS);
                    
                    //ManualExecutionResult execTestRes = (ManualExecutionResult) execResult.getExecutionTestResultFromModel(testInExecution);
                    execTestRes.addStatusForActionInModel(testInExecution.getActionFromModel((String)actionsTableModel.getValueAt(selectedRowIndex, 0)), SUCCESS);
                    if (selectedRowIndex < (actionsTableModel.getRowCount() - 1)) {
                        
                        blockIndexCalculation(selectedRowIndex);
                        ListSelectionModel lsm = actionsTable.getSelectionModel();
                        lsm.setSelectionInterval(selectedRowIndex+1, selectedRowIndex+1);
                    } else {
                        
                        blockIndexCalculation(0);
                        changeTest();
                    }
                }
            }
        });
        
        statusFailedButton.setToolTipText(Language.getInstance().getText("L_action_a_echouee"));
        statusFailedButton.setIcon(Tools.createAppletImageIcon(PATH_TO_FAIL_ICON,""));
        statusFailedButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                int selectedRowIndex = actionsTable.getSelectedRow();
                Action pAction = testInExecution.getActionFromModel((String)actionsTableModel.getValueAt(selectedRowIndex, 0));
                
                if (selectedRowIndex != -1 && pAction !=null) {
                    
                    //execResult.addStatusForAction(testInExecution.getActionFromModel((String)actionsTableModel.getValueAt(selectedRowIndex, 0)), FAIL);
                    execTestRes.addStatusForActionInModel(testInExecution.getActionFromModel((String)actionsTableModel.getValueAt(selectedRowIndex, 0)), FAIL);
                    actionsTableModel.setValueAt(Tools.createAppletImageIcon(PATH_TO_FAIL_ICON,""), selectedRowIndex, actionsTableModel.getColumnCount() - 1);
                    if (selectedRowIndex < (actionsTableModel.getRowCount() - 1)) {
                        //blockIndex = selectedRowIndex + 1;
                        blockIndexCalculation(selectedRowIndex);
                    } else {
                        //blockIndex = 0;
                        blockIndexCalculation(0);
                    }
                }
            }
        });
        
        statusUnknowButton.setToolTipText(Language.getInstance().getText("Pas_de_resultat_pour_cette_action"));
        statusUnknowButton.setIcon(Tools.createAppletImageIcon(PATH_TO_UNKNOW_ICON, ""));
        statusUnknowButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                int selectedRowIndex = actionsTable.getSelectedRow();
                Action pAction = testInExecution.getActionFromModel((String)actionsTableModel.getValueAt(selectedRowIndex, 0));
                
                if (selectedRowIndex != -1 && pAction !=null) {
                    
                    //execResult.addStatusForAction(testInExecution.getActionFromModel((String)actionsTableModel.getValueAt(selectedRowIndex, 0)), UNKNOWN);
                    execTestRes.addStatusForActionInModel(testInExecution.getActionFromModel((String)actionsTableModel.getValueAt(selectedRowIndex, 0)), UNKNOWN);
                    actionsTableModel.setValueAt(Tools.createAppletImageIcon(PATH_TO_UNKNOW_ICON,""), selectedRowIndex, actionsTableModel.getColumnCount() - 1);
                    if (selectedRowIndex < (actionsTableModel.getRowCount() - 1)) {
                        //blockIndex = selectedRowIndex + 1;
                        blockIndexCalculation(selectedRowIndex);
                    } else {
                        //blockIndex = 0;
                        blockIndexCalculation(0);
                    }
                }
            }
        });
        
        prevTestButton.setToolTipText(Language.getInstance().getText("Revenir_au_test_precedent"));
        prevTestButton.setEnabled(false);
        prevTestButton.setIcon(Tools.createAppletImageIcon(PATH_TO_ARROW_BACK_ICON,""));
        prevTestButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                changeTestBack();
                blockIndexCalculation(0);
            }
        });
        
        
        nextTestButton.setToolTipText(Language.getInstance().getText("Passer_au_test_suivant"));
        nextTestButton.setIcon(Tools.createAppletImageIcon(PATH_TO_ARROW_ICON,""));
        nextTestButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                changeTest();
                blockIndexCalculation(0);
            }
        });
        if (testList.size() < 2) {
            nextTestButton.setEnabled(false);
        }
        
        /*
        stopButton.setToolTipText(Language.getInstance().getText("Interrompre_l_execution"));
        stopButton.setIcon(Tools.createAppletImageIcon(PATH_TO_STOP_ICON,""));
        stopButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                testStatusCalculation();
                stop_exec = true;
                execResult.setExecutionStatusInModel(INTERRUPT);
                //execView.libere();
                finished = true;
                ManualExecution.this.dispose();
            }
        });
        */
        
        endButton.setToolTipText(Language.getInstance().getText("Sortir_de_l_execution"));
        endButton.setIcon(Tools.createAppletImageIcon(PATH_TO_MOVE_ICON,""));
        endButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                testStatusCalculation();
                
                finished = true;
                ManualExecution.this.dispose();
            }
        });
        
        
        screenShootButton.addActionListener(new ActionListener () {
			public void actionPerformed(java.awt.event.ActionEvent e){
				screeshootFrame(e);
			}
		});
        
        /*
        JButton attachButton = new JButton(Language.getInstance().getText("Attachements"));
        attachButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
            	//new AttachmentViewWindow(SalomeTMF.ptrSalomeTMF, EXECUTION_RESULT_TEST, DataModel.getCurrentExecutionTestResult(), null, null, null);
            	new AttachmentViewWindow(SalomeTMFContext.getInstance().getBaseIHM(), EXECUTION_RESULT_TEST, DataModel.getCurrentExecutionTestResult());
                
            }
        });
        */
        
      /*
        JMenu attachMenu = new JMenu(Language.getInstance().getText("Attachements"));
        
        // Test execution result attachments
        JMenuItem resExecAttachSubMenu = new JMenuItem(Language.getInstance().getText("Res_Exec_Attachs"));
        resExecAttachSubMenu.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
            	//new AttachmentViewWindow(SalomeTMF.ptrSalomeTMF, EXECUTION_RESULT_TEST, DataModel.getCurrentExecutionTestResult(), null, null, null);
            	new AttachmentViewWindow(SalomeTMFContext.getInstance().getBaseIHM(), EXECUTION_RESULT_TEST, DataModel.getCurrentExecutionTestResult());
            	
            }
        });
        attachMenu.add(resExecAttachSubMenu);
        attachMenu.addSeparator();
        
        // Current test in execution attachments
        JMenuItem currentTestAttachSubMenu = new JMenuItem(Language.getInstance().getText("Current_Test_Attachs"));
        currentTestAttachSubMenu.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
            	DataModel.setCurrentTest(testInExecution);
            	JDialog testAttachsDialog = new JDialog(SalomeTMFContext.getInstance().ptrFrame, true);
                AttachmentView attachView = new AttachmentView(SalomeTMFContext.getInstance().getBaseIHM(), TEST, TEST, null, NORMAL_SIZE_FOR_ATTACH, testAttachsDialog);
                attachView.setReadOnly();
                Container contentPaneFrame = testAttachsDialog.getContentPane();
                contentPaneFrame.add(attachView, BorderLayout.CENTER);
                testAttachsDialog.setTitle(Language.getInstance().getText("Attachements") + " (Test = " + testInExecution.getNameFromModel() + ")");
                testAttachsDialog.pack();
                testAttachsDialog.setLocation((t_x2 - contentPaneFrame.getSize().width) /2 , (t_y2 - contentPaneFrame.getSize().height)/2);
                
                testAttachsDialog.setVisible(true);
            }
        });
        attachMenu.add(currentTestAttachSubMenu);
        
        // Currents action in execution attachments
        JMenuItem currentActionAttachSubMenu = new JMenuItem(Language.getInstance().getText("Current_Action_Attachs"));
        currentActionAttachSubMenu.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
            	int selectedRowIndex = actionsTable.getSelectedRow();
            	ArrayList dataList = actionsTableModel.getData(selectedRowIndex);
                Action action = testInExecution.getActionFromModel((String)dataList.get(0));
            	DataModel.setCurrentAction(action);
            	JDialog actionAttachsDialog = new JDialog(SalomeTMFContext.getInstance().ptrFrame, true);
                AttachmentView attachView = new AttachmentView(SalomeTMFContext.getInstance().getBaseIHM(), ACTION, ACTION, null, NORMAL_SIZE_FOR_ATTACH, actionAttachsDialog);
                attachView.setReadOnly();
                Container contentPaneFrame = actionAttachsDialog.getContentPane();
                contentPaneFrame.add(attachView, BorderLayout.CENTER);
                
                //actionAttachsDialog.setLocation(400,400);
                
                actionAttachsDialog.setTitle(Language.getInstance().getText("Attachements") + " (Test = " + testInExecution.getNameFromModel()+ ", " + "Action = " + action.getNameFromModel() + ")");
                actionAttachsDialog.pack();
                actionAttachsDialog.setLocation((t_x2 - contentPaneFrame.getSize().width)/2, (t_y2 - contentPaneFrame.getSize().height)/2);
                
                actionAttachsDialog.setVisible(true);
            }
        });
        attachMenu.add(currentActionAttachSubMenu);
        */
        
        // Details sur le test
        testDescription.setEditable(false);
        JScrollPane testDescriptionScrollPane = new JScrollPane(testDescription, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        //testDescriptionScrollPane.setPreferredSize(new Dimension(150, 100));
        testDescriptionScrollPane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK),Language.getInstance().getText("Description_du_test")));
        testDescriptionScrollPane.setMaximumSize(new Dimension(t_x/3,t_y/3)); //MM
        
        // Table d'actions
        
        actionsTableModel.addColumnNameAndColumn(Language.getInstance().getText("Action"));
        actionsTableModel.addColumnNameAndColumn(Language.getInstance().getText("Attachement"));
        actionsTableModel.addColumnNameAndColumn(Language.getInstance().getText("Date"));
        actionsTableModel.addColumnNameAndColumn(Language.getInstance().getText("Statut"));
        
                /*TableSorter  sorter = new TableSorter(actionsTableModel);
                actionsTable.setModel(sorter);
                sorter.setTableHeader(actionsTable.getTableHeader()); */
        actionsTable.setModel(actionsTableModel);
        
        
        
        //actionsTable.setPreferredScrollableViewportSize(new Dimension(700, 150));
        actionsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        
        rowSM = actionsTable.getSelectionModel();
        rowSM.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (e.getValueIsAdjusting())
                    return;
                
                int selectedRowIndex = actionsTable.getSelectedRow();
                if (selectedRowIndex != -1 && actionsTableModel.getRowCount() > 0) {
                    if (blockIndex != -1 && selectedRowIndex > blockIndex) {
                        selectedRowIndex = blockIndex;
                        rowSM.setSelectionInterval(selectedRowIndex, selectedRowIndex);
                    }
                    Rectangle rect = actionsTable.getCellRect(selectedRowIndex, actionsTable.getSelectedColumn(), false);
                    actionsTable.scrollRectToVisible(rect);
                    ArrayList dataList = actionsTableModel.getData(selectedRowIndex);
                    Action action = testInExecution.getActionFromModel((String)dataList.get(0));
                    /*actionDescription.setText(Tools.getInstantiedDescription(action.getDescriptionFromModel(), currentExecution.getDataSetFromModel()));
                    actionDescription.getCaret().setDot(0);
                    awaitedResult.setText(Tools.getInstantiedDescription(action.getAwaitedResultFromModel(), currentExecution.getDataSetFromModel()));
                    awaitedResult.getCaret().setDot(0);*/
                    actionDescription.setText(Tools.getInstantiedDescription(action.getDescriptionFromModel(), currentExecution));
                    actionDescription.getCaret().setDot(0);
                    awaitedResult.setText(Tools.getInstantiedDescription(action.getAwaitedResultFromModel(), currentExecution));
                    awaitedResult.getCaret().setDot(0);
                    //effectiveResult.setText(execResult.getEffectivResult(action));
                    effectiveResult.setText(execTestRes.getEffectivResultFromModel(action));
                    effectiveResult.getCaret().setDot(0);
                }
            }
        });
        
        // Mapping entre objets graphiques et constantes
        SalomeTMFContext.getInstance().addToUIComponentsMap(UICompCst.MANUAL_EXECUTION_ACTIONS_TABLE,actionsTable);
        // Activation des plugins associes
        PluginsTools.activateAssociatedPlgs(UICompCst.MANUAL_EXECUTION_ACTIONS_TABLE);
        
        JScrollPane actionsTableScrollPane = new JScrollPane(actionsTable); 
        actionsTableScrollPane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK),Language.getInstance().getText("Actions")));
        actionsTableScrollPane.setMinimumSize(new Dimension(t_x/2,t_y/3)); //MM
        
        
        //actionsTableScrollPane.setMaximumSize(new Dimension(t_x/2,t_y/3)); //MM
        
        //actionsTableScrollPane.setPreferredSize(new Dimension(200,250)); //MM
        
        // Details sur les actions
        actionDescription.setEditable(false);
        JScrollPane actionDescriptionScrollPane = new JScrollPane(actionDescription);
        actionDescriptionScrollPane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK),Language.getInstance().getText("Description_de_l_action")));
        actionDescriptionScrollPane.setMinimumSize(new Dimension(t_x,t_y/3)); //MM
        
        
        awaitedResult.setEditable(false);
        JScrollPane awaitedResultScrollPane = new JScrollPane(awaitedResult);
        awaitedResultScrollPane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK),Language.getInstance().getText("Resultat_attendu")));
        awaitedResultScrollPane.setMinimumSize(new Dimension(t_x/2,t_y/3)); //MM
        
        effectiveResult.addCaretListener(new EffectivResultListener());
        JScrollPane effectiveResultScrollPane = new JScrollPane(effectiveResult);
        effectiveResultScrollPane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK),Language.getInstance().getText("Resultat_effectif")));
        effectiveResultScrollPane.setMinimumSize(new Dimension(t_x/2,t_y/3)); //MM
        
        // Construction de la fen?tre
        JPanel buttonsPanel = new JPanel();
        buttonsPanel.setLayout(new BoxLayout(buttonsPanel, BoxLayout.X_AXIS));
        
        JPanel basicbuttonPanel = new JPanel(new GridLayout(1,8)); 
        int bouton_x = 120;
        int bouton_y = 30;
        statusOkButton.setMinimumSize(new Dimension(t_x/10, bouton_y)); 
        statusOkButton.setMaximumSize(new Dimension(t_x2/10, bouton_y));
        buttonsPanel.add(statusOkButton);
        
        statusFailedButton.setMinimumSize(new Dimension(t_x/10, bouton_y)); 
        statusFailedButton.setMaximumSize(new Dimension(t_x2/10, bouton_y));
        buttonsPanel.add(statusFailedButton);
        
        statusUnknowButton.setMinimumSize(new Dimension(t_x/10, bouton_y)); 
        statusUnknowButton.setMaximumSize(new Dimension(t_x2/10, bouton_y));
        buttonsPanel.add(statusUnknowButton);
        buttonsPanel.add(Box.createRigidArea(new Dimension(t_x/20,bouton_y)));
        
        prevTestButton.setMinimumSize(new Dimension(t_x/10, bouton_y)); 
        prevTestButton.setMaximumSize(new Dimension(t_x2/10, bouton_y));
        buttonsPanel.add(prevTestButton);
       
        nextTestButton.setMinimumSize(new Dimension(t_x/10, bouton_y)); 
        nextTestButton.setMaximumSize(new Dimension(t_x2/10, bouton_y));
        buttonsPanel.add(nextTestButton); 
        
        
        endButton.setMinimumSize(new Dimension(t_x/10, bouton_y)); 
        endButton.setMaximumSize(new Dimension(t_x2/10, bouton_y));
        buttonsPanel.add(endButton);
        

        buttonsPanel.add(Box.createRigidArea(new Dimension(t_x/20,bouton_y)));
        
        
        screenShootButton.setMinimumSize(new Dimension(t_x/20, bouton_y)); 
        screenShootButton.setMaximumSize(new Dimension(t_x2/20, bouton_y));
        buttonsPanel.add(screenShootButton);
        
       
       /* JMenuBar attachsMenuBar = new JMenuBar();
        attachsMenuBar.add(attachMenu);
        buttonsPanel.add(attachsMenuBar);
        */
        
        // Menu for bugtrackers
        //Vector bugTrackers = SalomeTMFContext.getInstance().bugTrackerExtensions; 
        Vector bugTrackers = SalomeTMFContext.getInstance().getBugTracker(); 
        if (bugTrackers.size() != 0) {
        	JMenu bugTrackMenu = new JMenu(Language.getInstance().getText("add_bug"));
        	for (int i=0 ; i<bugTrackers.size() ; i++) {
        		//final BugTracker bugTracker = (BugTracker)SalomeTMFContext.getInstance().bugTrackers.elementAt(i);
        		final BugTracker bugTracker = (BugTracker)bugTrackers.elementAt(i);
        		JMenuItem bugTrackerItem = new JMenuItem(bugTracker.getBugTrackerName());
        		bugTrackerItem.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                    	int selectedRowIndex = actionsTable.getSelectedRow();
                    	ArrayList dataList = actionsTableModel.getData(selectedRowIndex);
                        Action action = testInExecution.getActionFromModel((String)dataList.get(0));
                        new AskNewBug(ManualExecution.this, bugTracker,true, action.getNameFromModel(),actionDescription.getText(),awaitedResult.getText(),effectiveResult.getText());   
                    	
                    }
                });
        		
        		if (!bugTracker.isUserExistsInBugDB()) {
                	bugTrackerItem.setEnabled(false);
                }
        		bugTrackMenu.add(bugTrackerItem);
        	}
        	JMenuBar menuBar = new JMenuBar();
            menuBar.add(bugTrackMenu);
        	buttonsPanel.add(menuBar);
        }
        
        // Mapping entre objets graphiques et constantes
        SalomeTMFContext.getInstance().addToUIComponentsMap(UICompCst.MANUAL_EXECUTION_BUTTONS_PANEL,buttonsPanel);
        // Activation des plugins associ?es
        PluginsTools.activateAssociatedPlgs(UICompCst.MANUAL_EXECUTION_BUTTONS_PANEL);
        
        JPanel testDetailsPanel = new JPanel();
        /*testDetailsPanel.setLayout(new BoxLayout(testDetailsPanel, BoxLayout.Y_AXIS));
        testDetailsPanel.add(testName);
        testDetailsPanel.add(Box.createRigidArea(new Dimension(1,10)));
        testDetailsPanel.add(testDescriptionScrollPane);
        */
        
        JPanel actionsResults = new JPanel();
        actionsResults.setLayout(new BoxLayout(actionsResults, BoxLayout.X_AXIS));
        actionsResults.add(awaitedResultScrollPane);
        actionsResults.add(effectiveResultScrollPane);
        
        
        /*JPanel actionsDetailsPanel = new JPanel(new BorderLayout());
        actionsDetailsPanel.add(actionDescriptionScrollPane, BorderLayout.NORTH);
        actionsDetailsPanel.add(actionsResults, BorderLayout.SOUTH);
        */
        JPanel actionsDetailsPanel = new JPanel();
        actionsDetailsPanel.setLayout(new BoxLayout(actionsDetailsPanel, BoxLayout.Y_AXIS));
        actionsDetailsPanel.add(actionDescriptionScrollPane);
        actionsDetailsPanel.add(actionsResults);
        
        /*JPanel showTestPanel = new JPanel(new BorderLayout());
        showTestPanel.add(testDetailsPanel, BorderLayout.WEST);
        showTestPanel.add(actionsTableScrollPane, BorderLayout.CENTER);
        */
        
        JPanel showTestPanel = new JPanel(new BorderLayout()); 
        
        JPanel showTestPanel2 = new JPanel(); 
        showTestPanel2.setLayout(new BoxLayout(showTestPanel2, BoxLayout.X_AXIS));
        //showTestPanel.add(testDetailsPanel);
        showTestPanel2.add(testDescriptionScrollPane);
        showTestPanel2.add(actionsTableScrollPane);
        
        showTestPanel.add(testName, BorderLayout.NORTH);
        showTestPanel.add(showTestPanel2, BorderLayout.CENTER);
        
        /*
        contentPaneFrame.add(actionsDetailsPanel, BorderLayout.SOUTH);
        contentPaneFrame.add(showTestPanel, BorderLayout.CENTER);
        contentPaneFrame.add(buttonsPanel, BorderLayout.NORTH);
        */
        initData(testList, execution, executionResult);
        if (contineExec) {
        	setFirstTestWithNoResult();
        }
        setSize(t_x, t_y); 
        
        pJTabbedPane = new JTabbedPane();
       
        JPanel mainPanel = new JPanel();
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
        buttonsPanel.setMaximumSize(new Dimension(t_x2, 55));
        mainPanel.add(buttonsPanel); 
        //nom du test
        //mainPanel.add(testName);
        showTestPanel.setPreferredSize(new Dimension(t_x, t_y/3));
        mainPanel.add(showTestPanel);
        actionsDetailsPanel.setPreferredSize(new Dimension(t_x, t_y/3*2 ));
        mainPanel.add(actionsDetailsPanel);
        pJTabbedPane.add(Language.getInstance().getText("Test"),mainPanel);	
        
        /****** Attachement en panel ***********/
        resExecTestAttachmentPanel = new AttachmentView(DataModel.getApplet(), EXECUTION_RESULT_TEST, EXECUTION_RESULT_TEST, execTestRes, SMALL_SIZE_FOR_ATTACH, this);
        resExecTestAttachmentPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK),Language.getInstance().getText("Attachements")));
        pJTabbedPane.add(resExecTestAttachmentPanel, Language.getInstance().getText("Res_Exec_Attachs"));
        
        
        testAttachmentPanel = new AttachmentView(DataModel.getApplet(), TEST, TEST, null, SMALL_SIZE_FOR_ATTACH, this);
        testAttachmentPanel.setReadOnly();
        pJTabbedPane.add(testAttachmentPanel, Language.getInstance().getText("Current_Test_Attachs"));
        
        actionAttachmentPanel = new AttachmentView(DataModel.getApplet(), ACTION, ACTION, null, NORMAL_SIZE_FOR_ATTACH, this);
        actionAttachmentPanel.setReadOnly();
        pJTabbedPane.add(actionAttachmentPanel, Language.getInstance().getText("Current_Action_Attachs"));
        
        pJTabbedPane.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				if (pJTabbedPane.getSelectedComponent().equals(resExecTestAttachmentPanel)){
					//DataModel.getCurrentExecutionTestResult()
					resExecTestAttachmentPanel.setAttachmentObject(DataModel.getCurrentExecutionTestResult());
				} else if (pJTabbedPane.getSelectedComponent().equals(testAttachmentPanel)){
					DataModel.setCurrentTest(testInExecution);
					testAttachmentPanel.setAttachmentObject(testInExecution);
				} else if (pJTabbedPane.getSelectedComponent().equals(actionAttachmentPanel)){
					int selectedRowIndex = actionsTable.getSelectedRow();
	            	ArrayList dataList = actionsTableModel.getData(selectedRowIndex);
	                Action action = testInExecution.getActionFromModel((String)dataList.get(0));
	            	DataModel.setCurrentAction(action);
	            	actionAttachmentPanel.setAttachmentObject(action);
				}
			}
        });
        /**************************************/
        
        
        contentPaneFrame = this.getContentPane();
        contentPaneFrame.add(pJTabbedPane);
        
       
        SalomeTMFContext.getInstance().addToUIComponentsMap(UICompCst.MANUAL_EXECUTION_TAB,pJTabbedPane);
		PluginsTools.activateAssociatedPlgs(UICompCst.MANUAL_EXECUTION_TAB);
		
        /*contentPaneFrame = this.getContentPane();
        contentPaneFrame.setLayout(new BoxLayout(contentPaneFrame, BoxLayout.Y_AXIS));
        buttonsPanel.setMaximumSize(new Dimension(t_x2, 55));
        contentPaneFrame.add(buttonsPanel); 
        showTestPanel.setPreferredSize(new Dimension(t_x, t_y/3));
        contentPaneFrame.add(showTestPanel);
        actionsDetailsPanel.setPreferredSize(new Dimension(t_x, t_y/3*2 ));
        contentPaneFrame.add(actionsDetailsPanel);
         */
        
        
        if (actionsTableModel.getRowCount() > 0) {
            rowSM.setSelectionInterval(0,0);
        }
        
        this.addWindowListener(new WindowListener() {
            public void windowClosing(WindowEvent e) {
                
                 testStatusCalculation();
                //execView.libere();
                //execResult.setExecutionStatus(INTERRUPT);
                stop_exec = true;
                finished = true;
            }
            public void windowDeiconified(WindowEvent e) {
            }
            public void windowOpened(WindowEvent e) {
            }
            public void windowActivated(WindowEvent e) {
            }
            public void windowDeactivated(WindowEvent e) {
            }
            public void windowClosed(WindowEvent e) {
            }
            public void windowIconified(WindowEvent e) {
            }
        });
        
        title = Language.getInstance().getText("Execution_d_un_test_manuel_exec_en_cours__") + execution.getNameFromModel() + ")";
        this.setTitle(title + ", test : " + testInExecution.getNameFromModel());

        /*this.setLocation(50,50);
        this.pack();
        */
        centerScreen();
    } // Fin du constructeur ManualExecution/0
    
    void centerScreen() {
		  Dimension dim = getToolkit().getScreenSize();
		  this.pack();
		  Rectangle abounds = getBounds();
		  setLocation((dim.width - abounds.width) / 2,
		      (dim.height - abounds.height) / 2);	 
	      //this.setVisible(true); 
		  requestFocus();
	}
    
    /**
     * M?thode qui initialise l'ex?cution avec la liste des tests pass?e en
     * param?tre
     * @param list une liste de tests
     */
    public void initData(ArrayList list, Execution execution, ExecutionResult executionResult) {
        
        
        execResult = executionResult;
        
        currentExecution = execution;
        int transNumber = -1;
        try  {
        transNumber = Api.beginTransaction(111, ApiConstants.LOADING);
        ProgressBar pProgressBar = new ProgressBar("Data Loading", "", list.size() +1);
        pProgressBar.show();
        if (list != null && list.size() > 0) {
            testsToBeExecuted = list;
            indexOfTextInExecution = 0;
            blockIndex = 0;
            testInExecution = (ManualTest)testsToBeExecuted.get(0);
            this.setTitle(title + ", test : " + testInExecution.getNameFromModel()); 
            //DataModel.setCurrentExecutionTestResult(execResult.getExecutionTestResultFromModel(testInExecution));
            execTestRes = (ManualExecutionResult) execResult.getExecutionTestResultFromModel(testInExecution); //Add FOR V2
            DataModel.setCurrentExecutionTestResult(execTestRes);
            testName.setText(Language.getInstance().getText("Nom_du_test__") + testInExecution.getNameFromModel());
            testDescription.setText(testInExecution.getDescriptionFromModel());
            ArrayList actionList = testInExecution.getActionListFromModel(false);
            pProgressBar.doTask("Load action");
            for (int i = 0; i < actionList.size(); i++) {
                Action action = (Action)actionList.get(i);
                ArrayList actionData = new ArrayList();
                actionData.add(action.getNameFromModel());
                actionData.add(action.getAttachmentMapFromModel().keySet());
                actionData.add(GregorianCalendar.getInstance().getTime().toString());
                /*actionData.add(Tools.getActionStatusIcon(execResult.getActionStatus(action)));
                if (execResult.getActionStatus(action) != null && !execResult.getActionStatus(action).equals("")) {
                    blockIndex = blockIndex + 1;
                }*/
                actionData.add(Tools.getActionStatusIcon(execTestRes.getActionStatusInModel(action)));
                if (execTestRes.getActionStatusInModel(action) != null && !execTestRes.getActionStatusInModel(action).equals("")) {
                    blockIndex = blockIndex + 1;
                }
                actionsTableModel.addRow(actionData);
            }
            for (int i = 0; i < list.size(); i++) {
            	pProgressBar.doTask("Load action");
            	ManualTest test = (ManualTest)list.get(i);
                //ADD FOR V2
                ManualExecutionResult pExecTestRes = (ManualExecutionResult) execResult.getExecutionTestResultFromModel(test);
                ArrayList actionList2 = test.getActionListFromModel(false);
                for (int j = 0; j < actionList2.size(); j++) {
                    Action action = (Action)actionList2.get(j);
                    //pExecTestRes.addDescriptionResultInModel(action, Tools.getInstantiedDescription(action.getDescriptionFromModel(), currentExecution.getDataSetFromModel()));
                    //pExecTestRes.addAwaitedResultInModel(action, Tools.getInstantiedDescription(action.getAwaitedResultFromModel(), currentExecution.getDataSetFromModel()));
                    pExecTestRes.addDescriptionResultInModel(action, Tools.getInstantiedDescription(action.getDescriptionFromModel(), currentExecution));
                    pExecTestRes.addAwaitedResultInModel(action, Tools.getInstantiedDescription(action.getAwaitedResultFromModel(), currentExecution));
                }
            }
        } 
        Api.commitTrans(transNumber);
        transNumber = -1;
        pProgressBar.finishAllTask();
        }catch (Exception e){
        	Api.forceRollBackTrans(transNumber);
        }
    } // Fin de la m?thode initData/1
    
    
    void setFirstTestWithNoResult(){
    	if (testsToBeExecuted.size() > 1) {
    		testInExecution = (ManualTest)testsToBeExecuted.get(indexOfTextInExecution);
    		ExecutionTestResult pExecutionTestResult = execResult.getExecutionTestResultFromModel(testInExecution);
    		String status = pExecutionTestResult.getStatusFromModel();
    		while ((indexOfTextInExecution + 1) < testsToBeExecuted.size() && status!=null && !status.equals("")) {
    			changeTest();
    			testInExecution = (ManualTest)testsToBeExecuted.get(indexOfTextInExecution);
        		pExecutionTestResult = execResult.getExecutionTestResultFromModel(testInExecution);
        		status = pExecutionTestResult.getStatusFromModel();
    		}
    	}
    }
    /**
     * M?thode qui permet de passer au test suivant.
     */
    public void changeTest() {
        if (testsToBeExecuted.size() > 1) {
            prevTestButton.setEnabled(true);
        }
        if ((indexOfTextInExecution + 1) == (testsToBeExecuted.size() - 1)) {
            
            nextTestButton.setEnabled(false);
        }
        testStatusCalculation();
        if ((indexOfTextInExecution + 1) < testsToBeExecuted.size()) {
            
            indexOfTextInExecution++;
            testInExecution = (ManualTest)testsToBeExecuted.get(indexOfTextInExecution);
            this.setTitle(title + ", test : " + testInExecution.getNameFromModel()); 
            
            DataModel.setCurrentExecutionTestResult(execResult.getExecutionTestResultFromModel(testInExecution));
            //NEW FOR V2
            execTestRes = (ManualExecutionResult) execResult.getExecutionTestResultFromModel(testInExecution);
            
            testName.setText(Language.getInstance().getText("Nom_du_test__") + testInExecution.getNameFromModel());
            testDescription.setText(testInExecution.getDescriptionFromModel());
            actionsTableModel.clearTable();
            ArrayList actionList = testInExecution.getActionListFromModel(false);
            if (actionList != null && actionList.size() > 0){
            	for (int i = 0; i < actionList.size(); i++) {
            		Action action = (Action)actionList.get(i);
            		ArrayList actionData = new ArrayList();
            		actionData.add(action.getNameFromModel());
            		actionData.add(action.getAttachmentMapFromModel().keySet());
            		actionData.add(GregorianCalendar.getInstance().getTime().toString());
            		//putIconStatusInTable(actionData, execResult.getActionStatus(action), i);
            		putIconStatusInTable(actionData, execTestRes.getActionStatusInModel(action), i);
            		actionsTableModel.addRow(actionData);
            	}
            } else {
            	/* clear INFO */
            	actionDescription.setText("");
                actionDescription.getCaret().setDot(0);
                awaitedResult.setText("");
                awaitedResult.getCaret().setDot(0);
                //effectiveResult.setText(execResult.getEffectivResult(action));
                effectiveResult.setText("");
                effectiveResult.getCaret().setDot(0);
            }
            ListSelectionModel lsm = actionsTable.getSelectionModel();
            lsm.setSelectionInterval(0, 0);
            
        } else if ((indexOfTextInExecution + 1) == testsToBeExecuted.size()){
            
        }
        
    } // Fin de la m?thode changeTest/0
    
    
    /**
     * M?thode qui permet de passer au test pr?c?dant.
     */
    public void changeTestBack() {
        if (testsToBeExecuted.size() > 1) {
            nextTestButton.setEnabled(true);
        }
        if ((indexOfTextInExecution - 1) == 0) {
            prevTestButton.setEnabled(false);
        }
        testStatusCalculation();
        if ((indexOfTextInExecution - 1) > -1) {
            indexOfTextInExecution--;
            testInExecution = (ManualTest)testsToBeExecuted.get(indexOfTextInExecution);
            DataModel.setCurrentExecutionTestResult(execResult.getExecutionTestResultFromModel(testInExecution));
            this.setTitle(title + ", test : " + testInExecution.getNameFromModel()); 
            //NEW FOR V2
            execTestRes = (ManualExecutionResult) execResult.getExecutionTestResultFromModel(testInExecution);
            testName.setText(Language.getInstance().getText("Nom_du_test__") + testInExecution.getNameFromModel());
            testDescription.setText(testInExecution.getDescriptionFromModel());
            actionsTableModel.clearTable();
            ArrayList actionList = testInExecution.getActionListFromModel(false);
            for (int i = 0; i < actionList.size(); i++) {
                Action action = (Action)actionList.get(i);
                ArrayList actionData = new ArrayList();
                actionData.add(action.getNameFromModel());
                actionData.add(action.getAttachmentMapFromModel().keySet());
                actionData.add(GregorianCalendar.getInstance().getTime().toString());
                //putIconStatusInTable(actionData, execResult.getActionStatus(action), i);
                putIconStatusInTable(actionData, execTestRes.getActionStatusInModel(action), i);
                actionsTableModel.addRow(actionData);
            }
            ListSelectionModel lsm = actionsTable.getSelectionModel();
            lsm.setSelectionInterval(0, 0);
            
        }
        
    } // Fin de la m?thode changeTestBack/0
    
    /**
     * M?thode qui retourne le r?sultat de l'ex?cution
     * @return le r?sultat de l'ex?cution
     */
    public ExecutionResult getExecutionResult() {
        return execResult;
    } // Fin de la m?thode getExecutionResult/0
    
    /**
     *
     * @param data
     * @param type
     * @param index
     */
    public void putIconStatusInTable(ArrayList data, String type, int index) {
        ImageIcon icon = getActionStatusIcon(type, index);
        if (icon != null) {
            data.add(icon);
        } else {
            data.add("");
        }
    } // Fin de la m?thode putIconStatusInTable/2
    
    
    /**
     * Retourne l'icone associ?e au type pass? au param?tre. On retourne <code>null</code>
     * si le type n'est pas SUCCESS, FAIL ou UNKNOW.
     * @param type
     * @param index
     * @return
     */
    public ImageIcon getActionStatusIcon(String type, int index) {
        if (type.equals(SUCCESS)) {
        } else if (type.equals(FAIL)) {
            blockIndex = index;
        } else if (type.equals(UNKNOWN)) {
            blockIndex = index;
        }
        return Tools.getActionStatusIcon(type);
    } // Fin de la m?thode getActionStatusIcon/1
    
    
    /**
     *
     *
     */
    public void testStatusCalculation() {
        int successAction = 0;
        ArrayList actionList = testInExecution.getActionListFromModel(false);
        for (int i = 0; i < actionList.size(); i++) {
            /*if (execResult.getActionStatus(((Action)actionList.get(i))).equals(FAIL)) {
                execResult.addTestResultStatusInModel(testInExecution, FAIL);
                return;
            } else if (execResult.getActionStatus(((Action)actionList.get(i))).equals(UNKNOWN)) {
                execResult.addTestResultStatusInModel(testInExecution, UNKNOWN);
                return;
            } else if (execResult.getActionStatus(((Action)actionList.get(i))).equals(SUCCESS)) {
                successAction++;
            }*/
        	if (execTestRes.getActionStatusInModel(((Action)actionList.get(i))).equals(FAIL)) {
                execResult.addTestResultStatusInModel(testInExecution, FAIL);
                return;
            } else if (execTestRes.getActionStatusInModel(((Action)actionList.get(i))).equals(UNKNOWN)) {
                execResult.addTestResultStatusInModel(testInExecution, UNKNOWN);
                return;
            } else if (execTestRes.getActionStatusInModel(((Action)actionList.get(i))).equals(SUCCESS)) {
                successAction++;
            }
            
        }
        if (successAction == actionList.size()) {
            execResult.addTestResultStatusInModel(testInExecution, SUCCESS);
            return;
        }
        execResult.addTestResultStatusInModel(testInExecution, "");
    } // Fin de la m?thode testStatusCalculation/0
    
    /**
     * Calcule l'index de bloquage de la s?lection.
     */
    private void blockIndexCalculation(int selectedRowIndex) {
        ArrayList actionList = testInExecution.getActionListFromModel(false);
        blockIndex = selectedRowIndex + 1;
        
        /*while (blockIndex < actionList.size() && !execResult.getActionStatus(((Action)actionList.get(blockIndex))).equals("")) {
            blockIndex++;
        }*/
        while (blockIndex < actionList.size() && !execTestRes.getActionStatusInModel(((Action)actionList.get(blockIndex))).equals("")) {
            blockIndex++;
        }
        
    } // Fin de la m?thode blockIndexCalculation/0
    
    
    /**
     * Retourne la liste des attachements ? supprimer
     * @return
     */
    public HashMap getOldAttachMap() {
        return oldMap;
    } // Fin de la m?thode getOldAttachMap/0
    
    
    /**
     *
     * @author teaml039
     */
    public class EffectivResultListener implements CaretListener {
        
                /* (non-Javadoc)
                 * @see javax.swing.event.CaretListener#caretUpdate(javax.swing.event.CaretEvent)
                 */
        public void caretUpdate(CaretEvent e) {
            int selectedRowIndex = actionsTable.getSelectedRow();
            /*if (selectedRowIndex != -1) {
                execResult.addEffectivResult(testInExecution.getActionFromModel((String)actionsTableModel.getValueAt(selectedRowIndex, 0)), ((JTextPane)e.getSource()).getText());
            }*/
            if (selectedRowIndex != -1) {
            	execTestRes.addEffectivResultInModel(testInExecution.getActionFromModel((String)actionsTableModel.getValueAt(selectedRowIndex, 0)), ((JTextPane)e.getSource()).getText());
            }
        }
    } // Fin de la classe EffectivResultListener
    
    public void lauchManualExecution() {
        t = new Thread(this);
        stop_exec = false;
        t.start();
    }
    
    public void run() {
        this.setVisible(true);
    }
    
    public boolean isStoped() {
        return stop_exec;
    }
    
    public boolean isFinished() {
        return finished;
    }
    
    public void setFinished(boolean b) {
        finished = b;
    }
    
    /************************** ScreenShot *********************/
    
    JDialog takeScreenShoot;
    boolean screenShootValided = false;
	
	public void screeshootFrame(java.awt.event.ActionEvent e){ 
		screenShootValided = false;
		takeScreenShoot = new JDialog();
		takeScreenShoot.setResizable(false);
		takeScreenShoot.setTitle("Capture Screen");
		JButton screeshootButton2 = new JButton();
		ImageIcon icon = Tools.createAppletImageIcon(PATH_TO_SCREENSHOT_ICON,"");
		screeshootButton2.setIcon(icon);
		screeshootButton2.setMinimumSize(new Dimension(icon.getIconWidth(),icon.getIconHeight()));
		//takeScreenShoot.setSize(new Dimension(icon.getIconWidth(),icon.getIconHeight()));
		screeshootButton2.addActionListener(new ActionListener () {
			public void actionPerformed(java.awt.event.ActionEvent e){
				screenShootValided = true;
				takeScreenShoot.dispose();
				screeshootPerformed(DataModel.getCurrentExecutionTestResult());
				//SalomeTMFContext.getInstance().getSalomeFrame().setVisible(true);	
				setVisible(true);
			}
		});
		
		takeScreenShoot.addWindowListener(new WindowListener () {
			public void windowActivated(WindowEvent e) {}
			public void windowClosed(WindowEvent e) {
				
			}
			public void windowClosing(WindowEvent e) {
				if (!screenShootValided){
					screenShootValided = true;
					takeScreenShoot.dispose();
					setVisible(true);
				}
			}
			
			public void windowDeactivated(WindowEvent e) {
				takeScreenShoot.toFront();
			}
			public void windowDeiconified(WindowEvent e) {}
			public void windowIconified(WindowEvent e) {
				takeScreenShoot.toFront();
			}
			public void windowOpened(WindowEvent e) {}
			
		});
		//SalomeTMFContext.getInstance().getSalomeFrame().setVisible(false);
		setVisible(false);
		takeScreenShoot.getContentPane().add(screeshootButton2);
		takeScreenShoot.pack();
		takeScreenShoot.setVisible(true);
		
	}
	
	public void screeshootPerformed(WithAttachment withAttachment){
		if (withAttachment == null){
			return;
		}
		try{
			FileAttachment fileAttachment = withAttachment.takeScreenShot();
			withAttachment.addAttachementInModel(fileAttachment);
			
		} catch (Exception ex1) {
			JOptionPane.showMessageDialog(ManualExecution.this,
					Language.getInstance().getText("Le_fichier_") + "Screenshot " +Language.getInstance().getText("_est_deja_attache_a_") +  withAttachment.getNameFromModel() +" !",
					Language.getInstance().getText("Erreur_"),
					JOptionPane.ERROR_MESSAGE);
		}
	}
} // Fin de la classe ManualExecution
