package org.objectweb.salome_tmf.server;

import java.io.IOException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.NoSuchAlgorithmException;
import java.util.Hashtable;
import java.util.Properties;

import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.axis.encoding.Base64;
import org.objectweb.salome_tmf.api.Api;
import org.objectweb.salome_tmf.api.MD5paswd;
import org.objectweb.salome_tmf.api.Permission;
import org.objectweb.salome_tmf.api.Util;
import org.objectweb.salome_tmf.api.data.UserWrapper;
import org.objectweb.salome_tmf.api.sql.IDataBase;
import org.objectweb.salome_tmf.api.sql.ISQLObjectFactory;
import org.objectweb.salome_tmf.databaseSQL.DataBase;
import org.objectweb.salome_tmf.databaseSQL.SQLEngine;

import com.opensymphony.oscache.base.NeedsRefreshException;
import com.opensymphony.oscache.general.GeneralCacheAdministrator;

//import salomeTMF_plug.requirements.sqlWrapper.SQLWrapper;

/**
 * Servlet implementation class for Servlet: ApiFilter
 *
 */
public class ApiFilter implements javax.servlet.Filter {
	
	final static String PLUGIN_REQUIREMENTS = "/SQLRequirement";
	final static String OSCACHE_CONFIG_FILE_PATH = "/oscache.properties";
	// Nom des donnees en cache
//	final static String CACHE_LOGIN = "SalomeTMF_login";
//	final static String CACHE_PASSWORD = "SalomeTMF_paswword";
	
	static GeneralCacheAdministrator cache = null;
	static int cacheDelai = 1800; // 30 minutes par defaut
	static int codeErreur = HttpServletResponse.SC_FORBIDDEN;
	
	private static IDataBase db = null;
	private static Hashtable<String, TransactionClient> transactionTable = new Hashtable<String, TransactionClient>();
	private static String clientId = null;
	private static Hashtable<String, Integer> idTransTable = new Hashtable<String, Integer>();
	private static TransactionThread transactionThread;

	public void init(FilterConfig arg0) throws ServletException {
		// initialisation
		Util.log("[ServletFilter] Init");
		
		// Lancement du timeout sur les transactions et sur le pool de connexion
		transactionThread = new TransactionThread();
		transactionThread.start();
		try {
			String maximumActiveTime = ServerConstants.getMaximumActiveTime();
			DataBase.setProxoolMaximumActiveTime(maximumActiveTime);
		} catch (Exception e1) {
		}

		// Initialisation du systeme de cache OSCache
		cache = new GeneralCacheAdministrator();
		Properties propDelai;
		try {
			propDelai = Util.getPropertiesFile(OSCACHE_CONFIG_FILE_PATH);
			String strDelai = propDelai.getProperty("cache.time");
			cacheDelai = Integer.parseInt(strDelai);
		} catch (Exception e) {}
		Util.log("[Server cache] time : " + cacheDelai + " ms");
		
		// determination de l'url pour l'ouverture de connexion
		String fileSeparator = System.getProperties().getProperty("file.separator");
		String protocol = "file:///";
		if(!fileSeparator.equals("\\")) {
			protocol += "/";
		}
		String url = protocol + arg0.getServletContext().getRealPath("/");
		
		// ouverture de la connexion pour initialisation des donnees
		try {
			/* L'ouverture de connextion par la fonction Api.openConnection()
			 * permet, entre autre, de charger le fichier de propriete
			 * 
			 * La methode Api.closeConnection() mets l'objet pISQLEngine a null
			 * donc la connexion est fermee par closeConnection() qui conserve
			 * la reference sur cette objet.
			 */
			Api.openConnection(new URL(url));
			
			// Initialisation du plugin Requirements
			try {
				// Pour pouvoir deployer la version avec ou sans plugin
				Class classSQLWrapper = Class.forName("salomeTMF_plug.requirements.sqlWrapper.SQLWrapper");
	        	Method methodInit = classSQLWrapper.getMethod("init", new Class[] {ISQLObjectFactory.class});
	        	methodInit.invoke(null ,new Object[] {Api.getISQLObjectFactory()});
			} catch (Exception e) {}

			closeConnection();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
	}

	public void destroy() {		
		Util.log("[ServletFilter] Destroy");
		// suppression du cache
		cache.destroy();
		
		// arret du timeout
		transactionThread.finish();

		Api.closeConnection();
	}

	public void doFilter(ServletRequest req, ServletResponse rep, FilterChain chain) throws IOException, ServletException {
		// avant les requetes
		Permission.setPermission(254);
		
		// Identifiant du client
		clientId = getUsername(req) + "_" + req.getRemoteAddr();
		
		//ouverture de la connexion
		openConnection();
		if(isValideUser(req)) {
			try {
				chain.doFilter(req, rep);
			} catch (Throwable t) {
				// apres avec exception
				throw new ServletException(t);
			} finally {
				// apres a tous les coups.
				// fermeture de la connexion
				closeConnection();
			}
		}
		else {
			((HttpServletResponse) rep).sendError(codeErreur);
			closeConnection();
		}
	}
	
	private void openConnection() {
		if(Api.getISQLObjectFactory() != null) {
			try {
				db = Api.getISQLObjectFactory().getInstanceOfSalomeDataBase();
				if(db != null) {
					
					// test si la demande est dans une transaction
					if(transactionTable.get(clientId) != null && transactionTable.get(clientId).getConnection() != null) {
						
						if(transactionTable.get(clientId).getConnection().isClosed()) {
							
							Util.log("[ServletFilter] openPoolConnection");
							//on ouvre une nouvelle connexion
							db.openPoolConnection(Api.getUrl(), Api.getUser(), Api.getPassword());
							
							// on affecte cette connexion au client
							transactionTable.get(clientId).setConnection(db.getCnt());
						}
						else {
							
							// on utilise la connexion deja ouverte
							Util.log("[ServletFilter] continue transaction " + clientId + " - " + transactionTable.get(clientId).getConnection());
							
							// init transDB
							SQLEngine.setIdTrans(1);
							
							db.setCnt(transactionTable.get(clientId).getConnection());
						}
					}
					else {
						// on est pas dans une transaction
						Util.log("[ServletFilter] openPoolConnection");
						
						// init transDB
						SQLEngine.setIdTrans(-1);
						
						//on ouvre une nouvelle connexion
						db.openPoolConnection(Api.getUrl(), Api.getUser(), Api.getPassword());
						
						// on affecte cette connexion au client
						transactionTable.put(clientId, new TransactionClient(clientId, db.getCnt()));

					}
					
					// affectation du numero de transaction du client
					if(idTransTable.get(clientId) == null) {
						SQLEngine.setIdTrans(-1);
					}
					else {
						SQLEngine.setIdTrans((int)idTransTable.get(clientId));
					}
					
					Util.log("[ServletFilter] [DataBase] " + db + ", cnt : " + db.getCnt() + ", clientId : " + clientId);
					
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	private void closeConnection() {
		if(db != null) {
			try {
				
				idTransTable.put(clientId, new Integer(SQLEngine.getIdTrans()));
				
				// test si la demande est dans une transaction
				if(transactionTable.get(clientId) != null) {
					
					System.out.println(clientId + " - transID : " + SQLEngine.getIdTrans());
					
					if(AxisAuthorizationHandler.OPERATION.equals("commitTransDB") || AxisAuthorizationHandler.OPERATION.equals("rollForceBackTransDB")) {
						System.out.println("DEMANDE COMMIT : " + SQLEngine.tmpIdTrans);
						
						// test si il y a un commit, (different de -1 pour les commit imbriques)
						if(SQLEngine.tmpIdTrans != -1) {
							// Un commit a ete realise, il faut fermer la connexion
							closeClientConnection(clientId);
						}
					}
				}
				else {
					db.close();
					// suppression du numero de transaction affectee au client
					idTransTable.remove(clientId);
					Util.log("[ServletFilter] closeConnection");
				}
				
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	private String getUsername(ServletRequest req) {
		String username = null;
		HttpServletRequest request = (HttpServletRequest) req;
		String auth = request.getHeader("Authorization");
		if(auth != null) {
			auth = auth.substring(auth.indexOf(" "));
			// Decodage
			String decoded = new String(Base64.decode(auth));
			// decoded contient username:password.
			int i = decoded.indexOf(":");
			username = decoded.substring(0,i);
		}
		
		return username;
	}

	private boolean isValideUser(ServletRequest req) {
		boolean isValide = false;
		// recuperation de l'en-tete d'authentification
		HttpServletRequest request = (HttpServletRequest) req;
		String auth = request.getHeader("Authorization");
		if(auth != null) {
			auth = auth.substring(auth.indexOf(" "));
			// Decodage
			String decoded = new String(Base64.decode(auth));
			// decoded contient username:password.
			int i = decoded.indexOf(":");
			String username = decoded.substring(0,i);
			String pwd = decoded.substring(i+1,decoded.length());
			try {
				if(username.equals(Api.DEFAULT_LOGIN) && pwd.equals(MD5paswd.getEncodedPassword(Api.DEFAULT_PASSWORD))) {
					// Cas de l'utilisateur par defaut.
					return true;
				}
			} catch (NoSuchAlgorithmException e) {}
			
//			String strCacheLogin = null;
			String strCachePassword = null;
			try {
				// On recupere les donnees en cache
//				strCacheLogin = (String)cache.getFromCache(CACHE_LOGIN, cacheDelai);
				strCachePassword = (String)cache.getFromCache(username, cacheDelai);
			} catch (NeedsRefreshException e1) {
				// Pas de donnees en cache ou donnees ayant depassees le delai de validite => authentification par la base de donnees
				isValide = isValideUserFromDB(username, pwd);
			}
			
			if(!isValide) {
				if(strCachePassword == null || !strCachePassword.equals(pwd)) {
					// Authentification par la base de donnees
					isValide = isValideUserFromDB(username, pwd);
				}
				else {
					// Authentification par le cache
					isValide = pwd.equals(strCachePassword);
				}
			}
		}
		else {
			codeErreur = HttpServletResponse.SC_UNAUTHORIZED;
		}
		
		return isValide;
	}
	
	private boolean isValideUserFromDB(String login, String MD5Password) {
		boolean isValide = false;
		
		UserWrapper user = null;
		try {
			user = Api.getISQLObjectFactory().getISQLPersonne().getUserByLogin(login);
			if(user != null) {
				isValide = MD5paswd.testPassword(MD5Password, user.getPassword());
				
				// mise en cache
				if(isValide) {
//					cache.putInCache(CACHE_LOGIN, login);
					cache.putInCache(login, MD5Password);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return isValide;
	}
	
	public static void closeClientConnection(String clientId) throws Exception {
		if(transactionTable.get(clientId) != null) {
			db.setCnt(transactionTable.get(clientId).getConnection());
			db.close();
			Util.log("[ServletFilter] closeConnection from transaction");
			
			// Suppression de la connexion affectee au client
			transactionTable.remove(clientId);
			
			// suppression du numero de transaction affectee au client
			idTransTable.remove(clientId);
		}
	}
	
	public static Hashtable<String, TransactionClient> getTransactionClient() {
		return transactionTable;
	}

	public static IDataBase getDb() {
		return db;
	}
	
	public static void rollback(String clientId) throws Exception {
		if(transactionTable.get(clientId) != null) {
			if(transactionTable.get(clientId).getConnection() != null) {
				db.setCnt(transactionTable.get(clientId).getConnection());
				db.rollback();
			}
		}
	}
}