package org.objectweb.salome_tmf.server;

import javax.servlet.http.HttpServletRequest;

import org.apache.axis.AxisFault;
import org.apache.axis.MessageContext;
import org.apache.axis.encoding.Base64;
import org.apache.axis.handlers.BasicHandler;
import org.apache.axis.transport.http.HTTPConstants;
import org.apache.axis.utils.Messages;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

@SuppressWarnings("serial")
public class AxisAuthorizationHandler extends BasicHandler {

	protected static Log log =
		LogFactory.getLog(AxisAuthorizationHandler.class);



	private final static String LOGIN = "LoginSalome";
	private final static String[] SERVICE_OPERATIONS = { 
		"SQLConfig_getSalomeConf", 
		"SQLProject_getAllProjects", 
		"SQLProject_getUsersOfProject", 
		"SQLProject_getAdminsOfProject",
		"SQLPersonne_getUserByLogin",
		"SQLConfig_updateSalomeConf"
	};

	public static String SERVICE = null;
	public static String OPERATION = null;
	
	public void invoke(MessageContext context) throws AxisFault {

		SERVICE = null;
		OPERATION = null;
		
		if (log.isDebugEnabled()) {
			log.debug("Enter: AxisAuthorizationHandler::invoke");
		}

		String service = context.getTargetService();
		String operation = null;
		
		if(context.getOperation() != null) {
			operation = context.getOperation().getName();
		}
		
		// affectation des valeurs d'appel
		SERVICE = service;
		OPERATION = operation;
		
		if(context != null && service != null && operation != null) {

			HttpServletRequest req = (HttpServletRequest) context.getProperty(HTTPConstants.MC_HTTP_SERVLETREQUEST);
			String auth = req.getHeader("Authorization");
			// auth is not null otherwise authent filter not deployed.
			if ( auth == null ) {
				throw new AxisFault( "Server.Unauthenticated",
						Messages.getMessage("cantAuth00", ""),
						null, null );
			}
			auth = auth.substring(auth.indexOf(" "));
			// Decodage
			String decoded = new String(Base64.decode(auth));
			// decoded contient username:password.
			int index = decoded.indexOf(":");
			String username = decoded.substring(0, index);

			if (username.equals(LOGIN) ) {
				boolean valid = false;
				String servop = service + "_" + operation;

				for(int i = 0; i < SERVICE_OPERATIONS.length; i++) {
					String so = SERVICE_OPERATIONS[i];
					if ( servop.equals(so) ) {
						valid = true;
						break;
					}
				}
				if ( ! valid ) {
					throw new AxisFault( "Server.Unauthorized",
							Messages.getMessage("notAuth00", username, servop),
							null, null );
				}
			}
		}
	}
}
