/*
* SalomeTMF is a Test Management Framework
* Copyright (C) 2005 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* @author Marche Mikael
*
* Contact: mikael.marche@rd.francetelecom.com
*/
package salomeTMF_plug.requirements.ihm;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.FontMetrics;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.objectweb.salome_tmf.data.Test;
import org.objectweb.salome_tmf.ihm.languages.Language;
import org.objectweb.salome_tmf.ihm.main.datawrapper.DataModel;
import org.objectweb.salome_tmf.ihm.models.TableSorter;

import salomeTMF_plug.requirements.data.ReqFilter;
import salomeTMF_plug.requirements.data.Requirement;
import salomeTMF_plug.requirements.sqlWrapper.HistoryWrapper;

public class HistoryPanel  extends JPanel implements ListSelectionListener , ActionListener{
	JTable historyTable;
	TableSorter  sorter;
	SimpleTableModel model;
	Requirement currentReq;
	
	JButton viewButton;
	HistoryWrapper[] tabHisto;
	
	Hashtable<Integer,String>  itemComplex;
	Hashtable<Integer,String>  itemPriority;
	Hashtable<Integer,String>  itemCat;
	Hashtable<Integer,String>  itemState;
	
	public HistoryPanel(){
		super(new BorderLayout());
		initData();
		initComponent();
	}
	
	
	void initData() {
		itemPriority = new Hashtable<Integer,String>();
		itemPriority.put(ReqFilter.P_HIGHT,  Language.getInstance().getText("Filtre_Exigence_Haute"));
		itemPriority.put(ReqFilter.P_MEDIUM, Language.getInstance().getText("Filtre_Exigence_Moyenne"));
		itemPriority.put(ReqFilter.P_LOW, Language.getInstance().getText("Filtre_Exigence_Faible"));
		itemPriority.put(ReqFilter.P_NONE, Language.getInstance().getText("Filtre_Exigence_Option"));
		
		
		itemCat = new Hashtable<Integer,String>();
		itemCat.put(0, Language.getInstance().getText("Cat_Fonctionnel"));
		itemCat.put(1, Language.getInstance().getText("Cat_Interoperabilite"));
		itemCat.put(2, Language.getInstance().getText("Cat_Charge"));
		itemCat.put(3, Language.getInstance().getText("Cat_Performance"));
		itemCat.put(4, Language.getInstance().getText("Cat_Disponibilite"));
		itemCat.put(5, Language.getInstance().getText("Cat_Securite"));
		itemCat.put(6, Language.getInstance().getText("Cat_Exploitabilite"));
		itemCat.put(7, Language.getInstance().getText("Cat_Autre"));

		itemState = new Hashtable<Integer,String>();
		itemState.put(0, Language.getInstance().getText("State_A_Analyser"));
		itemState.put(1, Language.getInstance().getText("State_Analysee"));
		itemState.put(2, Language.getInstance().getText("State_Approuvee"));
		itemState.put(3, Language.getInstance().getText("State_Verifiee"));
		itemState.put(4, Language.getInstance().getText("State_Finalisee"));
		itemState.put(5, Language.getInstance().getText("State_Reportee"));
		itemState.put(6, Language.getInstance().getText("State_Abandonnee"));
		
		itemComplex = new Hashtable<Integer,String>();
		itemComplex.put(ReqFilter.C0, Language.getInstance().getText("Com_C0"));
		itemComplex.put(ReqFilter.C1, Language.getInstance().getText("Com_C1"));
		itemComplex.put(ReqFilter.C2, Language.getInstance().getText("Com_C2"));
		itemComplex.put(ReqFilter.C3, Language.getInstance().getText("Com_C3"));
	}
	
	void initComponent(){
		int t_y = 768; 
		int t_x = 1024;
		try {
	        	GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
	        	GraphicsDevice[] gs = ge.getScreenDevices();
	        	GraphicsDevice gd = gs[0];
	        	GraphicsConfiguration[] gc = gd.getConfigurations();
	        	Rectangle r = gc[0].getBounds();
	        	t_y = r.height;   ;
	        } catch(Exception E){
	        	
	        }
		model = new SimpleTableModel();
		model.addColumn(Language.getInstance().getText("Date"));
		model.addColumn(Language.getInstance().getText("Action"));
		model.addColumn(Language.getInstance().getText("Nouvelle_valeur"));
		sorter = new TableSorter(model);
		
	    
		historyTable = new JTable(sorter);
		
		historyTable.addMouseListener(new MouseListener(){
			public void mouseClicked(MouseEvent me){
				if (me.getButton() == MouseEvent.BUTTON1){
					if (me.getClickCount() == 2){
						viewDescriptionPerformed();
					}
				}
			}
		 
			public void mouseEntered(MouseEvent me){}
			public void mouseExited(MouseEvent me){}
			public void mousePressed(MouseEvent me){}
			public void mouseReleased(MouseEvent me){}
		});
		
	    ListSelectionModel rowSM = historyTable.getSelectionModel();
		rowSM.addListSelectionListener(this);
		setColumnSize(historyTable);
		sorter.setTableHeader(historyTable.getTableHeader());
		
		historyTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		JScrollPane tablePane = new JScrollPane(historyTable, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		tablePane.setBorder(BorderFactory.createRaisedBevelBorder());
		setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK),Language.getInstance().getText("Historique")));
		add(tablePane, BorderLayout.CENTER);
		

		JPanel panelButton = new JPanel(new FlowLayout());
		viewButton = new JButton(Language.getInstance().getText("Visualiser") + " " +Language.getInstance().getText("Test"));
		viewButton.setEnabled(false);
		viewButton.addActionListener(this);
		panelButton.add(viewButton);
		add(panelButton, BorderLayout.SOUTH);
		
		setMaximumSize(new Dimension(t_x/3*2,t_y/5)); //MM
		
	}
	
	public void updateData(Requirement pReq, boolean force){
		if (pReq == null){
			return;
		} 
		if (currentReq != null && !force){
			if (currentReq.getIdBdd() == pReq.getIdBdd()){
				return;
			}
		}
		boolean isSorting  = sorter.isSorting();
		int statusSort_col0 = -1;
		int statusSort_col1 = -1;
		int statusSort_col2 = -1;
		if (isSorting){
			statusSort_col0 = sorter.getSortingStatus(0);
			statusSort_col1 = sorter.getSortingStatus(1);
			statusSort_col2 = sorter.getSortingStatus(2);
		}
		model.clearTable();
		//System.out.println("UpDateHistory for  " + pReq.getLongName());
		
		try {
		tabHisto = pReq.getHistory();
			for (int i = 0 ; i < tabHisto.length ; i++){
				HistoryWrapper pHistoryWrapper = tabHisto[i];
				Date pDate = pHistoryWrapper.getDate();
				int code = pHistoryWrapper.getCode();
				String strcode = Requirement.getStringCodeValue(code);
				String valeur = pHistoryWrapper.getValeur();
				if (Requirement.isTestIdValeur(code)){
					Test pTest = (Test) DataModel.getCurrentProject().getTestFromModel(new Integer(valeur).intValue());
					if (pTest != null){
						valeur = pTest.getNameFromModel();
					}
				} else if (Requirement.isStateValeur(code)){
					int state = new Integer(valeur).intValue();
					valeur =  itemState.get(state);
				} else if (Requirement.isPriorityValeur(code)){
					int priority = new Integer(valeur).intValue();
					valeur = itemPriority.get(priority);
				} else if (Requirement.isComplexityValeur(code)){
					int comp = new Integer(valeur).intValue();
					valeur =  itemComplex.get(comp);
				} else if (Requirement.isCategoryValeur(code)){
					int cat = new Integer(valeur).intValue();
					valeur = itemCat.get(cat);
				}
				
				Vector<String> data = new Vector<String>();
				SimpleDateFormat sdf = new SimpleDateFormat("dd-MMM-yyyy");
				data.add(sdf.format(pDate)); 
				data.add(strcode);
				data.add(valeur);
				model.insertRow(i,data);
			}
		} catch (Exception e){
			e.printStackTrace();
		}
		currentReq = pReq;
		setColumnSize(historyTable);
		if (isSorting){
			sorter.setSortingStatus(0, statusSort_col0);
			sorter.setSortingStatus(1, statusSort_col1);
			sorter.setSortingStatus(2, statusSort_col2);
		}
	}
	
	
	/**
	 * redefine size of table columns
	 */
	public void setColumnSize(JTable table){   
		FontMetrics fm = table.getFontMetrics(table.getFont());
		for (int i = 0 ; i < table.getColumnCount() ; i++) {
			int max = 0;
			for (int j = 0 ; j < table.getRowCount() ; j++) {
				int taille = fm.stringWidth((String)table.getValueAt(j,i));
				if (taille > max)
					max = taille;
			}
			String nom = (String)table.getColumnModel().getColumn(i).getIdentifier();
			int taille = fm.stringWidth(nom);
			if (taille > max){
				max = taille;
			}
			table.getColumnModel().getColumn(i).setPreferredWidth(max+10);
		}
	}
	
	/********************** ListSelectionListener  ***************************/
	public void valueChanged(ListSelectionEvent e) {
		if (e.getSource().equals(historyTable.getSelectionModel())){
			assigendTableValueChanged(e);
		}
	}
	
	void assigendTableValueChanged(ListSelectionEvent e) {
		
		if (e.getValueIsAdjusting())
			return;
		
		int selectedRow = historyTable.getSelectedRow();
		if (selectedRow != -1) {
			/* Mono Selection */
			int modeIndex = sorter.modelIndex(selectedRow);
			if (tabHisto != null){
				HistoryWrapper pHistoryWrapper = tabHisto[modeIndex];
				int code = pHistoryWrapper.getCode();
				String valeur = pHistoryWrapper.getValeur();
				if (Requirement.isTestIdValeur(code)){
					Test pTest = (Test) DataModel.getCurrentProject().getTestFromModel(new Integer(valeur).intValue());
					if (pTest != null){
						viewButton.setEnabled(true);
					} 
				}  else {
					viewButton.setEnabled(false);
				}
			} else {
				viewButton.setEnabled(false);
			}
			
			
			//viewButton.setEnabled(true);
		} else {
			/* Pas de Selection */
			viewButton.setEnabled(false);
		}
		
	}
/********************************** Implements ActionListener *********************************/
	
	public void actionPerformed(ActionEvent e){
		if (e.getSource().equals(viewButton)){
			viewTestPerformed();
		}
	}
	
	void doubleClicPerformed(){
		viewDescriptionPerformed();
	}
	
	void viewTestPerformed(){
		int selectedRow = historyTable.getSelectedRow();
		if (selectedRow != -1) {
			int modeIndex = sorter.modelIndex(selectedRow);
			if (tabHisto != null){
				HistoryWrapper pHistoryWrapper = tabHisto[modeIndex];
				int code = pHistoryWrapper.getCode();
				String valeur = pHistoryWrapper.getValeur();
				if (Requirement.isTestIdValeur(code)){
					Test pTest = (Test) DataModel.getCurrentProject().getTestFromModel(new Integer(valeur).intValue());
					if (pTest != null){
						DataModel.view(pTest);
					}
				}
			}
		}
	}
	
	void viewDescriptionPerformed(){
		int selectedRow = historyTable.getSelectedRow();
		if (selectedRow != -1) {
			int modeIndex = sorter.modelIndex(selectedRow);
			if (tabHisto != null){
				HistoryWrapper pHistoryWrapper = tabHisto[modeIndex];
				int code = pHistoryWrapper.getCode();
				if (Requirement.isDescriptionValeur(code)){
					String valeur = pHistoryWrapper.getValeur();
					System.out.println("Valeur = " + valeur);
					HistoryDescViewDialog pHistoryDescViewDialog = new HistoryDescViewDialog(valeur);
				}
			}
		}
	}
}
